"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const schematics_1 = require("@angular-devkit/schematics");
const core_1 = require("@angular-devkit/core");
const config_1 = require("../schematics-angular-utils/config");
const find_module_1 = require("../schematics-angular-utils/find-module");
const tasks_1 = require("@angular-devkit/schematics/tasks");
const ast_utils_1 = require("../schematics-angular-utils/ast-utils");
const ts = require("typescript");
const change_1 = require("../schematics-angular-utils/change");
// You don't have to export the function as default. You can also have more than one rule factory
// per file.
function hello(_options) {
    return (tree, _context) => {
        editPackageJson(tree);
        const installTaskId = _context.addTask(new tasks_1.NodePackageInstallTask());
        _context.addTask(new tasks_1.RunSchematicTask('@angular/material/schematics/collection.json', 'ng-add', { gestures: true, animations: true, theme: 'indigo-pink' }), [installTaskId]);
        let appTsConfig = "./tsconfig.app.json";
        if (!tree.exists(appTsConfig))
            appTsConfig = "./src/tsconfig.app.json";
        editJson(tree, appTsConfig, j => {
            j.exclude.push("./app/server**");
        });
        editGitIgnore(tree);
        let entryComponents = [{ name: 'YesNoQuestionComponent', path: './common/yes-no-question/yes-no-question.component' },
            { name: 'SignInComponent', path: './common/sign-in/sign-in.component' },
            { name: 'SelectPopupComponent', path: './common/select-popup/select-popup.component' },
            { name: 'InputAreaComponent', path: './common/input-area/input-area.component' }
        ];
        addToNgModule(tree, {
            declarations: [
                { name: 'UsersComponent', path: './users/users.component' },
                { name: 'UpdateInfoComponent', path: './users/update-info/update-info.component' },
                { name: 'RegisterComponent', path: './users/register/register.component' },
                { name: 'HomeComponent', path: './home/home.component' },
                ...entryComponents
            ],
            imports: [
                { name: 'FormsModule', path: '@angular/forms' },
                { name: 'MatSidenavModule', path: '@angular/material/sidenav' },
                { name: 'MatListModule', path: '@angular/material/list' },
                { name: 'MatToolbarModule', path: '@angular/material/toolbar' },
                { name: 'MatCheckboxModule', path: '@angular/material/checkbox' },
                { name: 'MatCardModule', path: '@angular/material/card' },
                { name: 'MatDialogModule', path: '@angular/material/dialog' },
                { name: 'MatSnackBarModule', path: '@angular/material/snack-bar' },
                { name: 'MatFormFieldModule', path: '@angular/material/form-field' },
                { name: 'MatInputModule', path: '@angular/material/input' },
                { name: 'MatButtonModule', path: '@angular/material/button' },
                { name: 'MatIconModule', path: '@angular/material/icon' },
                { name: 'RadWebModule', path: 'radweb' },
            ],
            providers: [
                { name: 'DialogService', path: './common/dialog' },
                { name: 'AdminGuard', path: './users/roles' }
            ],
            entryComponents: entryComponents
        });
        var project = 'did not find project';
        editJson(tree, "./angular.json", j => {
            for (const p in j.projects) {
                project = p;
                if (j.projects.hasOwnProperty(p)) {
                    const element = j.projects[p];
                    element.architect.serve.options.proxyConfig = "proxy.conf.json";
                    element.architect.build.options.styles.push("./node_modules/radweb/input-styles.scss");
                    return;
                }
            }
        });
        tree.delete('./src/app/app.component.html');
        tree.delete('./src/app/app.component.spec.ts');
        tree.delete('./src/app/app.component.ts');
        tree.delete('./src/app/app.component.scss');
        tree.delete('./src/app/app-routing.module.ts');
        //    addDeclarationToModule()
        const sourceTemplates = schematics_1.url('./files');
        const sourceParametrizedTemplates = schematics_1.apply(sourceTemplates, [
            schematics_1.template(Object.assign({}, _options, core_1.strings, { project }, {
                postgresUserName: 'postgres',
                postgresPassword: 'MASTERKEY',
                tokenSignKey: 'secret-key-replace-me-soon'
            }))
        ]);
        return schematics_1.mergeWith(sourceParametrizedTemplates);
    };
    function editPackageJson(tree) {
        editJson(tree, './package.json', json => {
            json.scripts["ng-serve"] = json.scripts.start;
            json.scripts["node-serve"] = "./node_modules/.bin/tsc-watch -p tsconfig.server.json --onSuccess \"npm run server:dev-run";
            json.scripts["server:dev-run"] = "node --inspect dist-server/server/server.js";
            json.scripts["ng:build"] = "ng build --prod";
            json.scripts["build"] = "npm run server:build && npm run ng:build";
            json.scripts.start = "node dist-server/server/server.js";
            json.scripts["server:build-watch"] = "tsc -p tsconfig.server.json --watch";
            json.scripts["server:build"] = "tsc -p tsconfig.server.json";
            json.scripts["server:debug"] = "node --inspect --debug-brk dist-server/server/server.js";
            json.dependencies["dotenv"] = "^8.1.0";
            json.dependencies["password-hash"] = "^1.2.2";
            json.dependencies["radweb"] = "^3.0.11";
            json.dependencies["radweb-server"] = "^3.0.5";
            json.dependencies["radweb-server-postgres"] = "^3.0.2";
            json.dependencies["@angular/material"] = "^7.3.4";
            json.dependencies["pg"] = "^7.6.1";
            json.dependencies["express-force-https"] = "^1.0.0";
            json.dependencies["jsonwebtoken"] = "^8.5.1";
            json.dependencies["@auth0/angular-jwt"] = "3.0.0";
            json.dependencies["express"] = "^4.16.4";
            json.dependencies["reflect-metadata"] = "^0.1.12";
            json.dependencies["compression"] = "^1.7.3";
            json.devDependencies["tsc-watch"] = "^4.0.0";
        });
    }
    function editJson(tree, path, edit) {
        let r = tree.read(path);
        let json = JSON.parse(r.toString('utf-8'));
        if (!json)
            console.error("couldn't find json file: " + path);
        edit(json);
        tree.overwrite(path, JSON.stringify(json, null, 2));
    }
    function editGitIgnore(tree) {
        let gitIgnorePath = './.gitignore';
        let r = tree.read(gitIgnorePath);
        let content = r.toString('utf-8');
        content += '\r\n.env\r\ndist-server/*';
        tree.overwrite(gitIgnorePath, content);
    }
    function addToNgModule(tree, what) {
        var options = {};
        options.name = "stam";
        const workspace = config_1.getWorkspace(tree);
        if (!options.project) {
            options.project = Object.keys(workspace.projects)[0];
        }
        const project = workspace.projects[options.project];
        if (options.path === undefined) {
            const projectDirName = project.projectType === 'application' ? 'app' : 'lib';
            options.path = `/${project.root}/src/${projectDirName}`;
        }
        options.module = find_module_1.findModuleFromOptions(tree, options);
        for (const iterator of [
            "imports",
            "declarations",
            "providers",
            "entryComponents"
        ]) {
            if (what[iterator]) {
                for (const d of what[iterator]) {
                    const text = tree.read(options.module);
                    if (text === null) {
                        throw new schematics_1.SchematicsException(`File ${options.module} does not exist!`);
                    }
                    const sourceText = text.toString('utf-8');
                    var ngModel = ts.createSourceFile(options.module, sourceText, ts.ScriptTarget.Latest, true);
                    var declarationChanges = ast_utils_1.addSymbolToNgModuleMetadata(ngModel, 'thePath', iterator, d.name, d.path);
                    let trans = tree.beginUpdate(options.module);
                    for (const c of declarationChanges) {
                        if (c instanceof change_1.InsertChange) {
                            trans.insertLeft(c.pos, c.toAdd);
                        }
                    }
                    tree.commitUpdate(trans);
                }
            }
        }
    }
}
exports.hello = hello;
//# sourceMappingURL=index.js.map