import { DataProviderFactory, DataApiRequest, FilterBase, FindOptionsPerEntity } from "../core/dataInterfaces1";
import { Entity, EntityOptions, Column, ColumnHashSet, IDataSettings, GridSettings, EntitySource, SQLQueryResult, LookupCache } from "../core/utils";
import { DataApiSettings } from "../server/DataApi";
import { HttpClient } from "@angular/common/http";
export declare class Context {
    clearAllCache(): any;
    isSignedIn(): boolean;
    constructor(http?: HttpClient);
    protected _dataSource: DataProviderFactory;
    protected _onServer: boolean;
    readonly onServer: boolean;
    protected _user: UserInfo;
    readonly user: UserInfo;
    _setUser(info: UserInfo): void;
    static apiBaseUrl: string;
    isAllowed(roles: Allowed): any;
    create<lookupIdType, T extends Entity<lookupIdType>>(c: {
        new (...args: any[]): T;
    }): T;
    cache: any;
    for<lookupIdType, T extends Entity<lookupIdType>>(c: {
        new (...args: any[]): T;
    }): SpecificEntityHelper<lookupIdType, T>;
    _lookupCache: LookupCache<any>[];
}
export declare class ServerContext extends Context {
    constructor();
    private req;
    setReq(req: DataApiRequest): void;
    setDataProvider(dataProvider: DataProviderFactory): void;
    getOrigin(): string;
}
export declare class SpecificEntityHelper<lookupIdType, T extends Entity<lookupIdType>> {
    private entity;
    private _lookupCache;
    private context;
    constructor(entity: T, _lookupCache: LookupCache<any>[], context: Context);
    lookup(filter: Column<lookupIdType> | ((entityType: T) => FilterBase)): T;
    lookupAsync(filter: Column<lookupIdType> | ((entityType: T) => FilterBase)): Promise<T>;
    count(where?: (entity: T) => FilterBase): Promise<number>;
    foreach(where: (entity: T) => FilterBase, what?: (entity: T) => Promise<void>): Promise<void>;
    find(options?: FindOptionsPerEntity<T>): Promise<T[]>;
    findFirst(where?: (entity: T) => FilterBase): Promise<T>;
    toPojoArray(items: T[]): Promise<any[]>;
    create(): T;
    gridSettings(settings?: IDataSettings<T>): GridSettings<T>;
}
export interface EntityType {
    new (...args: any[]): Entity<any>;
    __key: string;
}
export declare const allEntities: EntityType[];
export declare function EntityClass<T extends EntityType>(theEntityClass: T): {
    new (...args: any[]): {
        factory?: () => Entity<any>;
        __entityType: EntityType;
        _noContextErrorWithStack: Error;
        __context: Context;
        _setContext(context: Context): void;
        __getApiAllowUpdate(allowed: Allowed): any;
        _setFactoryClassAndDoInitColumns(entityType: EntityType): void;
        _getExcludedColumns(x: Entity<any>, context: Context): Column<any>[];
        _getEntityApiSettings(r: Context): DataApiSettings<Entity<any>>;
        __createInstance(): Entity<any>;
        __options: EntityOptions;
        __getName(): string;
        __getDbName(): string;
        __getCaption(): string;
        __entityData: import("../core/utils").__EntityValueProvider;
        __onSavingRow: () => void | Promise<void>;
        __onValidate: () => void | Promise<void>;
        error: string;
        __idColumn: Column<any>;
        __initColumns(idColumn?: Column<any>): void;
        isValid(): boolean;
        isNew(): boolean;
        __getValidationError(): any;
        setSource(dp: DataProviderFactory): void;
        __assertValidity(): void;
        save(validate?: (row: any) => any, onSavingRow?: (row: any) => any): Promise<any>;
        catchSaveErrors(err: any): any;
        delete(): Promise<any>;
        reset(): void;
        __clearErrors(): void;
        wasChanged(): boolean;
        __toPojo(excludeColumns: ColumnHashSet): Promise<any>;
        __fromPojo(r: any, excludeColumns: ColumnHashSet): any;
        source: EntitySource<any>;
        __applyColumn(y: Column<any>): void;
        __columns: Column<any>[];
        __getColumn<T_1>(col: Column<T_1>): Column<any>;
        __getColumnByJsonName(key: string): Column<any>;
        __iterateColumns(): Column<any>[];
    };
    __key: string;
} & T;
export interface UserInfo {
    id: string;
    name: string;
    roles: string[];
}
export declare abstract class DirectSQL {
    abstract execute(sql: string): Promise<SQLQueryResult>;
}
export declare class Role {
    key: string;
    constructor(key: string);
    static not(allowed: Allowed): Allowed;
}
declare type AllowedRule = string | Role | ((c: Context) => boolean) | boolean;
export declare type Allowed = AllowedRule | AllowedRule[];
export declare type AngularComponent = {
    new (...args: any[]): any;
};
export {};
