import { DataColumnSettings, ColumnOptions, FilterBase, ColumnValueProvider, FindOptionsPerEntity, RowEvents, DataProvider, DataProviderFactory, FilterConsumer, ColumnStorage, EntitySourceFindOptions } from './dataInterfaces1';
import { Allowed, Context, EntityType, DirectSQL } from '../context/Context';
import { DataApiSettings } from '../server/DataApi';
export interface dataAreaSettings {
    columns: ColumnCollection<any>;
    lines: ColumnSetting<any>[][];
}
export declare const testing = "testing 123";
export interface DropDownOptions {
    items?: DropDownItem[] | string[] | any[];
    source?: Entity<any>;
    idColumn?: Column<any>;
    captionColumn?: Column<any>;
    orderBy?: ((rowType: Entity<any>) => Sort) | ((rowType: Entity<any>) => (Column<any>)) | ((rowType: Entity<any>) => (Column<any> | SortSegment)[]);
}
export interface DropDownItem {
    id?: any;
    caption?: any;
}
export declare type DataArealColumnSetting<rowType> = ColumnSetting<rowType> | ColumnSetting<rowType>[];
export interface IDataAreaSettings<rowType> {
    columnSettings?: (rowType: rowType) => DataArealColumnSetting<rowType>[];
    numberOfColumnAreas?: number;
    labelWidth?: number;
}
export declare class DataAreaSettings<rowType extends Entity<any>> {
    settings?: IDataAreaSettings<rowType>;
    columns?: ColumnCollection<rowType>;
    lines: ColumnSetting<any>[][];
    constructor(settings?: IDataAreaSettings<rowType>, columns?: ColumnCollection<rowType>, entity?: rowType);
}
export declare class GridSettings<rowType extends Entity<any>> {
    private entity?;
    settings?: IDataSettings<rowType>;
    constructor(entity?: rowType, settings?: IDataSettings<rowType>);
    currList: ColumnSetting<any>[];
    origList: ColumnSetting<any>[];
    origNumOfColumns: number;
    showSelectColumn: boolean;
    initOrigList(): void;
    userChooseColumns(): void;
    resetColumns(): void;
    addCol(c: ColumnSetting<any>): void;
    deleteCol(c: ColumnSetting<any>): void;
    adjustColumns(): void;
    private setGetOptions;
    addNewRow(): void;
    noam: string;
    addArea(settings: IDataAreaSettings<rowType>): DataAreaSettings<rowType>;
    currentRow: rowType;
    setCurrentRow(row: rowType): void;
    nextRow(): void;
    previousRowAllowed(): boolean;
    previousRow(): void;
    deleteCurentRow(): void;
    currentRowAsRestListItemRow(): any;
    cancelCurrentRowChanges(): void;
    deleteCurrentRowAllowed(): boolean;
    currentRowChanged(): any;
    saveCurrentRow(): void;
    allowUpdate: boolean;
    allowInsert: boolean;
    allowDelete: boolean;
    hideDataArea: boolean;
    _buttons: RowButton<Entity<any>>[];
    rowClass?: (row: any) => string;
    onSavingRow?: (row: any) => Promise<any> | any;
    onValidate?: (row: rowType) => Promise<any> | any;
    onEnterRow: (row: rowType) => void;
    onNewRow: (row: rowType) => void;
    _doSavingRow(s: rowType): Promise<any>;
    caption: string;
    filterHelper: FilterHelper<rowType>;
    columns: ColumnCollection<rowType>;
    page: number;
    nextPage(): Promise<DataList<rowType>>;
    previousPage(): Promise<DataList<rowType>>;
    firstPage(): Promise<DataList<rowType>>;
    rowsPerPage: number;
    rowsPerPageOptions: number[];
    get(options: FindOptionsPerEntity<rowType>): Promise<DataList<rowType>>;
    _currentOrderBy: Sort;
    sort(column: Column<any>): void;
    sortedAscending(column: Column<any>): boolean;
    sortedDescending(column: Column<any>): boolean;
    private getOptions;
    totalRows: number;
    getRecords(): Promise<DataList<rowType>>;
    private restList;
    readonly items: rowType[];
}
export declare class FilterHelper<rowType extends Entity<any>> {
    private reloadData;
    filterRow: rowType;
    filterColumns: Column<any>[];
    forceEqual: Column<any>[];
    constructor(reloadData: () => void);
    isFiltered(column: Column<any>): boolean;
    filterColumn(column: Column<any>, clearFilter: boolean, forceEqual: boolean): void;
    addToFindOptions(opt: FindOptionsPerEntity<rowType>): void;
}
export interface IDataSettings<rowType extends Entity<any>> {
    allowUpdate?: boolean;
    allowInsert?: boolean;
    allowDelete?: boolean;
    hideDataArea?: boolean;
    confirmDelete?: (r: rowType, yes: () => void) => void;
    columnSettings?: (row: rowType) => ColumnSetting<rowType>[];
    areas?: {
        [areaKey: string]: ColumnSetting<any>[];
    };
    rowCssClass?: (row: rowType) => string;
    rowButtons?: RowButton<rowType>[];
    get?: FindOptionsPerEntity<rowType>;
    knowTotalRows?: boolean;
    onSavingRow?: (r: rowType) => void;
    onValidate?: (r: rowType) => void;
    onEnterRow?: (r: rowType) => void;
    onNewRow?: (r: rowType) => void;
    numOfColumnsInGrid?: number;
    caption?: string;
}
export declare type rowEvent<T> = (row: T, doInScope: ((what: (() => void)) => void)) => void;
export interface ColumnSetting<rowType> {
    caption?: string;
    readonly?: boolean;
    inputType?: string;
    designMode?: boolean;
    getValue?: (row: rowType) => any;
    hideDataOnInput?: boolean;
    cssClass?: (string | ((row: rowType) => string));
    defaultValue?: (row: rowType) => any;
    onUserChangedValue?: (row: rowType) => void;
    click?: rowEvent<rowType>;
    allowClick?: (row: rowType) => boolean;
    clickIcon?: string;
    dropDown?: DropDownOptions;
    column?: Column<any>;
    width?: string;
}
export interface FilteredColumnSetting<rowType> extends ColumnSetting<rowType> {
    _showFilter?: boolean;
}
export interface RowButton<rowType extends Entity<any>> {
    name?: string;
    visible?: (r: rowType) => boolean;
    click?: (r: rowType) => void;
    icon?: string;
    cssClass?: (string | ((row: rowType) => string));
}
export declare function isNewRow(r: Entity<any>): boolean;
export declare class DataList<T extends Entity<any>> implements Iterable<T> {
    private entity;
    [Symbol.iterator](): Iterator<T>;
    items: T[];
    constructor(entity: T);
    translateOptions(options: FindOptionsPerEntity<T>): EntitySourceFindOptions;
    _rowReplacedListeners: ((oldRow: T, newRow: T) => void)[];
    private map;
    lastGetId: number;
    count(where?: (rowType: T) => FilterBase): Promise<number>;
    get(options?: FindOptionsPerEntity<T>): Promise<T[]>;
    add(): T;
    private replaceRow;
}
export declare class Sort {
    constructor(...segments: SortSegment[]);
    Segments: SortSegment[];
}
export interface SortSegment {
    column: Column<any>;
    descending?: boolean;
}
export declare class Lookup<lookupIdType, entityType extends Entity<lookupIdType>> {
    private entity;
    constructor(entity: entityType);
    private restList;
    private cache;
    get(filter: Column<lookupIdType> | ((entityType: entityType) => FilterBase)): entityType;
    found(filter: Column<lookupIdType> | ((entityType: entityType) => FilterBase)): boolean;
    private getInternal;
    _internalGetByOptions(find: FindOptionsPerEntity<entityType>): lookupRowInfo<entityType>;
    whenGet(filter: Column<lookupIdType> | ((entityType: entityType) => FilterBase)): Promise<entityType>;
}
export declare class UrlBuilder {
    url: string;
    constructor(url: string);
    add(key: string, value: any): void;
    addObject(object: any, suffix?: string): void;
}
export declare class FilterConsumnerBridgeToUrlBuilder implements FilterConsumer {
    private url;
    constructor(url: UrlBuilder);
    isEqualTo(col: Column<any>, val: any): void;
    isDifferentFrom(col: Column<any>, val: any): void;
    isGreaterOrEqualTo(col: Column<any>, val: any): void;
    isGreaterThan(col: Column<any>, val: any): void;
    isLessOrEqualTo(col: Column<any>, val: any): void;
    isLessThan(col: Column<any>, val: any): void;
    isContains(col: StringColumn, val: any): void;
    isStartsWith(col: StringColumn, val: any): void;
}
export declare class lookupRowInfo<type> {
    found: boolean;
    loading: boolean;
    value: type;
    promise: Promise<lookupRowInfo<type>>;
}
export declare class DefaultStorage<dataType> implements ColumnStorage<dataType> {
    toDb(val: dataType): dataType;
    fromDb(val: any): dataType;
}
export declare class DateTimeDateStorage implements ColumnStorage<string> {
    toDb(val: string): Date;
    fromDb(val: any): string;
}
export declare class CharDateStorage implements ColumnStorage<string> {
    toDb(val: string): string;
    fromDb(val: any): string;
}
export declare class DateTimeStorage implements ColumnStorage<string> {
    toDb(val: string): Date;
    fromDb(val: any): string;
}
export declare class Column<dataType> {
    __calcVirtuals(): Promise<void>;
    private _entity;
    __setEntity(e: Entity<any>): void;
    __isVirtual(): boolean;
    __dbReadOnly(): boolean;
    __clearErrors(): any;
    __performValidation(): void;
    onValidate: () => void;
    onValueChange: () => void;
    jsonName: string;
    caption: string;
    includeInApi: Allowed;
    dbName: string | (() => string);
    private __settings;
    __getMemberName(): string;
    constructor(settingsOrCaption?: ColumnOptions<dataType>);
    __decorateDataSettings(x: ColumnSetting<any>): void;
    __getStorage(): ColumnStorage<dataType>;
    __defaultStorage(): DefaultStorage<any>;
    error: string;
    __getDbName(): string;
    allowApiUpdate: Allowed;
    inputType: string;
    isEqualTo(value: Column<dataType> | dataType): Filter;
    isDifferentFrom(value: Column<dataType> | dataType): Filter;
    isGreaterOrEqualTo(value: Column<dataType> | dataType): Filter;
    isGreaterThan(value: Column<dataType> | dataType): Filter;
    isLessOrEqualTo(value: Column<dataType> | dataType): Filter;
    isLessThan(value: Column<dataType> | dataType): Filter;
    __getVal(value: Column<dataType> | dataType): dataType;
    __valueProvider: ColumnValueProvider;
    value: dataType;
    readonly originalValue: dataType;
    readonly displayValue: string;
    protected __processValue(value: dataType): dataType;
    fromRawValue(value: any): dataType;
    toRawValue(value: dataType): any;
    rawValue: any;
    inputValue: string;
    __addToPojo(pojo: any): void;
    __loadFromToPojo(pojo: any): void;
}
export declare class Filter implements FilterBase {
    private apply;
    constructor(apply: (add: FilterConsumer) => void);
    and(filter: FilterBase): AndFilter;
    __applyToConsumer(add: FilterConsumer): void;
}
export declare class AndFilter implements FilterBase {
    private a;
    private b;
    constructor(a: FilterBase, b: FilterBase);
    and(filter: FilterBase): AndFilter;
    __applyToConsumer(add: FilterConsumer): void;
}
export interface EntityOptions {
    name: string;
    dbName?: string | (() => string);
    caption?: string;
    allowApiRead?: Allowed;
    allowApiUpdate?: Allowed;
    allowApiDelete?: Allowed;
    allowApiInsert?: Allowed;
    allowApiCRUD?: Allowed;
    apiDataFilter?: () => FilterBase;
    onSavingRow?: () => Promise<any> | any;
    onValidate?: (e: Entity<any>) => Promise<any> | any;
}
export declare class Entity<idType> {
    factory?: () => Entity<idType>;
    constructor(options?: EntityOptions | string, factory?: () => Entity<idType>);
    static __key: string;
    _noContextErrorWithStack: Error;
    _setContext(context: Context): void;
    __getApiAllowUpdate(allowed: Allowed): any;
    _setFactoryClassAndDoInitColumns(entityType: EntityType): void;
    _getExcludedColumns(x: Entity<any>, context: Context): Column<any>[];
    _getEntityApiSettings(r: Context): DataApiSettings<Entity<any>>;
    __options: EntityOptions;
    __getName(): string;
    __getDbName(): string;
    __getCaption(): string;
    __entityData: __EntityValueProvider;
    error: string;
    __idColumn: Column<idType>;
    __initColumns(idColumn?: Column<idType>): void;
    isValid(): boolean;
    isNew(): boolean;
    __getValidationError(): any;
    setSource(dp: DataProviderFactory): void;
    __assertValidity(): void;
    save(validate?: (row: this) => Promise<any> | any, onSavingRow?: (row: this) => Promise<any> | any): Promise<any>;
    catchSaveErrors(err: any): any;
    delete(): Promise<any>;
    reset(): void;
    wasChanged(): boolean;
    __toPojo(excludeColumns: ColumnHashSet): Promise<any>;
    __fromPojo(r: any, excludeColumns: ColumnHashSet): any;
    source: EntitySource<this>;
    __getColumn<T>(col: Column<T>): Column<any>;
    __getColumnByJsonName(key: string): Column<any>;
    __iterateColumns(): Column<any>[];
}
export declare class ColumnHashSet {
    private _names;
    add(...columns: Column<any>[]): void;
    contains(column: Column<any>): boolean;
}
export interface LookupCache<T extends Entity<any>> {
    key: string;
    lookup: Lookup<any, T>;
}
export declare class CompoundIdColumn extends Column<string> {
    private columns;
    constructor(entity: Entity<string>, ...columns: Column<any>[]);
    __isVirtual(): boolean;
    isEqualTo(value: Column<string> | string): Filter;
    __addIdToPojo(p: any): void;
    resultIdFilter(id: string, data: any): Filter;
}
export declare class EntitySource<T extends Entity<any>> {
    private factory;
    private _provider;
    constructor(name: string, factory: () => T, dataProvider: DataProviderFactory);
    find(options?: EntitySourceFindOptions): Promise<T[]>;
    fromPojo(r: any): T;
    count(where?: FilterBase): Promise<number>;
    max(col: NumberColumn, filter?: FilterBase): Promise<number>;
    __getDataProvider(): DataProvider;
    createNewItem(): T;
    Insert(doOnRow: (item: T) => Promise<void> | void): Promise<T>;
}
export declare class __EntityValueProvider implements ColumnValueProvider {
    private getDataProvider;
    listeners: RowEvents[];
    register(listener: RowEvents): void;
    delete(): Promise<void>;
    constructor(getDataProvider: () => DataProvider);
    isNewRow(): boolean;
    wasChanged(): boolean;
    reset(): void;
    save(e: Entity<any>): Promise<void>;
    private id;
    private newRow;
    private data;
    private originalData;
    setData(data: any, r: Entity<any>): void;
    getValue(key: string): any;
    getOriginalValue(key: string): any;
    setValue(key: string, value: any): void;
}
export declare class StringColumn extends Column<string> {
    constructor(settingsOrCaption?: ColumnOptions<string>);
    isContains(value: StringColumn | string): Filter;
    isStartsWith(value: StringColumn | string): Filter;
}
export declare class DateColumn extends Column<Date> {
    constructor(settingsOrCaption?: ColumnOptions<Date>);
    getDayOfWeek(): number;
    readonly displayValue: string;
    __defaultStorage(): DateTimeDateStorage;
    toRawValue(value: Date): string;
    fromRawValue(value: any): Date;
    static stringToDate(value: string): Date;
    static dateToString(val: Date): string;
}
export declare class DateTimeColumn extends Column<Date> {
    constructor(settingsOrCaption?: ColumnOptions<Date>);
    getDayOfWeek(): number;
    readonly displayValue: string;
    __defaultStorage(): DateTimeStorage;
    fromRawValue(value: any): Date;
    toRawValue(value: Date): string;
    static stringToDate(val: string): Date;
    static dateToString(val: Date): string;
}
export declare class NumberColumn extends Column<number> {
    constructor(settingsOrCaption?: NumberColumnOptions);
    __numOfDecimalDigits: number;
    protected __processValue(value: number): number;
}
export interface NumberColumnSettings extends DataColumnSettings<number> {
    decimalDigits?: number;
}
export declare type NumberColumnOptions = NumberColumnSettings | string;
export declare class BoolColumn extends Column<boolean> {
    constructor(settingsOrCaption?: ColumnOptions<boolean>);
    __defaultStorage(): BoolStorage;
}
export declare class BoolStorage implements ColumnStorage<any> {
    toDb(val: any): any;
    fromDb(val: any): any;
}
export interface ClosedListItem {
    id: number;
    toString(): string;
}
export declare class ClosedListColumn<closedListType extends ClosedListItem> extends Column<closedListType> {
    private closedListType;
    constructor(closedListType: any, settingsOrCaption?: ColumnOptions<closedListType>);
    getOptions(): DropDownItem[];
    toRawValue(value: closedListType): number;
    fromRawValue(value: any): closedListType;
    readonly displayValue: string;
    byId(id: number): closedListType;
}
export declare class ColumnCollection<rowType extends Entity<any>> {
    currentRow: () => Entity<any>;
    private allowUpdate;
    filterHelper: FilterHelper<rowType>;
    private showArea;
    constructor(currentRow: () => Entity<any>, allowUpdate: () => boolean, filterHelper: FilterHelper<rowType>, showArea: () => boolean);
    __showArea(): boolean;
    __getColumn(map: ColumnSetting<any>, record: Entity<any>): Column<any>;
    __dataControlStyle(map: ColumnSetting<any>): string;
    private settingsByKey;
    allowDesignMode: boolean;
    add(...columns: ColumnSetting<rowType>[]): Promise<void>;
    add(...columns: string[]): Promise<void>;
    buildDropDown(s: ColumnSetting<any>): Promise<void>;
    designMode: boolean;
    colListChanged(): void;
    _colListChangeListeners: (() => void)[];
    onColListChange(action: (() => void)): void;
    moveCol(col: ColumnSetting<any>, move: number): void;
    filterRows(col: FilteredColumnSetting<any>): void;
    clearFilter(col: FilteredColumnSetting<any>): void;
    _shouldShowFilterDialog(col: FilteredColumnSetting<any>): boolean;
    showFilterDialog(col: FilteredColumnSetting<any>): void;
    deleteCol(col: ColumnSetting<any>): void;
    addCol(col: ColumnSetting<any>): void;
    designColumn(col: ColumnSetting<any>): void;
    _getEditable(col: ColumnSetting<any>): boolean;
    _click(col: ColumnSetting<any>, row: any): void;
    _getColDisplayValue(col: ColumnSetting<any>, row: rowType): any;
    _getColDataType(col: ColumnSetting<any>): string;
    _getColumnClass(col: ColumnSetting<any>, row: any): any;
    _getError(col: ColumnSetting<any>, r: Entity<any>): string;
    autoGenerateColumnsBasedOnData(r: Entity<any>): void;
    __columnSettingsTypeScript(): string;
    __columnTypeScriptDescription(c: ColumnSetting<any>, memberName: string): string;
    __changeWidth(col: ColumnSetting<any>, what: number): void;
    _colValueChanged(col: ColumnSetting<any>, r: any): void;
    items: ColumnSetting<any>[];
    private gridColumns;
    private nonGridColumns;
    numOfColumnsInGrid: number;
    private _lastColumnCount;
    private _lastNumOfColumnsInGrid;
    private _initColumnsArrays;
    getGridColumns(): ColumnSetting<any>[];
    getNonGridColumns(): ColumnSetting<any>[];
}
export declare function extractSortFromSettings<T extends Entity<any>>(entity: T, opt: FindOptionsPerEntity<T>): Sort;
export declare function translateSort(sort: any): Sort;
export interface SQLCommand {
    addParameterToCommandAndReturnParameterName(col: Column<any>, val: any): string;
    query(sql: string): Promise<SQLQueryResult>;
}
export interface SQLQueryResult {
    rows: any[];
    getColumnIndex(name: string): number;
    getcolumnNameAtIndex(index: number): string;
}
export interface SQLConnectionProvider {
    createCommand(): SQLCommand;
}
export interface SupportsDirectSql {
    getDirectSql(): DirectSQL;
}
export declare class FilterConsumerBridgeToSqlRequest implements FilterConsumer {
    private r;
    where: string;
    constructor(r: SQLCommand);
    isEqualTo(col: Column<any>, val: any): void;
    isDifferentFrom(col: Column<any>, val: any): void;
    isGreaterOrEqualTo(col: Column<any>, val: any): void;
    isGreaterThan(col: Column<any>, val: any): void;
    isLessOrEqualTo(col: Column<any>, val: any): void;
    isLessThan(col: Column<any>, val: any): void;
    isContains(col: StringColumn, val: any): void;
    isStartsWith(col: StringColumn, val: any): void;
    private add;
}
