import { DataApiError } from './DataApi';
import { Entity, Column } from '../core/utils';
import { FindOptionsPerEntity, DataApiRequest } from '../core/dataInterfaces1';
export declare class DataApi<T extends Entity<any>> {
    private rowType;
    private options?;
    getRoute(): string;
    constructor(rowType: T, options?: DataApiSettings<T>);
    private excludedColumns;
    private readonlyColumns;
    get(response: DataApiResponse, id: any): Promise<void>;
    count(response: DataApiResponse, request: DataApiRequest): Promise<void>;
    getArray(response: DataApiResponse, request: DataApiRequest): Promise<void>;
    private buildWhere;
    private doOnId;
    put(response: DataApiResponse, id: any, body: any): Promise<void>;
    delete(response: DataApiResponse, id: any): Promise<void>;
    post(response: DataApiResponse, body: any): Promise<void>;
}
export interface DataApiSettings<rowType extends Entity<any>> {
    allowUpdate?: boolean;
    allowInsert?: boolean;
    allowDelete?: boolean;
    allowRead?: boolean;
    excludeColumns?: (r: rowType) => Column<any>[];
    readonlyColumns?: (r: rowType) => Column<any>[];
    name?: string;
    get?: FindOptionsPerEntity<rowType>;
    validate?: (r: rowType) => void;
    onSavingRow?: (r: rowType) => Promise<any> | any;
}
export interface DataApiResponse {
    success(data: any): void;
    deleted(): void;
    created(data: any): void;
    notFound(): void;
    error(data: DataApiError): void;
    methodNotAllowed(): void;
    forbidden(): void;
}
export interface DataApiError {
    message: string;
}
