import { ErrorStateMatcher } from '@angular/material/core';
import { CommonModule } from '@angular/common';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { isBoolean, isString, isArray, isFunction } from 'util';
import { JwtHelperService } from '@auth0/angular-jwt';
import { Router, ActivatedRouteSnapshot } from '@angular/router';
import { HttpClient, HttpClientModule, HTTP_INTERCEPTORS } from '@angular/common/http';
import { MatProgressSpinnerModule } from '@angular/material/progress-spinner';
import { finalize } from 'rxjs/operators';
import { MatDialogModule } from '@angular/material/dialog';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatInputModule } from '@angular/material/input';
import { MatDialog, MatIconModule, MatButtonModule, MatCheckboxModule } from '@angular/material';
import { Component, Input, ViewChild, Injectable, Injector, ViewEncapsulation, NgModule } from '@angular/core';
import 'reflect-metadata';
import { __awaiter } from 'tslib';
import { v4 } from 'uuid';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} name
 * @return {?}
 */
function makeTitle(name) {
    // insert a space before all caps
    return name.replace(/([A-Z])/g, ' $1')
        // uppercase the first character
        .replace(/^./, (/**
     * @param {?} str
     * @return {?}
     */
    (str) => str.toUpperCase())).replace('Email', 'eMail').replace(" I D", " ID");
}
/**
 * @param {?} functionToCheck
 * @return {?}
 */
function isFunction$1(functionToCheck) {
    /** @type {?} */
    var getType = {};
    return functionToCheck && getType.toString.call(functionToCheck) === '[object Function]';
}
/**
 * @param {?} f
 * @return {?}
 */
function functionOrString(f) {
    if (isFunction$1(f)) {
        /** @type {?} */
        let x = (/** @type {?} */ (f));
        return x();
    }
    return f.toString();
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const testing = 'testing 123';
/**
 * @template rowType
 */
class DataAreaSettings {
    /**
     * @param {?=} settings
     * @param {?=} columns
     * @param {?=} entity
     */
    constructor(settings, columns, entity) {
        this.settings = settings;
        this.columns = columns;
        this.lines = [];
        if (columns == undefined) {
            columns = new ColumnCollection((/**
             * @return {?}
             */
            () => undefined), (/**
             * @return {?}
             */
            () => true), undefined, (/**
             * @return {?}
             */
            () => true));
            columns.numOfColumnsInGrid = 0;
            this.columns = columns;
        }
        if (settings && settings.columnSettings) {
            for (const colSettings of settings.columnSettings(entity)) {
                if (isArray(colSettings)) {
                    /** @type {?} */
                    let x = columns.items.length;
                    //@ts-ignore
                    columns.add(...colSettings);
                    /** @type {?} */
                    let line = [];
                    for (let index = x; index < columns.items.length; index++) {
                        line.push(columns.items[index]);
                    }
                    this.lines.push(line);
                }
                else {
                    columns.add((/** @type {?} */ (colSettings)));
                    this.lines.push([columns.items[columns.items.length - 1]]);
                }
            }
        }
    }
}
/**
 * @template rowType
 */
class GridSettings {
    /**
     * @param {?=} entity
     * @param {?=} settings
     */
    constructor(entity, settings) {
        this.entity = entity;
        this.settings = settings;
        this.showSelectColumn = false;
        this.allowUpdate = false;
        this.allowInsert = false;
        this.allowDelete = false;
        this.hideDataArea = false;
        this._buttons = [];
        this.filterHelper = new FilterHelper((/**
         * @return {?}
         */
        () => {
            this.page = 1;
            this.getRecords();
        }));
        this.page = 1;
        this.rowsPerPageOptions = [10, 25, 50, 100, 500, 1000];
        this.restList = new DataList(entity);
        if (entity)
            this.filterHelper.filterRow = entity.source.createNewItem();
        this.columns = new ColumnCollection((/**
         * @return {?}
         */
        () => this.currentRow), (/**
         * @return {?}
         */
        () => this.allowUpdate), this.filterHelper, (/**
         * @return {?}
         */
        () => this.currentRow ? true : false));
        this.restList._rowReplacedListeners.push((/**
         * @param {?} old
         * @param {?} curr
         * @return {?}
         */
        (old, curr) => {
            if (old == this.currentRow)
                this.setCurrentRow(curr);
        }));
        if (settings) {
            if (settings.columnSettings)
                this.columns.add(...settings.columnSettings(entity));
            if (settings.allowUpdate)
                this.allowUpdate = true;
            if (settings.allowDelete)
                this.allowDelete = true;
            if (settings.allowInsert)
                this.allowInsert = true;
            if (settings.hideDataArea)
                this.hideDataArea = settings.hideDataArea;
            if (settings.numOfColumnsInGrid != undefined)
                this.columns.numOfColumnsInGrid = settings.numOfColumnsInGrid;
            if (settings.rowButtons)
                this._buttons = settings.rowButtons;
            if (settings.rowCssClass)
                this.rowClass = settings.rowCssClass;
            if (settings.onSavingRow)
                this.onSavingRow = settings.onSavingRow;
            if (settings.onEnterRow)
                this.onEnterRow = settings.onEnterRow;
            if (settings.onNewRow)
                this.onNewRow = settings.onNewRow;
            if (settings.onValidate)
                this.onValidate = settings.onValidate;
            if (settings.caption)
                this.caption = settings.caption;
            if (!this.caption && entity) {
                this.caption = entity.source.createNewItem().__getCaption();
            }
            this.setGetOptions(settings.get);
        }
    }
    /**
     * @return {?}
     */
    initOrigList() {
        if (!this.origList) {
            this.origList = [];
            this.origNumOfColumns = this.columns.numOfColumnsInGrid;
            this.origList.push(...this.columns.items);
        }
    }
    /**
     * @return {?}
     */
    userChooseColumns() {
        this.initOrigList();
        if (!this.currList) {
            this.resetColumns();
        }
        this.showSelectColumn = !this.showSelectColumn;
    }
    /**
     * @return {?}
     */
    resetColumns() {
        this.currList = [];
        this.columns.items = this.currList;
        this.columns.numOfColumnsInGrid = this.origNumOfColumns;
        for (let i = 0; i < this.origList.length; i++) {
            if (i < this.columns.numOfColumnsInGrid)
                this.currList.push(this.origList[i]);
        }
    }
    /**
     * @param {?} c
     * @return {?}
     */
    addCol(c) {
        this.columns.addCol(c);
        this.adjustColumns();
    }
    /**
     * @param {?} c
     * @return {?}
     */
    deleteCol(c) {
        this.columns.deleteCol(c);
        this.adjustColumns();
    }
    /**
     * @return {?}
     */
    adjustColumns() {
        this.columns.items.forEach((/**
         * @param {?} c
         * @return {?}
         */
        c => c.designMode = false));
        this.columns.numOfColumnsInGrid = this.columns.items.length;
    }
    /**
     * @private
     * @param {?} get
     * @return {?}
     */
    setGetOptions(get) {
        this.getOptions = get;
        if (get && get.limit)
            this.rowsPerPage = get.limit;
        else
            this.rowsPerPage = 7;
        if (this.rowsPerPageOptions.indexOf(this.rowsPerPage) < 0) {
            this.rowsPerPageOptions.push(this.rowsPerPage);
            this.rowsPerPageOptions.sort((/**
             * @param {?} a
             * @param {?} b
             * @return {?}
             */
            (a, b) => +a - +b));
        }
        this._currentOrderBy = undefined;
        if (this.getOptions && this.getOptions.orderBy)
            this._currentOrderBy = extractSortFromSettings(this.entity, this.getOptions);
    }
    /**
     * @return {?}
     */
    addNewRow() {
        /** @type {?} */
        let r = this.restList.add();
        this.columns.items.forEach((/**
         * @param {?} item
         * @return {?}
         */
        item => {
            if (item.defaultValue) {
                /** @type {?} */
                let result = item.defaultValue(r);
            }
        }));
        if (this.onNewRow)
            this.onNewRow(r);
        this.setCurrentRow(r);
    }
    /**
     * @param {?} settings
     * @return {?}
     */
    addArea(settings) {
        /** @type {?} */
        let col = new ColumnCollection((/**
         * @return {?}
         */
        () => this.currentRow), (/**
         * @return {?}
         */
        () => this.allowUpdate), this.filterHelper, (/**
         * @return {?}
         */
        () => this.currentRow ? true : false));
        col.numOfColumnsInGrid = 0;
        return new DataAreaSettings(settings, col, this.entity);
    }
    /**
     * @param {?} row
     * @return {?}
     */
    setCurrentRow(row) {
        if (this.currentRow != row) {
            this.currentRow = row;
            if (this.onEnterRow && row) {
                this.onEnterRow(row);
            }
        }
    }
    /**
     * @return {?}
     */
    nextRow() {
        if (!this.currentRow && this.items.length > 0)
            this.setCurrentRow(this.items[0]);
        if (this.currentRow) {
            /** @type {?} */
            let currentRowPosition = this.items.indexOf(this.currentRow);
            if (currentRowPosition < this.items.length - 1)
                this.setCurrentRow(this.items[currentRowPosition + 1]);
            else
                this.nextPage().then((/**
                 * @return {?}
                 */
                () => {
                    if (this.items.length > 0)
                        this.setCurrentRow(this.items[0]);
                }));
        }
    }
    /**
     * @return {?}
     */
    previousRowAllowed() {
        return this.currentRow && this.items.indexOf(this.currentRow) > 0 || this.page > 1;
    }
    /**
     * @return {?}
     */
    previousRow() {
        if (!this.previousRowAllowed())
            return;
        /** @type {?} */
        let currentRowPosition = this.items.indexOf(this.currentRow);
        if (currentRowPosition > 0)
            this.setCurrentRow(this.items[currentRowPosition - 1]);
        else {
            if (this.page > 1)
                this.previousPage().then((/**
                 * @return {?}
                 */
                () => {
                    if (this.items.length > 0)
                        this.setCurrentRow(this.items[this.items.length - 1]);
                }));
        }
    }
    /**
     * @return {?}
     */
    deleteCurentRow() {
        if (!this.deleteCurrentRowAllowed)
            return;
        this.currentRowAsRestListItemRow().delete();
    }
    /**
     * @return {?}
     */
    currentRowAsRestListItemRow() {
        if (!this.currentRow)
            return undefined;
        return (/** @type {?} */ (this.currentRow));
    }
    /**
     * @return {?}
     */
    cancelCurrentRowChanges() {
        if (this.currentRowAsRestListItemRow() && this.currentRowAsRestListItemRow().reset)
            this.currentRowAsRestListItemRow().reset();
    }
    /**
     * @return {?}
     */
    deleteCurrentRowAllowed() {
        return this.currentRowAsRestListItemRow() && this.currentRowAsRestListItemRow().delete && this.allowDelete && !isNewRow(this.currentRow);
    }
    /**
     * @return {?}
     */
    currentRowChanged() {
        return this.currentRowAsRestListItemRow() && this.currentRowAsRestListItemRow().__wasChanged && this.currentRowAsRestListItemRow().__wasChanged();
    }
    /**
     * @return {?}
     */
    saveCurrentRow() {
        if (this.currentRowAsRestListItemRow() && this.currentRowAsRestListItemRow().save)
            this.currentRowAsRestListItemRow().save();
    }
    /**
     * @param {?} s
     * @return {?}
     */
    _doSavingRow(s) {
        return s.save(this.onValidate, this.onSavingRow);
    }
    /**
     * @return {?}
     */
    nextPage() {
        this.page++;
        return this.getRecords();
    }
    /**
     * @return {?}
     */
    previousPage() {
        if (this.page <= 1)
            return;
        this.page--;
        return this.getRecords();
    }
    /**
     * @return {?}
     */
    firstPage() {
        this.page = 1;
        return this.getRecords();
    }
    /**
     * @param {?} options
     * @return {?}
     */
    get(options) {
        this.setGetOptions(options);
        this.page = 1;
        return this.getRecords();
    }
    /**
     * @param {?} column
     * @return {?}
     */
    sort(column) {
        /** @type {?} */
        let done = false;
        if (this._currentOrderBy && this._currentOrderBy.Segments.length > 0) {
            if (this._currentOrderBy.Segments[0].column == column) {
                this._currentOrderBy.Segments[0].descending = !this._currentOrderBy.Segments[0].descending;
                done = true;
            }
        }
        if (!done)
            this._currentOrderBy = new Sort({ column: column });
        this.getRecords();
    }
    /**
     * @param {?} column
     * @return {?}
     */
    sortedAscending(column) {
        if (!this._currentOrderBy)
            return false;
        if (!column)
            return false;
        return this._currentOrderBy.Segments.length > 0 &&
            this._currentOrderBy.Segments[0].column == column &&
            !this._currentOrderBy.Segments[0].descending;
    }
    /**
     * @param {?} column
     * @return {?}
     */
    sortedDescending(column) {
        if (!this._currentOrderBy)
            return false;
        if (!column)
            return false;
        return this._currentOrderBy.Segments.length > 0 &&
            this._currentOrderBy.Segments[0].column == column &&
            !!this._currentOrderBy.Segments[0].descending;
    }
    /**
     * @return {?}
     */
    getRecords() {
        /** @type {?} */
        let opt = {};
        if (this.getOptions) {
            opt = Object.assign(opt, this.getOptions);
        }
        if (this._currentOrderBy)
            opt.orderBy = (/**
             * @param {?} r
             * @return {?}
             */
            r => this._currentOrderBy);
        opt.limit = this.rowsPerPage;
        if (this.page > 1)
            opt.page = this.page;
        this.filterHelper.addToFindOptions(opt);
        /** @type {?} */
        let result = this.restList.get(opt).then((/**
         * @return {?}
         */
        () => {
            if (this.restList.items.length == 0) {
                this.setCurrentRow(undefined);
                this.columns.autoGenerateColumnsBasedOnData(this.entity);
            }
            else {
                this.setCurrentRow(this.restList.items[0]);
                this.columns.autoGenerateColumnsBasedOnData(this.entity);
            }
            return this.restList;
        }));
        if (this.settings && this.settings.knowTotalRows) {
            this.restList.count(opt.where).then((/**
             * @param {?} x
             * @return {?}
             */
            x => {
                this.totalRows = x;
            }));
        }
        return result;
    }
    ;
    /**
     * @return {?}
     */
    get items() {
        if (this.restList)
            return this.restList.items;
        return undefined;
    }
}
/**
 * @template rowType
 */
class FilterHelper {
    /**
     * @param {?} reloadData
     */
    constructor(reloadData) {
        this.reloadData = reloadData;
        this.filterColumns = [];
        this.forceEqual = [];
    }
    /**
     * @param {?} column
     * @return {?}
     */
    isFiltered(column) {
        return this.filterColumns.indexOf(column) >= 0;
    }
    /**
     * @param {?} column
     * @param {?} clearFilter
     * @param {?} forceEqual
     * @return {?}
     */
    filterColumn(column, clearFilter, forceEqual) {
        if (!column)
            return;
        if (clearFilter) {
            this.filterColumns.splice(this.filterColumns.indexOf(column, 1), 1);
            this.forceEqual.splice(this.forceEqual.indexOf(column, 1), 1);
        }
        else if (this.filterColumns.indexOf(column) < 0) {
            this.filterColumns.push(column);
            if (forceEqual)
                this.forceEqual.push(column);
        }
        this.reloadData();
    }
    /**
     * @param {?} opt
     * @return {?}
     */
    addToFindOptions(opt) {
        this.filterColumns.forEach((/**
         * @param {?} c
         * @return {?}
         */
        c => {
            /** @type {?} */
            let val = this.filterRow.__getColumn(c).value;
            /** @type {?} */
            let f = c.isEqualTo(val);
            if (c instanceof StringColumn) {
                /** @type {?} */
                let fe = this.forceEqual;
                if (fe.indexOf(c) < 0)
                    f = c.isContains(val);
            }
            if (c instanceof DateTimeColumn) {
                if (val) {
                    /** @type {?} */
                    let v = DateTimeColumn.stringToDate(val);
                    v = new Date(v.getFullYear(), v.getMonth(), v.getDate());
                    f = c.isGreaterOrEqualTo(v).and(c.isLessThan((new Date(v.getFullYear(), v.getMonth(), v.getDate() + 1))));
                }
            }
            if (opt.where) {
                /** @type {?} */
                let x = opt.where;
                opt.where = (/**
                 * @param {?} r
                 * @return {?}
                 */
                r => new AndFilter(x(r), f));
            }
            else
                opt.where = (/**
                 * @param {?} r
                 * @return {?}
                 */
                r => f);
        }));
    }
}
/**
 * @param {?} r
 * @return {?}
 */
function isNewRow(r) {
    if (r) {
        r.__entityData.isNewRow();
    }
    return false;
}
/**
 * @template T
 */
class DataList {
    /**
     * @param {?} entity
     */
    constructor(entity) {
        this.entity = entity;
        this.items = [];
        this._rowReplacedListeners = [];
        this.lastGetId = 0;
    }
    /**
     * @return {?}
     */
    [Symbol.iterator]() {
        return this.items[Symbol.iterator]();
    }
    /**
     * @param {?} options
     * @return {?}
     */
    translateOptions(options) {
        if (!options)
            return undefined;
        /** @type {?} */
        let getOptions = {};
        if (options.where)
            getOptions.where = options.where(this.entity);
        if (options.orderBy)
            getOptions.orderBy = extractSortFromSettings(this.entity, options);
        if (options.limit)
            getOptions.limit = options.limit;
        if (options.page)
            getOptions.page = options.page;
        if (options.additionalUrlParameters)
            getOptions.additionalUrlParameters = options.additionalUrlParameters;
        return getOptions;
    }
    /**
     * @private
     * @param {?} item
     * @return {?}
     */
    map(item) {
        item.__entityData.register({
            rowReset: (/**
             * @param {?} newRow
             * @return {?}
             */
            (newRow) => {
                if (newRow)
                    this.items.splice(this.items.indexOf(item), 1);
            }),
            rowDeleted: (/**
             * @return {?}
             */
            () => { this.items.splice(this.items.indexOf(item), 1); })
        });
        return item;
    }
    /**
     * @param {?=} where
     * @return {?}
     */
    count(where) {
        /** @type {?} */
        let w = undefined;
        if (where)
            w = where(this.entity);
        return this.entity.source.count(w);
    }
    /**
     * @param {?=} options
     * @return {?}
     */
    get(options) {
        /** @type {?} */
        let getId = ++this.lastGetId;
        return this.entity.source.find(this.translateOptions(options)).then((/**
         * @param {?} r
         * @return {?}
         */
        r => {
            /** @type {?} */
            let result = r.map((/**
             * @param {?} x
             * @return {?}
             */
            (x) => this.map(x)));
            if (getId == this.lastGetId)
                this.items = result;
            return result;
        }));
    }
    /**
     * @return {?}
     */
    add() {
        /** @type {?} */
        let x = this.map(this.entity.source.createNewItem());
        this.items.push(x);
        return x;
    }
    /**
     * @private
     * @param {?} originalRow
     * @param {?} newRow
     * @return {?}
     */
    replaceRow(originalRow, newRow) {
        newRow = this.map(newRow);
        this.items[this.items.indexOf(originalRow)] = newRow;
        this._rowReplacedListeners.forEach((/**
         * @param {?} x
         * @return {?}
         */
        x => x(originalRow, newRow)));
    }
}
class Sort {
    /**
     * @param {...?} segments
     */
    constructor(...segments) {
        this.Segments = segments;
    }
}
/**
 * @template lookupIdType, entityType
 */
class Lookup {
    /**
     * @param {?} entity
     */
    constructor(entity) {
        this.entity = entity;
        this.cache = {};
        this.restList = new DataList(entity);
    }
    /**
     * @param {?} filter
     * @return {?}
     */
    get(filter) {
        return this.getInternal(filter).value;
    }
    /**
     * @param {?} filter
     * @return {?}
     */
    found(filter) {
        return this.getInternal(filter).found;
    }
    /**
     * @private
     * @param {?} filter
     * @return {?}
     */
    getInternal(filter) {
        /** @type {?} */
        let find = {};
        if (filter instanceof Column)
            find.where = (/**
             * @param {?} e
             * @return {?}
             */
            (e) => e.__idColumn.isEqualTo(filter));
        else if (isFunction$1(filter)) {
            find.where = (/**
             * @param {?} e
             * @return {?}
             */
            e => filter(e));
        }
        return this._internalGetByOptions(find);
    }
    /**
     * @param {?} find
     * @return {?}
     */
    _internalGetByOptions(find) {
        /** @type {?} */
        let key = "";
        /** @type {?} */
        let url = new UrlBuilder("");
        if (find.where)
            find.where(this.entity).__applyToConsumer(new FilterConsumnerBridgeToUrlBuilder(url));
        key = url.url;
        if (this.cache == undefined)
            this.cache = {};
        if (this.cache[key]) {
            return this.cache[key];
        }
        else {
            /** @type {?} */
            let res = new lookupRowInfo();
            this.cache[key] = res;
            if (find == undefined || key == undefined) {
                res.loading = false;
                res.found = false;
                return res;
            }
            else {
                res.value = this.entity.source.createNewItem();
                res.promise = this.restList.get(find).then((/**
                 * @param {?} r
                 * @return {?}
                 */
                r => {
                    res.loading = false;
                    if (r.length > 0) {
                        res.value = r[0];
                        res.found = true;
                    }
                    return res;
                }));
            }
            return res;
        }
    }
    /**
     * @param {?} filter
     * @return {?}
     */
    whenGet(filter) {
        return this.getInternal(filter).promise.then((/**
         * @param {?} r
         * @return {?}
         */
        r => r.value));
    }
}
class UrlBuilder {
    /**
     * @param {?} url
     */
    constructor(url) {
        this.url = url;
    }
    /**
     * @param {?} key
     * @param {?} value
     * @return {?}
     */
    add(key, value) {
        if (this.url.indexOf('?') >= 0)
            this.url += '&';
        else
            this.url += '?';
        this.url += encodeURIComponent(key) + '=' + encodeURIComponent(value);
    }
    /**
     * @param {?} object
     * @param {?=} suffix
     * @return {?}
     */
    addObject(object, suffix = '') {
        if (object != undefined)
            for (var key in object) {
                /** @type {?} */
                let val = object[key];
                if (val instanceof Column)
                    val = val.value;
                this.add(key + suffix, val);
            }
    }
}
class FilterConsumnerBridgeToUrlBuilder {
    /**
     * @param {?} url
     */
    constructor(url) {
        this.url = url;
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isEqualTo(col, val) {
        this.url.add(col.jsonName, val);
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isDifferentFrom(col, val) {
        this.url.add(col.jsonName + '_ne', val);
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isGreaterOrEqualTo(col, val) {
        this.url.add(col.jsonName + '_gte', val);
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isGreaterThan(col, val) {
        this.url.add(col.jsonName + '_gt', val);
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isLessOrEqualTo(col, val) {
        this.url.add(col.jsonName + '_lte', val);
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isLessThan(col, val) {
        this.url.add(col.jsonName + '_lt', val);
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isContains(col, val) {
        this.url.add(col.jsonName + "_contains", val);
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isStartsWith(col, val) {
        this.url.add(col.jsonName + "_st", val);
    }
}
/**
 * @template type
 */
class lookupRowInfo {
    constructor() {
        this.found = false;
        this.loading = true;
        this.value = (/** @type {?} */ ({}));
    }
}
/**
 * @template dataType
 */
class DefaultStorage {
    /**
     * @param {?} val
     * @return {?}
     */
    toDb(val) {
        return val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    fromDb(val) {
        return val;
    }
}
class DateTimeDateStorage {
    /**
     * @param {?} val
     * @return {?}
     */
    toDb(val) {
        return DateColumn.stringToDate(val);
    }
    /**
     * @param {?} val
     * @return {?}
     */
    fromDb(val) {
        /** @type {?} */
        var d = (/** @type {?} */ (val));
        return DateColumn.dateToString(d);
    }
}
class CharDateStorage {
    /**
     * @param {?} val
     * @return {?}
     */
    toDb(val) {
        return val.replace(/-/g, '');
    }
    /**
     * @param {?} val
     * @return {?}
     */
    fromDb(val) {
        return val.substring(0, 4) + '-' + val.substring(4, 6) + '-' + val.substring(6, 8);
    }
}
class DateTimeStorage {
    /**
     * @param {?} val
     * @return {?}
     */
    toDb(val) {
        return DateTimeColumn.stringToDate(val);
    }
    /**
     * @param {?} val
     * @return {?}
     */
    fromDb(val) {
        /** @type {?} */
        var d = (/** @type {?} */ (val));
        return DateTimeColumn.dateToString(d);
    }
}
/**
 * @param {?} number
 * @param {?=} stringLength
 * @return {?}
 */
function addZeros(number, stringLength = 2) {
    /** @type {?} */
    let to = number.toString();
    while (to.length < stringLength)
        to = '0' + to;
    return to;
}
/**
 * @template dataType
 */
class Column {
    /**
     * @param {?=} settingsOrCaption
     */
    constructor(settingsOrCaption) {
        this.includeInApi = true;
        this.allowApiUpdate = true;
        this.__valueProvider = new dummyColumnStorage();
        if (settingsOrCaption) {
            if (typeof (settingsOrCaption) === "string") {
                this.caption = settingsOrCaption;
                this.__settings = { caption: settingsOrCaption };
            }
            else {
                this.__settings = settingsOrCaption;
                if (settingsOrCaption.jsonName)
                    this.jsonName = settingsOrCaption.jsonName;
                if (settingsOrCaption.caption)
                    this.caption = settingsOrCaption.caption;
                if (settingsOrCaption.includeInApi != undefined)
                    this.includeInApi = settingsOrCaption.includeInApi;
                if (settingsOrCaption.allowApiUpdate != undefined)
                    this.allowApiUpdate = settingsOrCaption.allowApiUpdate;
                if (settingsOrCaption.inputType)
                    this.inputType = settingsOrCaption.inputType;
                if (settingsOrCaption.dbName)
                    this.dbName = settingsOrCaption.dbName;
                if (settingsOrCaption.value != undefined)
                    this.value = settingsOrCaption.value;
                if (settingsOrCaption.valueChange)
                    this.onValueChange = (/**
                     * @return {?}
                     */
                    () => this.__settings.valueChange(this.value));
                if (settingsOrCaption.onValidate)
                    this.onValidate = (/**
                     * @return {?}
                     */
                    () => settingsOrCaption.onValidate());
            }
        }
    }
    /**
     * @return {?}
     */
    __calcVirtuals() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.__settings && this.__settings.virtualData) {
                /** @type {?} */
                let x = this.__settings.virtualData();
                if (x instanceof Promise)
                    x = yield x;
                this.value = x;
            }
        });
    }
    /**
     * @param {?} e
     * @return {?}
     */
    __setEntity(e) {
        this._entity = e;
    }
    /**
     * @return {?}
     */
    __isVirtual() {
        if (this.__settings && this.__settings.virtualData)
            return true;
        return false;
    }
    /**
     * @return {?}
     */
    __dbReadOnly() {
        if (this.__settings && this.__settings.dbReadOnly)
            return true;
        return this.__isVirtual();
    }
    /**
     * @return {?}
     */
    __clearErrors() {
        this.error = undefined;
    }
    /**
     * @return {?}
     */
    __performValidation() {
        if (this.onValidate) {
            this.onValidate();
        }
    }
    /**
     * @return {?}
     */
    __getMemberName() { return this.jsonName; }
    /**
     * @param {?} x
     * @return {?}
     */
    __decorateDataSettings(x) {
        if (!x.caption && this.caption)
            x.caption = this.caption;
        if (x.readonly == undefined)
            if (this._entity)
                x.readonly = !this._entity.__getApiAllowUpdate(this.allowApiUpdate);
        if (x.inputType == undefined)
            x.inputType = this.inputType;
        if (x.getValue == undefined) {
            if (this.__settings && this.__settings.getValue)
                x.getValue = (/**
                 * @param {?} e
                 * @return {?}
                 */
                e => {
                    /** @type {?} */
                    let c = this;
                    if (e)
                        c = (/** @type {?} */ (e.__getColumn(c)));
                    return c.__settings.getValue(c.value);
                });
        }
    }
    /**
     * @return {?}
     */
    __getStorage() {
        if (!this.__settings)
            this.__settings = {};
        if (!this.__settings.storage)
            this.__settings.storage = this.__defaultStorage();
        return this.__settings.storage;
    }
    /**
     * @return {?}
     */
    __defaultStorage() {
        return new DefaultStorage();
    }
    /**
     * @return {?}
     */
    __getDbName() {
        if (this.dbName)
            return functionOrString(this.dbName);
        return this.jsonName;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    isEqualTo(value) {
        return new Filter((/**
         * @param {?} add
         * @return {?}
         */
        add => add.isEqualTo(this, this.__getVal(value))));
    }
    /**
     * @param {?} value
     * @return {?}
     */
    isDifferentFrom(value) {
        return new Filter((/**
         * @param {?} add
         * @return {?}
         */
        add => add.isDifferentFrom(this, this.__getVal(value))));
    }
    /**
     * @param {?} value
     * @return {?}
     */
    isGreaterOrEqualTo(value) {
        return new Filter((/**
         * @param {?} add
         * @return {?}
         */
        add => add.isGreaterOrEqualTo(this, this.__getVal(value))));
    }
    /**
     * @param {?} value
     * @return {?}
     */
    isGreaterThan(value) {
        return new Filter((/**
         * @param {?} add
         * @return {?}
         */
        add => add.isGreaterThan(this, this.__getVal(value))));
    }
    /**
     * @param {?} value
     * @return {?}
     */
    isLessOrEqualTo(value) {
        return new Filter((/**
         * @param {?} add
         * @return {?}
         */
        add => add.isLessOrEqualTo(this, this.__getVal(value))));
    }
    /**
     * @param {?} value
     * @return {?}
     */
    isLessThan(value) {
        return new Filter((/**
         * @param {?} add
         * @return {?}
         */
        add => add.isLessThan(this, this.__getVal(value))));
    }
    /**
     * @param {?} value
     * @return {?}
     */
    __getVal(value) {
        if (value instanceof Column)
            return this.toRawValue(value.value);
        else
            return this.toRawValue(value);
    }
    /**
     * @return {?}
     */
    get value() {
        return this.fromRawValue(this.rawValue);
    }
    /**
     * @return {?}
     */
    get originalValue() {
        return this.fromRawValue(this.__valueProvider.getOriginalValue(this.jsonName));
    }
    /**
     * @return {?}
     */
    get displayValue() {
        if (this.value)
            return this.value.toString();
        return '';
    }
    /**
     * @protected
     * @param {?} value
     * @return {?}
     */
    __processValue(value) {
        return value;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    fromRawValue(value) {
        return value;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    toRawValue(value) {
        return value;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set rawValue(value) {
        this.__valueProvider.setValue(this.jsonName, this.__processValue(value));
        this.error = undefined;
        if (this.onValueChange)
            this.onValueChange();
    }
    /**
     * @return {?}
     */
    get rawValue() {
        return this.__valueProvider.getValue(this.jsonName);
    }
    /**
     * @return {?}
     */
    get inputValue() {
        return this.rawValue;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set inputValue(value) {
        this.rawValue = value;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set value(value) {
        this.rawValue = this.toRawValue(value);
    }
    /**
     * @param {?} pojo
     * @return {?}
     */
    __addToPojo(pojo) {
        pojo[this.jsonName] = this.rawValue;
    }
    /**
     * @param {?} pojo
     * @return {?}
     */
    __loadFromToPojo(pojo) {
        /** @type {?} */
        let x = pojo[this.jsonName];
        if (x != undefined)
            this.rawValue = x;
    }
}
class dummyColumnStorage {
    /**
     * @param {?} key
     * @return {?}
     */
    getValue(key) {
        return this._val;
    }
    /**
     * @param {?} key
     * @return {?}
     */
    getOriginalValue(key) {
        return this._val;
    }
    /**
     * @param {?} key
     * @param {?} value
     * @return {?}
     */
    setValue(key, value) {
        this._val = value;
    }
}
class Filter {
    /**
     * @param {?} apply
     */
    constructor(apply) {
        this.apply = apply;
    }
    /**
     * @param {?} filter
     * @return {?}
     */
    and(filter) {
        return new AndFilter(this, filter);
    }
    /**
     * @param {?} add
     * @return {?}
     */
    __applyToConsumer(add) {
        this.apply(add);
    }
}
class AndFilter {
    /**
     * @param {?} a
     * @param {?} b
     */
    constructor(a, b) {
        this.a = a;
        this.b = b;
    }
    /**
     * @param {?} filter
     * @return {?}
     */
    and(filter) {
        return new AndFilter(this, filter);
    }
    /**
     * @param {?} add
     * @return {?}
     */
    __applyToConsumer(add) {
        if (this.a)
            this.a.__applyToConsumer(add);
        if (this.b)
            this.b.__applyToConsumer(add);
    }
}
//@dynamic
/**
 * @template idType
 */
class Entity {
    /**
     * @param {?=} options
     * @param {?=} factory
     */
    constructor(options, factory) {
        this.factory = factory;
        //@internal
        this.__onSavingRow = (/**
         * @return {?}
         */
        () => { });
        //@internal
        this.__onValidate = (/**
         * @return {?}
         */
        () => { });
        //@internal
        this.__columns = [];
        if (!factory) {
            this.factory = (/**
             * @return {?}
             */
            () => this.__createInstance());
        }
        if (options) {
            if (typeof (options) === "string") {
                this.__options = { name: options };
            }
            else {
                this.__options = options;
                if (options.onSavingRow)
                    this.__onSavingRow = (/**
                     * @return {?}
                     */
                    () => options.onSavingRow());
                if (options.onValidate)
                    this.__onValidate = (/**
                     * @return {?}
                     */
                    () => options.onValidate(this));
            }
        }
        else {
            this.__options = {
                name: undefined
            };
        }
        this.__entityData = new __EntityValueProvider((/**
         * @return {?}
         */
        () => this.source.__getDataProvider()));
        this._noContextErrorWithStack = new Error('@EntityClass not used or context was not set for ' + this.constructor.name);
    }
    /**
     * @param {?} context
     * @return {?}
     */
    _setContext(context) {
        this.__context = context;
    }
    /**
     * @param {?} allowed
     * @return {?}
     */
    __getApiAllowUpdate(allowed) {
        if (!this.__context && isBoolean(allowed))
            return allowed;
        return this.__context.isAllowed(allowed);
    }
    /**
     * @param {?} entityType
     * @return {?}
     */
    _setFactoryClassAndDoInitColumns(entityType) {
        this.__entityType = entityType;
        this.__initColumns(((/** @type {?} */ (this))).id);
    }
    /**
     * @param {?} x
     * @param {?} context
     * @return {?}
     */
    _getExcludedColumns(x, context) {
        /** @type {?} */
        let r = x.__iterateColumns().filter((/**
         * @param {?} c
         * @return {?}
         */
        c => {
            return !context.isAllowed(c.includeInApi);
        }));
        return r;
    }
    /**
     * @param {?} r
     * @return {?}
     */
    _getEntityApiSettings(r) {
        /** @type {?} */
        let x = (/** @type {?} */ (r.for(this.__entityType).create()));
        /** @type {?} */
        let options = x.__options;
        if (options.allowApiCRUD) {
            options.allowApiDelete = true;
            options.allowApiInsert = true;
            options.allowApiUpdate = true;
        }
        return {
            allowRead: r.isAllowed(options.allowApiRead),
            allowUpdate: r.isAllowed(options.allowApiUpdate),
            allowDelete: r.isAllowed(options.allowApiDelete),
            allowInsert: r.isAllowed(options.allowApiInsert),
            excludeColumns: (/**
             * @param {?} x
             * @return {?}
             */
            x => this._getExcludedColumns(x, r)),
            readonlyColumns: (/**
             * @param {?} x
             * @return {?}
             */
            x => {
                return x.__iterateColumns().filter((/**
                 * @param {?} c
                 * @return {?}
                 */
                c => !r.isAllowed(c.allowApiUpdate)));
            }),
            get: {
                where: (/**
                 * @param {?} x
                 * @return {?}
                 */
                x => options.apiDataFilter ? options.apiDataFilter() : undefined)
            }
        };
    }
    //@internal
    /**
     * @return {?}
     */
    __createInstance() {
        if (!this.__context) {
            throw this._noContextErrorWithStack;
        }
        if (!this.__entityType) {
            throw this._noContextErrorWithStack;
        }
        return this.__context.create(this.__entityType);
    }
    /**
     * @return {?}
     */
    __getName() {
        return this.__options.name;
    }
    /**
     * @return {?}
     */
    __getDbName() {
        if (!this.__options.dbName)
            this.__options.dbName = this.__getName();
        return functionOrString(this.__options.dbName);
    }
    /**
     * @return {?}
     */
    __getCaption() {
        if (!this.__options.caption) {
            this.__options.caption = makeTitle(this.__getName());
        }
        return this.__options.caption;
    }
    /**
     * @param {?=} idColumn
     * @return {?}
     */
    __initColumns(idColumn) {
        if (idColumn)
            this.__idColumn = idColumn;
        /** @type {?} */
        let x = (/** @type {?} */ (this));
        for (let c in x) {
            /** @type {?} */
            let y = x[c];
            if (y instanceof Column) {
                if (!y.jsonName)
                    y.jsonName = c;
                if (!this.__idColumn && y.jsonName == 'id')
                    this.__idColumn = y;
                this.__applyColumn(y);
            }
        }
        if (!this.__idColumn)
            this.__idColumn = this.__iterateColumns()[0];
    }
    /**
     * @return {?}
     */
    isValid() {
        /** @type {?} */
        let ok = true;
        this.__iterateColumns().forEach((/**
         * @param {?} c
         * @return {?}
         */
        c => {
            if (c.error)
                ok = false;
        }));
        return ok;
    }
    /**
     * @return {?}
     */
    isNew() {
        return this.__entityData.isNewRow();
    }
    /**
     * @return {?}
     */
    __getValidationError() {
        /** @type {?} */
        let result = {};
        result.modelState = {};
        this.__iterateColumns().forEach((/**
         * @param {?} c
         * @return {?}
         */
        c => {
            if (c.error)
                result.modelState[c.jsonName] = c.error;
        }));
        return result;
    }
    /**
     * @param {?} dp
     * @return {?}
     */
    setSource(dp) {
        this.source = new EntitySource(this.__getName(), (/**
         * @template THIS
         * @this {THIS}
         * @return {THIS}
         */
        () => (/** @type {?} */ (this.factory()))), dp);
    }
    /**
     * @return {?}
     */
    __assertValidity() {
        if (!this.isValid()) {
            throw this.__getValidationError();
        }
    }
    /**
     * @param {?=} validate
     * @param {?=} onSavingRow
     * @return {?}
     */
    save(validate, onSavingRow) {
        this.__clearErrors();
        this.__iterateColumns().forEach((/**
         * @param {?} c
         * @return {?}
         */
        c => {
            c.__performValidation();
        }));
        if (this.__onValidate)
            this.__onValidate();
        if (validate)
            validate(this);
        this.__assertValidity();
        /** @type {?} */
        let performEntitySave = (/**
         * @return {?}
         */
        () => {
            /** @type {?} */
            let x = this.__onSavingRow();
            /** @type {?} */
            let doSave = (/**
             * @return {?}
             */
            () => {
                this.__assertValidity();
                return this.__entityData.save(this).catch((/**
                 * @param {?} e
                 * @return {?}
                 */
                e => this.catchSaveErrors(e)));
            });
            if (x instanceof Promise) {
                return x.then((/**
                 * @return {?}
                 */
                () => {
                    return doSave();
                }));
            }
            else {
                return doSave();
            }
        });
        if (!onSavingRow)
            return performEntitySave();
        /** @type {?} */
        let y = onSavingRow(this);
        if (y instanceof Promise) {
            return y.then((/**
             * @return {?}
             */
            () => { return performEntitySave(); }));
        }
        return performEntitySave();
    }
    /**
     * @param {?} err
     * @return {?}
     */
    catchSaveErrors(err) {
        /** @type {?} */
        let e = err;
        if (e instanceof Promise) {
            return e.then((/**
             * @param {?} x
             * @return {?}
             */
            x => this.catchSaveErrors(x)));
        }
        if (e.error) {
            e = e.error;
        }
        if (e.message)
            this.error = e.message;
        else if (e.Message)
            this.error = e.Message;
        else
            this.error = e;
        /** @type {?} */
        let s = e.modelState;
        if (!s)
            s = e.ModelState;
        if (s) {
            Object.keys(s).forEach((/**
             * @param {?} k
             * @return {?}
             */
            k => {
                /** @type {?} */
                let c = this.__getColumnByJsonName(k);
                if (c)
                    c.error = s[k];
            }));
        }
        throw err;
    }
    /**
     * @return {?}
     */
    delete() {
        return this.__entityData.delete().catch((/**
         * @param {?} e
         * @return {?}
         */
        e => this.catchSaveErrors(e)));
    }
    /**
     * @return {?}
     */
    reset() {
        this.__entityData.reset();
        this.__clearErrors();
    }
    //@internal
    /**
     * @return {?}
     */
    __clearErrors() {
        this.__iterateColumns().forEach((/**
         * @param {?} c
         * @return {?}
         */
        c => c.__clearErrors()));
        this.error = undefined;
    }
    /**
     * @return {?}
     */
    wasChanged() {
        return this.__entityData.wasChanged();
    }
    /**
     * @param {?} excludeColumns
     * @return {?}
     */
    __toPojo(excludeColumns) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            let r = {};
            yield Promise.all(this.__iterateColumns().map((/**
             * @param {?} c
             * @return {?}
             */
            (c) => __awaiter(this, void 0, void 0, function* () {
                yield c.__calcVirtuals();
            }))));
            this.__iterateColumns().forEach((/**
             * @param {?} c
             * @return {?}
             */
            c => {
                if (!excludeColumns.contains(c))
                    c.__addToPojo(r);
            }));
            return r;
        });
    }
    /**
     * @param {?} r
     * @param {?} excludeColumns
     * @return {?}
     */
    __fromPojo(r, excludeColumns) {
        this.__iterateColumns().forEach((/**
         * @param {?} c
         * @return {?}
         */
        c => {
            if (!excludeColumns.contains(c))
                c.__loadFromToPojo(r);
        }));
    }
    //@internal
    /**
     * @param {?} y
     * @return {?}
     */
    __applyColumn(y) {
        if (!y.caption)
            y.caption = makeTitle(y.jsonName);
        y.__valueProvider = this.__entityData;
        if (this.__columns.indexOf(y) < 0)
            this.__columns.push(y);
        y.__setEntity(this);
    }
    /**
     * @template T
     * @param {?} col
     * @return {?}
     */
    __getColumn(col) {
        return this.__getColumnByJsonName(col.jsonName);
    }
    /**
     * @param {?} key
     * @return {?}
     */
    __getColumnByJsonName(key) {
        /** @type {?} */
        let result;
        this.__iterateColumns().forEach((/**
         * @param {?} c
         * @return {?}
         */
        c => {
            if (c.jsonName == key)
                result = c;
        }));
        return result;
    }
    /**
     * @return {?}
     */
    __iterateColumns() {
        return this.__columns;
    }
}
class ColumnHashSet {
    constructor() {
        this._names = [];
    }
    /**
     * @param {...?} columns
     * @return {?}
     */
    add(...columns) {
        if (columns)
            for (let c of columns)
                this._names.push(c.__getMemberName());
    }
    /**
     * @param {?} column
     * @return {?}
     */
    contains(column) {
        return this._names.indexOf(column.__getMemberName()) >= 0;
    }
}
class CompoundIdColumn extends Column {
    /**
     * @param {?} entity
     * @param {...?} columns
     */
    constructor(entity, ...columns) {
        super();
        this.columns = columns;
    }
    /**
     * @return {?}
     */
    __isVirtual() { return true; }
    /**
     * @param {?} value
     * @return {?}
     */
    isEqualTo(value) {
        return new Filter((/**
         * @param {?} add
         * @return {?}
         */
        add => {
            /** @type {?} */
            let val = this.__getVal(value);
            /** @type {?} */
            let id = val.split(',');
            /** @type {?} */
            let result;
            this.columns.forEach((/**
             * @param {?} c
             * @param {?} i
             * @return {?}
             */
            (c, i) => {
                if (!result)
                    result = c.isEqualTo(id[i]);
                else
                    result = new AndFilter(result, c.isEqualTo(id[i]));
            }));
            return result.__applyToConsumer(add);
        }));
    }
    /**
     * @param {?} p
     * @return {?}
     */
    __addIdToPojo(p) {
        if (p.id)
            return;
        /** @type {?} */
        let r = "";
        this.columns.forEach((/**
         * @param {?} c
         * @return {?}
         */
        c => {
            if (r.length > 0)
                r += ',';
            r += p[c.jsonName];
        }));
        p.id = r;
    }
    /**
     * @param {?} id
     * @param {?} data
     * @return {?}
     */
    resultIdFilter(id, data) {
        return new Filter((/**
         * @param {?} add
         * @return {?}
         */
        add => {
            /** @type {?} */
            let idParts = [];
            if (id != undefined)
                idParts = id.split(',');
            /** @type {?} */
            let result;
            this.columns.forEach((/**
             * @param {?} c
             * @param {?} i
             * @return {?}
             */
            (c, i) => {
                /** @type {?} */
                let val = undefined;
                if (i < idParts.length)
                    val = idParts[i];
                if (data[c.jsonName] != undefined)
                    val = data[c.jsonName];
                if (!result)
                    result = c.isEqualTo(val);
                else
                    result = new AndFilter(result, c.isEqualTo(val));
            }));
            return result.__applyToConsumer(add);
        }));
    }
}
/**
 * @template T
 */
class EntitySource {
    /**
     * @param {?} name
     * @param {?} factory
     * @param {?} dataProvider
     */
    constructor(name, factory, dataProvider) {
        this.factory = factory;
        if (dataProvider)
            this._provider = dataProvider.provideFor(name, factory);
    }
    /**
     * @param {?=} options
     * @return {?}
     */
    find(options) {
        if (options)
            options.orderBy = translateSort(options.orderBy);
        return this._provider.find((/** @type {?} */ (options)))
            .then((/**
         * @param {?} arr
         * @return {?}
         */
        arr => {
            return arr.map((/**
             * @param {?} i
             * @return {?}
             */
            i => {
                /** @type {?} */
                let r = this.factory();
                r.__entityData.setData(i, r);
                r.source = this;
                return r;
            }));
        }));
    }
    /**
     * @param {?} r
     * @return {?}
     */
    fromPojo(r) {
        /** @type {?} */
        let f = this.factory();
        f.__entityData.setData(r, f);
        f.source = this;
        return f;
    }
    /**
     * @param {?=} where
     * @return {?}
     */
    count(where) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._provider.count(where);
        });
    }
    /**
     * @param {?} col
     * @param {?=} filter
     * @return {?}
     */
    max(col, filter) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            let x = yield this.find({ where: filter, limit: 1, orderBy: new Sort({ column: col, descending: true }) });
            if (x.length == 0) {
                return 0;
            }
            return x[0].__getColumn(col).value;
        });
    }
    /**
     * @return {?}
     */
    __getDataProvider() {
        return this._provider;
    }
    /**
     * @return {?}
     */
    createNewItem() {
        /** @type {?} */
        let r = this.factory();
        r.source = this;
        return r;
    }
    /**
     * @param {?} doOnRow
     * @return {?}
     */
    Insert(doOnRow) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            var i = this.createNewItem();
            /** @type {?} */
            let x = doOnRow(i);
            if (x instanceof Promise)
                yield x;
            yield i.save();
            return i;
        });
    }
}
class __EntityValueProvider {
    /**
     * @param {?} getDataProvider
     */
    constructor(getDataProvider) {
        this.getDataProvider = getDataProvider;
        this.listeners = [];
        this.newRow = true;
        this.data = {};
        this.originalData = {};
    }
    /**
     * @param {?} listener
     * @return {?}
     */
    register(listener) {
        this.listeners.push(listener);
    }
    /**
     * @return {?}
     */
    delete() {
        return this.getDataProvider().delete(this.id).then((/**
         * @return {?}
         */
        () => {
            this.listeners.forEach((/**
             * @param {?} x
             * @return {?}
             */
            x => {
                if (x.rowDeleted)
                    x.rowDeleted();
            }));
        }));
    }
    /**
     * @return {?}
     */
    isNewRow() {
        return this.newRow;
    }
    /**
     * @return {?}
     */
    wasChanged() {
        return JSON.stringify(this.originalData) != JSON.stringify(this.data) || this.newRow;
    }
    /**
     * @return {?}
     */
    reset() {
        this.data = JSON.parse(JSON.stringify(this.originalData));
        this.listeners.forEach((/**
         * @param {?} x
         * @return {?}
         */
        x => {
            if (x.rowReset)
                x.rowReset(this.newRow);
        }));
    }
    /**
     * @param {?} e
     * @return {?}
     */
    save(e) {
        /** @type {?} */
        let d = JSON.parse(JSON.stringify(this.data));
        if (e.__idColumn instanceof CompoundIdColumn)
            d.id = undefined;
        if (this.newRow) {
            return this.getDataProvider().insert(d).then((/**
             * @param {?} newData
             * @return {?}
             */
            (newData) => {
                this.setData(newData, e);
                this.listeners.forEach((/**
                 * @param {?} x
                 * @return {?}
                 */
                x => {
                    if (x.rowSaved)
                        x.rowSaved(true);
                }));
            }));
        }
        else {
            return this.getDataProvider().update(this.id, d).then((/**
             * @param {?} newData
             * @return {?}
             */
            (newData) => {
                this.setData(newData, e);
                this.listeners.forEach((/**
                 * @param {?} x
                 * @return {?}
                 */
                x => {
                    if (x.rowSaved)
                        x.rowSaved(false);
                }));
            }));
        }
    }
    /**
     * @param {?} data
     * @param {?} r
     * @return {?}
     */
    setData(data, r) {
        if (!data)
            data = {};
        if (r.__idColumn instanceof CompoundIdColumn) {
            r.__idColumn.__addIdToPojo(data);
        }
        /** @type {?} */
        let id = data[r.__idColumn.jsonName];
        if (id != undefined) {
            this.id = id;
            this.newRow = false;
        }
        this.data = data;
        this.originalData = JSON.parse(JSON.stringify(this.data));
    }
    /**
     * @param {?} key
     * @return {?}
     */
    getValue(key) {
        return this.data[key];
    }
    /**
     * @param {?} key
     * @return {?}
     */
    getOriginalValue(key) {
        return this.originalData[key];
    }
    /**
     * @param {?} key
     * @param {?} value
     * @return {?}
     */
    setValue(key, value) {
        this.data[key] = value;
    }
}
class StringColumn extends Column {
    /**
     * @param {?=} settingsOrCaption
     */
    constructor(settingsOrCaption) {
        super(settingsOrCaption);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    isContains(value) {
        return new Filter((/**
         * @param {?} add
         * @return {?}
         */
        add => add.isContains(this, this.__getVal(value))));
    }
    /**
     * @param {?} value
     * @return {?}
     */
    isStartsWith(value) {
        return new Filter((/**
         * @param {?} add
         * @return {?}
         */
        add => add.isStartsWith(this, this.__getVal(value))));
    }
}
class DateColumn extends Column {
    /**
     * @param {?=} settingsOrCaption
     */
    constructor(settingsOrCaption) {
        super(settingsOrCaption);
        if (!this.inputType)
            this.inputType = 'date';
    }
    /**
     * @return {?}
     */
    getDayOfWeek() {
        return new Date(this.value).getDay();
    }
    /**
     * @return {?}
     */
    get displayValue() {
        if (!this.value)
            return '';
        return this.value.toLocaleDateString();
    }
    /**
     * @return {?}
     */
    __defaultStorage() {
        return new DateTimeDateStorage();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    toRawValue(value) {
        return DateColumn.dateToString(value);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    fromRawValue(value) {
        return DateColumn.stringToDate(value);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    static stringToDate(value) {
        if (!value || value == '' || value == '0000-00-00')
            return undefined;
        return new Date(Date.parse(value));
    }
    /**
     * @param {?} val
     * @return {?}
     */
    static dateToString(val) {
        /** @type {?} */
        var d = (/** @type {?} */ (val));
        if (!d)
            return '';
        /** @type {?} */
        let month = addZeros(d.getMonth() + 1);
        /** @type {?} */
        let day = addZeros(d.getDate());
        /** @type {?} */
        let year = d.getFullYear();
        return [year, month, day].join('-');
    }
}
class DateTimeColumn extends Column {
    /**
     * @param {?=} settingsOrCaption
     */
    constructor(settingsOrCaption) {
        super(settingsOrCaption);
        if (!this.inputType)
            this.inputType = 'date';
    }
    /**
     * @return {?}
     */
    getDayOfWeek() {
        return this.value.getDay();
    }
    /**
     * @return {?}
     */
    get displayValue() {
        if (!this.value)
            return '';
        return this.value.toLocaleString();
    }
    /**
     * @return {?}
     */
    __defaultStorage() {
        return new DateTimeStorage();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    fromRawValue(value) {
        return DateTimeColumn.stringToDate(value);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    toRawValue(value) {
        return DateTimeColumn.dateToString(value);
    }
    /**
     * @param {?} val
     * @return {?}
     */
    static stringToDate(val) {
        if (val == undefined)
            return undefined;
        if (val == '')
            return undefined;
        if (val.startsWith('0000-00-00'))
            return undefined;
        return new Date(Date.parse(val));
    }
    /**
     * @param {?} val
     * @return {?}
     */
    static dateToString(val) {
        /** @type {?} */
        var d = (/** @type {?} */ (val));
        if (!d)
            return '';
        return d.toISOString();
    }
}
class NumberColumn extends Column {
    /**
     * @param {?=} settingsOrCaption
     */
    constructor(settingsOrCaption) {
        super(settingsOrCaption);
        this.__numOfDecimalDigits = 0;
        if (!this.inputType)
            this.inputType = 'number';
        /** @type {?} */
        let s = (/** @type {?} */ (settingsOrCaption));
        if (s && s.decimalDigits) {
            this.__numOfDecimalDigits = s.decimalDigits;
        }
    }
    /**
     * @protected
     * @param {?} value
     * @return {?}
     */
    __processValue(value) {
        if (value != undefined && !(typeof value === "number"))
            return +value;
        return value;
    }
}
class BoolColumn extends Column {
    /**
     * @param {?=} settingsOrCaption
     */
    constructor(settingsOrCaption) {
        super(settingsOrCaption);
        if (!this.inputType)
            this.inputType = 'checkbox';
    }
    /**
     * @return {?}
     */
    __defaultStorage() {
        return new BoolStorage();
    }
}
class BoolStorage {
    /**
     * @param {?} val
     * @return {?}
     */
    toDb(val) {
        return val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    fromDb(val) {
        if (isString(val))
            return val === "true";
        return val;
    }
}
/**
 * @template closedListType
 */
class ClosedListColumn extends Column {
    /**
     * @param {?} closedListType
     * @param {?=} settingsOrCaption
     */
    constructor(closedListType, settingsOrCaption) {
        super(settingsOrCaption);
        this.closedListType = closedListType;
    }
    /**
     * @return {?}
     */
    getOptions() {
        /** @type {?} */
        let result = [];
        for (let member in this.closedListType) {
            /** @type {?} */
            let s = (/** @type {?} */ (this.closedListType[member]));
            if (s && s.id != undefined) {
                result.push({
                    id: s.id,
                    caption: s.toString()
                });
            }
        }
        return result;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    toRawValue(value) {
        return value.id;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    fromRawValue(value) {
        return this.byId(+value);
    }
    /**
     * @return {?}
     */
    get displayValue() {
        if (this.value)
            return this.value.toString();
        return '';
    }
    /**
     * @param {?} id
     * @return {?}
     */
    byId(id) {
        for (let member in this.closedListType) {
            /** @type {?} */
            let s = (/** @type {?} */ (this.closedListType[member]));
            if (s && s.id == id)
                return s;
        }
        return undefined;
    }
}
/**
 * @template rowType
 */
class ColumnCollection {
    /**
     * @param {?} currentRow
     * @param {?} allowUpdate
     * @param {?} filterHelper
     * @param {?} showArea
     */
    constructor(currentRow, allowUpdate, filterHelper, showArea) {
        this.currentRow = currentRow;
        this.allowUpdate = allowUpdate;
        this.filterHelper = filterHelper;
        this.showArea = showArea;
        this.settingsByKey = {};
        this.designMode = false;
        this._colListChangeListeners = [];
        this.items = [];
        this.numOfColumnsInGrid = 5;
        if (this.allowDesignMode == undefined) {
            if (location.search)
                if (location.search.toLowerCase().indexOf('design=y') >= 0)
                    this.allowDesignMode = true;
        }
    }
    /**
     * @return {?}
     */
    __showArea() {
        return this.showArea();
    }
    /**
     * @param {?} map
     * @param {?} record
     * @return {?}
     */
    __getColumn(map, record) {
        /** @type {?} */
        let result;
        if (record)
            result = record.__getColumn(map.column);
        if (!result)
            result = map.column;
        return result;
    }
    /**
     * @param {?} map
     * @return {?}
     */
    __dataControlStyle(map) {
        if (map.width && map.width.trim().length > 0) {
            if ((+map.width).toString() == map.width)
                return map.width + "px";
            return map.width;
        }
        return undefined;
    }
    /**
     * @param {...?} columns
     * @return {?}
     */
    add(...columns) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            var promises = [];
            for (let c of columns) {
                /** @type {?} */
                let x = (/** @type {?} */ (c));
                if (!x.column && c instanceof Column) {
                    x = {
                        column: c,
                    };
                }
                if (x.column) {
                    x.column.__decorateDataSettings(x);
                }
                if (x.getValue) ;
                else {
                    promises.push(this.buildDropDown(x));
                }
                this.items.push(x);
            }
            yield Promise.all(promises);
            return Promise.resolve();
        });
    }
    /**
     * @param {?} s
     * @return {?}
     */
    buildDropDown(s) {
        return __awaiter(this, void 0, void 0, function* () {
            if (s.dropDown) {
                /** @type {?} */
                let orig = s.dropDown.items;
                /** @type {?} */
                let result = [];
                s.dropDown.items = result;
                /** @type {?} */
                let populateBasedOnArray = (/**
                 * @param {?} arr
                 * @return {?}
                 */
                (arr) => {
                    for (let item of arr) {
                        /** @type {?} */
                        let type = typeof (item);
                        if (type == "string" || type == "number")
                            result.push({ id: item, caption: item });
                        else if (item instanceof Entity) {
                            /** @type {?} */
                            let col;
                            if (!s.dropDown.idColumn) {
                                if (col = item.__getColumnByJsonName('id'))
                                    s.dropDown.idColumn = col;
                                else {
                                    for (let colInEntity of item.__iterateColumns()) {
                                        s.dropDown.idColumn = colInEntity;
                                        break;
                                    }
                                }
                            }
                            if (!s.dropDown.captionColumn) {
                                if (col = item.__getColumnByJsonName('caption'))
                                    s.dropDown.captionColumn = col;
                                else {
                                    for (let keyInItem of item.__iterateColumns()) {
                                        if (keyInItem != item.__getColumn(s.dropDown.idColumn)) {
                                            s.dropDown.captionColumn = keyInItem;
                                            break;
                                        }
                                    }
                                }
                            }
                            /** @type {?} */
                            let p = { id: item.__getColumn(s.dropDown.idColumn).value, caption: item.__getColumn(s.dropDown.captionColumn).value };
                            if (p.id instanceof Column) {
                                p.id = p.id.value;
                            }
                            if (p.caption instanceof Column)
                                p.caption = p.caption.value;
                            if (!p.caption)
                                p.caption = p.id;
                            result.push(p);
                        }
                        else {
                            /** @type {?} */
                            let x = (/** @type {?} */ (item));
                            if (x && x.id != undefined) {
                                result.push(x);
                            }
                        }
                    }
                });
                if (orig instanceof Array) {
                    populateBasedOnArray(orig);
                }
                else if (s.dropDown.source) {
                    if (s.dropDown.source instanceof Entity) {
                        return new DataList(s.dropDown.source).get({ limit: 5000, orderBy: s.dropDown.orderBy }).then((/**
                         * @param {?} arr
                         * @return {?}
                         */
                        arr => populateBasedOnArray(arr)));
                    }
                }
            }
            return Promise.resolve();
        });
    }
    /**
     * @return {?}
     */
    colListChanged() {
        this._lastNumOfColumnsInGrid = -1;
        this._colListChangeListeners.forEach((/**
         * @param {?} x
         * @return {?}
         */
        x => x()));
    }
    ;
    /**
     * @param {?} action
     * @return {?}
     */
    onColListChange(action) {
        this._colListChangeListeners.push(action);
    }
    /**
     * @param {?} col
     * @param {?} move
     * @return {?}
     */
    moveCol(col, move) {
        /** @type {?} */
        let currentIndex = this.items.indexOf(col);
        /** @type {?} */
        let newIndex = currentIndex + move;
        if (newIndex < 0 || newIndex >= this.items.length)
            return;
        this.items.splice(currentIndex, 1);
        this.items.splice(newIndex, 0, col);
        this.colListChanged();
    }
    /**
     * @param {?} col
     * @return {?}
     */
    filterRows(col) {
        col._showFilter = false;
        this.filterHelper.filterColumn(col.column, false, (col.dropDown != undefined || col.click != undefined));
    }
    /**
     * @param {?} col
     * @return {?}
     */
    clearFilter(col) {
        col._showFilter = false;
        this.filterHelper.filterColumn(col.column, true, false);
    }
    /**
     * @param {?} col
     * @return {?}
     */
    _shouldShowFilterDialog(col) {
        return col && col._showFilter;
    }
    /**
     * @param {?} col
     * @return {?}
     */
    showFilterDialog(col) {
        col._showFilter = !col._showFilter;
    }
    /**
     * @param {?} col
     * @return {?}
     */
    deleteCol(col) {
        this.items.splice(this.items.indexOf(col), 1);
        this.colListChanged();
    }
    /**
     * @param {?} col
     * @return {?}
     */
    addCol(col) {
        this.items.splice(this.items.indexOf(col) + 1, 0, { designMode: true });
        this.colListChanged();
    }
    /**
     * @param {?} col
     * @return {?}
     */
    designColumn(col) {
        col.designMode = !col.designMode;
    }
    /**
     * @param {?} col
     * @return {?}
     */
    _getEditable(col) {
        if (!this.allowUpdate())
            return false;
        if (!col.column)
            return false;
        return !col.readonly;
    }
    /**
     * @param {?} col
     * @param {?} row
     * @return {?}
     */
    _click(col, row) {
        col.click(row, (/**
         * @param {?} what
         * @return {?}
         */
        what => {
            what();
        }));
    }
    /**
     * @param {?} col
     * @param {?} row
     * @return {?}
     */
    _getColDisplayValue(col, row) {
        /** @type {?} */
        let r;
        if (col.getValue) {
            r = col.getValue(row);
            if (r instanceof Column)
                r = r.value;
        }
        else if (col.column) {
            if (col.dropDown && col.dropDown.items) {
                for (let x of col.dropDown.items) {
                    if (x.id == this.__getColumn(col, row).value)
                        return x.caption;
                }
            }
            r = this.__getColumn(col, row).displayValue;
        }
        return r;
    }
    /**
     * @param {?} col
     * @return {?}
     */
    _getColDataType(col) {
        if (col.inputType)
            return col.inputType;
        return "text";
    }
    /**
     * @param {?} col
     * @param {?} row
     * @return {?}
     */
    _getColumnClass(col, row) {
        if (col.cssClass)
            if (isFunction$1(col.cssClass)) {
                /** @type {?} */
                let anyFunc = col.cssClass;
                return anyFunc(row);
            }
            else
                return col.cssClass;
        return '';
    }
    /**
     * @param {?} col
     * @param {?} r
     * @return {?}
     */
    _getError(col, r) {
        if (!col.column)
            return undefined;
        return this.__getColumn(col, r).error;
    }
    /**
     * @param {?} r
     * @return {?}
     */
    autoGenerateColumnsBasedOnData(r) {
        if (this.items.length == 0) {
            if (r) {
                this.add(...r.__iterateColumns());
            }
        }
    }
    /**
     * @return {?}
     */
    __columnSettingsTypeScript() {
        /** @type {?} */
        let memberName = 'x';
        if (this.currentRow())
            memberName = this.currentRow().__getName();
        memberName = memberName[0].toLocaleLowerCase() + memberName.substring(1);
        /** @type {?} */
        let result = '';
        this.items.forEach((/**
         * @param {?} c
         * @return {?}
         */
        c => {
            if (result.length > 0)
                result += ',\n';
            result += '  ' + this.__columnTypeScriptDescription(c, memberName);
        }));
        result = `columnSettings: ${memberName} => [\n` + result + "\n]";
        return result;
    }
    /**
     * @param {?} c
     * @param {?} memberName
     * @return {?}
     */
    __columnTypeScriptDescription(c, memberName) {
        /** @type {?} */
        let properties = "";
        /**
         * @param {?} name
         * @param {?} value
         * @return {?}
         */
        function addToProperties(name, value) {
            if (properties.length > 0)
                properties += ', ';
            properties += "\n    " + name + ": " + value;
        }
        /**
         * @param {?} name
         * @param {?} value
         * @return {?}
         */
        function addString(name, value) {
            addToProperties(name, "'" + value + "'");
        }
        /** @type {?} */
        let columnMember = '';
        if (c.column) {
            columnMember += memberName + "." + c.column.__getMemberName();
            if (c == c.column)
                columnMember += '/*equal*/';
            if (c.caption != c.column.caption) {
                addString('caption', c.caption);
            }
        }
        else {
            addString('caption', c.caption);
        }
        if (c.width && c.width.length > 0)
            addString('width', c.width);
        if (properties.length > 0) {
            if (columnMember != '') {
                properties = '\n    column: ' + columnMember + ', ' + properties;
            }
        }
        /** @type {?} */
        let whatToAdd = '';
        if (properties.length > 0)
            whatToAdd = "{" + properties + "\n  }";
        else if (columnMember != '')
            whatToAdd = columnMember;
        return whatToAdd;
    }
    /**
     * @param {?} col
     * @param {?} what
     * @return {?}
     */
    __changeWidth(col, what) {
        /** @type {?} */
        let width = col.width;
        if (!width)
            width = '50';
        width = ((+width) + what).toString();
        col.width = width;
    }
    /**
     * @param {?} col
     * @param {?} r
     * @return {?}
     */
    _colValueChanged(col, r) {
        if (col.onUserChangedValue)
            col.onUserChangedValue(r);
    }
    /**
     * @private
     * @return {?}
     */
    _initColumnsArrays() {
        if (this._lastColumnCount != this.items.length || this._lastNumOfColumnsInGrid != this.numOfColumnsInGrid) {
            this._lastNumOfColumnsInGrid = this.numOfColumnsInGrid;
            this._lastColumnCount = this.items.length;
            this.gridColumns = [];
            this.nonGridColumns = [];
            /** @type {?} */
            let i = 0;
            for (let c of this.items) {
                if (i++ < this._lastNumOfColumnsInGrid)
                    this.gridColumns.push(c);
                else
                    this.nonGridColumns.push(c);
            }
        }
    }
    /**
     * @return {?}
     */
    getGridColumns() {
        this._initColumnsArrays();
        return this.gridColumns;
    }
    /**
     * @return {?}
     */
    getNonGridColumns() {
        this._initColumnsArrays();
        return this.nonGridColumns;
    }
}
/**
 * @template T
 * @param {?} entity
 * @param {?} opt
 * @return {?}
 */
function extractSortFromSettings(entity, opt) {
    if (!opt)
        return undefined;
    if (!opt.orderBy)
        return undefined;
    /** @type {?} */
    let x = opt.orderBy(entity);
    return translateSort(x);
}
/**
 * @param {?} sort
 * @return {?}
 */
function translateSort(sort) {
    if (sort instanceof Sort)
        return sort;
    if (sort instanceof Column)
        return new Sort({ column: sort });
    if (sort instanceof Array) {
        /** @type {?} */
        let r = new Sort();
        sort.forEach((/**
         * @param {?} i
         * @return {?}
         */
        i => {
            if (i instanceof Column)
                r.Segments.push({ column: i });
            else
                r.Segments.push(i);
        }));
        return r;
    }
}
class FilterConsumerBridgeToSqlRequest {
    /**
     * @param {?} r
     */
    constructor(r) {
        this.r = r;
        this.where = "";
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isEqualTo(col, val) {
        this.add(col, val, "=");
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isDifferentFrom(col, val) {
        this.add(col, val, "<>");
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isGreaterOrEqualTo(col, val) {
        this.add(col, val, ">=");
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isGreaterThan(col, val) {
        this.add(col, val, ">");
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isLessOrEqualTo(col, val) {
        this.add(col, val, "<=");
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isLessThan(col, val) {
        this.add(col, val, "<");
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isContains(col, val) {
        this.add(col, '%' + val + '%', 'like');
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isStartsWith(col, val) {
        this.add(col, val + '%', 'like');
    }
    /**
     * @private
     * @param {?} col
     * @param {?} val
     * @param {?} operator
     * @return {?}
     */
    add(col, val, operator) {
        if (this.where.length == 0) {
            this.where += ' where ';
        }
        else
            this.where += ' and ';
        this.where += col.__getDbName() + ' ' + operator + ' ' + this.r.addParameterToCommandAndReturnParameterName(col, val);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DataControlComponent {
    constructor() {
        this.settings = new ColumnCollection(undefined, (/**
         * @return {?}
         */
        () => true), undefined, undefined);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set column(value) {
        this.map = {
            column: value
        };
    }
    /**
     * @return {?}
     */
    showDescription() {
        return (this.map.column) && this.map.getValue;
    }
    /**
     * @return {?}
     */
    dataControlStyle() {
        return this.settings.__dataControlStyle(this.map);
    }
    /**
     * @return {?}
     */
    _getColumn() {
        return this.settings.__getColumn(this.map, this.record);
    }
    /**
     * @return {?}
     */
    _getEditable() {
        if (this.notReadonly)
            return true;
        return this.settings._getEditable(this.map);
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
    }
    /**
     * @return {?}
     */
    isSelect() {
        if (this.map.dropDown)
            return true;
        return false;
    }
    /**
     * @return {?}
     */
    showTextBox() {
        return !this.isSelect() && !this.showCheckbox();
    }
    /**
     * @return {?}
     */
    showCheckbox() {
        return this.settings._getColDataType(this.map) == 'checkbox';
    }
}
DataControlComponent.decorators = [
    { type: Component, args: [{
                selector: 'data-control',
                template: `
<span *ngIf="!_getEditable()" >{{settings._getColDisplayValue(map,record)}}</span>
<div *ngIf="_getEditable()" class="" [class.has-error]="settings._getError(map,record)">
    <div >
        <div [class.input-group]="showDescription()||map.click" *ngIf="showTextBox()" style="width:100%">
            <div class="input-group-btn" *ngIf="map.click">
                <button type="button" class="btn btn-default" (click)="settings._click(map,record)" > <span class="glyphicon glyphicon-chevron-down"></span></button>
            </div>

            <input class="form-control"  [(ngModel)]="_getColumn().inputValue" type="{{settings._getColDataType(map)}}" (ngModelChange)="settings._colValueChanged(map,record)" *ngIf="!map.hideDataOnInput" />
            <div class="input-group-addon" style="width:60%;text-align:start" *ngIf="showDescription()">{{settings._getColDisplayValue(map,record)}}</div>

        </div>
        <div *ngIf="isSelect()">
            <select  class="form-control" [(ngModel)]="_getColumn().inputValue" (ngModelChange)="settings._colValueChanged(map,record)" >
                <option *ngFor="let v of map.dropDown.items" value="{{v.id}}">{{v.caption}}</option>

            </select>
        </div>
        <div *ngIf="showCheckbox()" style="width:100%">
        <input class="form-control"  [(ngModel)]="_getColumn().inputValue" type="checkbox" (ngModelChange)="settings._colValueChanged(map,record)" *ngIf="!map.hideDataOnInput" />
        

    </div>
    <span class="help-block" *ngIf="settings._getError(map,record)">{{settings._getError(map,record)}}</span>

    </div>
</div>`
            }] }
];
DataControlComponent.propDecorators = {
    map: [{ type: Input }],
    column: [{ type: Input }],
    record: [{ type: Input }],
    notReadonly: [{ type: Input }],
    settings: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DataControl2Component {
    constructor() {
        this.settings = new ColumnCollection(undefined, (/**
         * @return {?}
         */
        () => true), undefined, undefined);
        this.ngErrorStateMatches = new class extends ErrorStateMatcher {
            /**
             * @param {?} parent
             */
            constructor(parent) {
                super();
                this.parent = parent;
            }
            /**
             * @return {?}
             */
            isErrorState() {
                return !!this.parent.getError();
            }
        }(this);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set column(value) {
        this.map = {
            column: value
        };
        this.map.column.__decorateDataSettings(this.map);
    }
    /**
     * @return {?}
     */
    showDescription() {
        return (this.map.column) && this.map.getValue || !this._getEditable();
    }
    /**
     * @return {?}
     */
    showClick() {
        if (!this.map.click)
            return false;
        if (this.map.allowClick === undefined) {
            return true;
        }
        return this.map.allowClick(this.record);
    }
    /**
     * @return {?}
     */
    getClickIcon() {
        if (this.map.clickIcon)
            return this.map.clickIcon;
        return 'keyboard_arrow_down';
    }
    /**
     * @return {?}
     */
    dataControlStyle() {
        return this.settings.__dataControlStyle(this.map);
    }
    /**
     * @return {?}
     */
    _getColumn() {
        if (!this.map.column)
            return new StringColumn();
        return this.settings.__getColumn(this.map, this.record);
    }
    /**
     * @return {?}
     */
    _getEditable() {
        if (this.notReadonly)
            return true;
        return this.settings._getEditable(this.map);
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
    }
    /**
     * @return {?}
     */
    isSelect() {
        if (this.map.dropDown)
            return true;
        return false;
    }
    /**
     * @return {?}
     */
    showTextBox() {
        return !this.isSelect() && !this.showCheckbox();
    }
    /**
     * @return {?}
     */
    showCheckbox() {
        return this.settings._getColDataType(this.map) == 'checkbox';
    }
    /**
     * @return {?}
     */
    getError() {
        return this.settings._getError(this.map, this.record);
    }
    /**
     * @return {?}
     */
    getStyle() {
        if (this.showDescription()) {
            if (this.map.hideDataOnInput || !this._getEditable()) {
                return { display: 'none' };
            }
            return { width: '50px' };
        }
        return {};
    }
    /**
     * @return {?}
     */
    getFloatLabel() {
        if (this.showDescription()) {
            if (this.settings._getColDisplayValue(this.map, this.record))
                return 'always';
        }
        return '';
    }
}
DataControl2Component.decorators = [
    { type: Component, args: [{
                selector: 'data-control2',
                template: "\r\n<div class=\"data-control2\">\r\n    <mat-form-field appearance=\"outline\" [floatLabel]=\"getFloatLabel()\" *ngIf=\"!showCheckbox()\" class=\"full-width-form-field dense-form-field\">\r\n        <mat-label>{{map.caption}}</mat-label>\r\n        <div style=\"display:flex;\" *ngIf=\"showTextBox()\">\r\n            <input #theId matInput [(ngModel)]=\"_getColumn().inputValue\" type=\"{{settings._getColDataType(map)}}\"\r\n                (ngModelChange)=\"settings._colValueChanged(map,record)\" [errorStateMatcher]=\"ngErrorStateMatches\"\r\n                [ngStyle]=\"getStyle()\" />\r\n            <div *ngIf=\"showDescription()\" style=\"white-space: nowrap;\r\n            overflow: hidden;\r\n            text-overflow: ellipsis;\r\n            padding-left:10px;\">\r\n                {{settings._getColDisplayValue(map,record)}}&nbsp;</div>\r\n        </div>\r\n        <div *ngIf=\"isSelect()\">\r\n            <select matNativeControl [(ngModel)]=\"_getColumn().inputValue\"\r\n                (ngModelChange)=\"settings._colValueChanged(map,record)\">\r\n                <option *ngFor=\"let v of map.dropDown.items\" value=\"{{v.id}}\">{{v.caption}}</option>\r\n\r\n            </select>\r\n        </div>\r\n\r\n        <button mat-icon-button matSuffix *ngIf=\"showClick()\" (click)=\"settings._click(map,record)\">\r\n            <mat-icon>{{getClickIcon()}}  </mat-icon>\r\n        </button>\r\n\r\n        <mat-error *ngIf=\"getError()\" [id]=\"theId\">{{getError()}}</mat-error>\r\n    </mat-form-field>\r\n    <div *ngIf=\"showCheckbox()\">\r\n        <mat-checkbox #theId1 [(ngModel)]=\"_getColumn().inputValue\"\r\n            (ngModelChange)=\"settings._colValueChanged(map,record)\" class=\"fix-label\">{{map.caption}}</mat-checkbox>\r\n        <mat-error *ngIf=\"getError()\" [id]=\"theId1\">{{getError()}}</mat-error>\r\n    </div>\r\n\r\n</div>",
                styles: [""]
            }] }
];
DataControl2Component.propDecorators = {
    map: [{ type: Input }],
    column: [{ type: Input }],
    record: [{ type: Input }],
    notReadonly: [{ type: Input }],
    settings: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DataControl3Component {
    constructor() {
        this.rightToLeft = false;
        this.settings = new ColumnCollection(undefined, (/**
         * @return {?}
         */
        () => true), undefined, undefined);
        this.ngErrorStateMatches = new class extends ErrorStateMatcher {
            /**
             * @param {?} parent
             */
            constructor(parent) {
                super();
                this.parent = parent;
            }
            /**
             * @return {?}
             */
            isErrorState() {
                return !!this.parent.getError();
            }
        }(this);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set column(value) {
        this.map = {
            column: value
        };
        this.map.column.__decorateDataSettings(this.map);
    }
    /**
     * @return {?}
     */
    showDescription() {
        return (this.map.column) && this.map.getValue || !this._getEditable();
    }
    /**
     * @return {?}
     */
    showClick() {
        if (!this.map.click)
            return false;
        if (this.map.allowClick === undefined) {
            return true;
        }
        return this.map.allowClick(this.record);
    }
    /**
     * @return {?}
     */
    getClickIcon() {
        if (this.map.clickIcon)
            return this.map.clickIcon;
        return 'keyboard_arrow_down';
    }
    /**
     * @return {?}
     */
    dataControlStyle() {
        return this.settings.__dataControlStyle(this.map);
    }
    /**
     * @return {?}
     */
    _getColumn() {
        if (!this.map.column)
            return new StringColumn();
        return this.settings.__getColumn(this.map, this.record);
    }
    /**
     * @return {?}
     */
    _getEditable() {
        if (this.notReadonly)
            return true;
        return this.settings._getEditable(this.map);
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
    }
    /**
     * @return {?}
     */
    isSelect() {
        if (this.map.dropDown)
            return true;
        return false;
    }
    /**
     * @return {?}
     */
    showTextBox() {
        return !this.isSelect() && !this.showCheckbox();
    }
    /**
     * @return {?}
     */
    showCheckbox() {
        return this.settings._getColDataType(this.map) == 'checkbox';
    }
    /**
     * @return {?}
     */
    getError() {
        return this.settings._getError(this.map, this.record);
    }
    /**
     * @return {?}
     */
    getStyle() {
        if (this.showDescription()) {
            if (this.map.hideDataOnInput || !this._getEditable()) {
                return { display: 'none' };
            }
            return { width: '50px' };
        }
        return {
            width: '100%'
        };
    }
    /**
     * @return {?}
     */
    getFloatLabel() {
        if (this.showDescription()) {
            if (this.settings._getColDisplayValue(this.map, this.record))
                return 'always';
        }
        return '';
    }
}
DataControl3Component.decorators = [
    { type: Component, args: [{
                selector: 'data-control3',
                template: "\r\n<div class=\"data-control3\" #myDiv >\r\n        \r\n        <div style=\"display:flex;width:100%;\" *ngIf=\"showTextBox()\">\r\n            <input #theId matInput [(ngModel)]=\"_getColumn().inputValue\" type=\"{{settings._getColDataType(map)}}\"\r\n                (ngModelChange)=\"settings._colValueChanged(map,record)\" [errorStateMatcher]=\"ngErrorStateMatches\"\r\n                [ngStyle]=\"getStyle()\" />\r\n            <div *ngIf=\"showDescription()\" style=\"white-space: nowrap;\r\n            overflow: hidden;\r\n            text-overflow: ellipsis;\r\n            padding-left:10px;\">\r\n                {{settings._getColDisplayValue(map,record)}}&nbsp;</div>\r\n        </div>\r\n        <div *ngIf=\"isSelect()\" class=\"my-select\" [class.rtl-select]=\"rightToLeft\">\r\n            <select matNativeControl [(ngModel)]=\"_getColumn().inputValue\" \r\n                (ngModelChange)=\"settings._colValueChanged(map,record)\">\r\n                <option *ngFor=\"let v of map.dropDown.items\" value=\"{{v.id}}\">{{v.caption}}</option>\r\n\r\n            </select>\r\n        </div>\r\n\r\n        <button mat-icon-button *ngIf=\"showClick()\" (click)=\"settings._click(map,record)\" class=\"row-button\">\r\n            <mat-icon style=\"height:19px;\">{{getClickIcon()}}  </mat-icon>\r\n        </button>\r\n\r\n        \r\n    <div *ngIf=\"showCheckbox()\">\r\n        <mat-checkbox #theId1 [(ngModel)]=\"_getColumn().inputValue\"\r\n            (ngModelChange)=\"settings._colValueChanged(map,record)\" class=\"fix-label\"></mat-checkbox>\r\n        <mat-error *ngIf=\"getError()\" [id]=\"theId1\">{{getError()}}</mat-error>\r\n    </div>\r\n\r\n</div>\r\n<mat-error *ngIf=\"getError()\" [id]=\"theId\" style=\"font-size: 75%;\">{{getError()}}</mat-error>",
                styles: [".my-select::after{content:\"\";width:0;height:0;border-left:5px solid transparent;border-right:5px solid transparent;border-top:5px solid;position:absolute;top:50%;right:0;pointer-events:none;color:rgba(0,0,0,.54);margin-top:-5px}.rtl-select::after{right:auto;left:0}.my-select{position:relative;width:100%}.data-control3{display:flex;width:100%}.row-button{height:auto;line-height:normal}"]
            }] }
];
DataControl3Component.propDecorators = {
    map: [{ type: Input }],
    column: [{ type: Input }],
    rightToLeft: [{ type: Input }],
    record: [{ type: Input }],
    notReadonly: [{ type: Input }],
    settings: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DataGridComponent {
    constructor() {
        this.displayButtons = true;
        this.displayVCR = true;
        this.rowButtons = [];
        this.keys = [];
        this.page = 1;
    }
    /**
     * @param {?} c
     * @return {?}
     */
    isFiltered(c) {
        return this.settings.columns.filterHelper.isFiltered(c);
    }
    /**
     * @param {?} b
     * @param {?} row
     * @return {?}
     */
    getButtonCssClass(b, row) {
        if (!b.cssClass)
            return "";
        if (isFunction$1(b.cssClass))
            return ((/** @type {?} */ (b.cssClass)))(row);
        return b.cssClass.toString();
    }
    /**
     * @private
     * @param {?} b
     * @return {?}
     */
    addButton(b) {
        if (!b.click)
            b.click = (/**
             * @param {?} r
             * @return {?}
             */
            (r) => { });
        if (!b.visible)
            b.visible = (/**
             * @param {?} r
             * @return {?}
             */
            r => true);
        if (!b.cssClass)
            b.cssClass = (/**
             * @param {?} r
             * @return {?}
             */
            r => "btn");
        else if (!isFunction$1(b.cssClass)) {
            /** @type {?} */
            let x = b.cssClass;
            b.cssClass = (/** @type {?} */ (((/**
             * @param {?} r
             * @return {?}
             */
            (r) => x))));
        }
        this.rowButtons.push(b);
        return b;
    }
    /**
     * @param {?} row
     * @return {?}
     */
    rowClicked(row) {
        this.settings.setCurrentRow(row);
    }
    /**
     * @return {?}
     */
    nextPage() {
        this.page++;
    }
    /**
     * @return {?}
     */
    previousPage() {
        if (this.page <= 1)
            return;
        this.page--;
    }
    /**
     * @param {?} what
     * @param {?} r
     * @return {?}
     */
    catchErrors(what, r) {
        what.catch((/**
         * @param {?} e
         * @return {?}
         */
        (e) => {
            this.showError(r);
        }));
    }
    /**
     * @private
     * @param {?} row
     * @return {?}
     */
    showError(row) {
        /** @type {?} */
        let message = row.error;
        if (!message)
            message = "";
        /** @type {?} */
        let foundStateErrors = false;
        row.__iterateColumns().forEach((/**
         * @param {?} c
         * @return {?}
         */
        c => {
            if (c.error) {
                if (!foundStateErrors) {
                    foundStateErrors = true;
                    message = "";
                }
                /** @type {?} */
                let m = c.caption + ": ";
                m += c.error;
                message += m + "\n";
            }
        }));
        alert(message);
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        if (!this.settings)
            return;
        this.rowButtons = [];
        if (this.settings.allowUpdate) {
            this.addButton({
                name: "",
                cssClass: "glyphicon glyphicon-ok btn-success",
                visible: (/**
                 * @param {?} r
                 * @return {?}
                 */
                r => r.wasChanged()),
                click: (/**
                 * @param {?} r
                 * @return {?}
                 */
                r => {
                    this.catchErrors(this.settings._doSavingRow(r), r);
                }),
            });
            this.addButton({
                name: "",
                cssClass: "btn btn-danger glyphicon glyphicon-ban-circle",
                visible: (/**
                 * @param {?} r
                 * @return {?}
                 */
                r => r.wasChanged()),
                click: (/**
                 * @param {?} r
                 * @return {?}
                 */
                r => {
                    r.reset();
                })
            });
        }
        if (this.settings.allowDelete)
            this.addButton({
                name: '',
                visible: (/**
                 * @param {?} r
                 * @return {?}
                 */
                (r) => !isNewRow(r)),
                click: (/**
                 * @param {?} r
                 * @return {?}
                 */
                r => {
                    if (this.settings.setCurrentRow && this.settings.settings.confirmDelete) {
                        this.settings.settings.confirmDelete(r, (/**
                         * @return {?}
                         */
                        () => this.catchErrors(r.delete(), r)));
                    }
                    else
                        this.catchErrors(r.delete(), r);
                }),
                cssClass: "btn-danger glyphicon glyphicon-trash"
            });
        if (this.settings._buttons)
            for (let b of this.settings._buttons) {
                this.addButton(b);
            }
        if (!this.records && this.settings) {
            this.settings.getRecords().then((/**
             * @param {?} r
             * @return {?}
             */
            (r) => {
                this.records = r;
            }));
        }
    }
    /**
     * @param {?} row
     * @return {?}
     */
    _getRowClass(row) {
        /** @type {?} */
        let r = '';
        if (this.settings.rowClass)
            r += this.settings.rowClass(row);
        if (row == this.settings.currentRow)
            r += " active";
        return r;
    }
}
DataGridComponent.decorators = [
    { type: Component, args: [{
                selector: 'data-grid',
                template: `

  <div *ngIf="settings&&settings.columns&& settings.columns.designMode">
<pre>
{{settings.columns.__columnSettingsTypeScript()}}
</pre>


  </div>
  <div  >
  <div  *ngIf="settings && records && displayVCR">
    <button class="btn glyphicon glyphicon-pencil btn-primary" *ngIf="settings.columns.allowDesignMode" (click)="settings.columns.designMode=!settings.columns.designMode"></button>
    <button class="btn glyphicon glyphicon-chevron-left" *ngIf="settings.page>1" (click)="settings.previousPage()"></button>
    <button class="btn glyphicon glyphicon-chevron-right" *ngIf="records.items&& records.items.length>0" (click)="settings.nextPage()"></button>
    <button class="btn btn-primary glyphicon glyphicon-plus" *ngIf="settings.allowUpdate &&settings.allowInsert" (click)="settings.addNewRow()"></button>
    <button class="btn glyphicon glyphicon glyphicon-cog" (click)="settings.userChooseColumns()"></button>
    <button class="btn glyphicon glyphicon glyphicon-filter" (click)="dataFilter.userFilterButton()"></button>
    <button class="btn glyphicon glyphicon glyphicon-repeat" (click)="settings.getRecords()"></button>

        
 
  </div>
  <Data-Filter [settings]="settings" #dataFilter></Data-Filter>
  <div *ngIf="settings.showSelectColumn" class="selectColumnsArea">
  lines per page
  <select class="form-control" style="width:100px;display:inline-block" [(ngModel)]="settings.rowsPerPage" (change)="settings.getRecords()">
          <option *ngFor="let r of settings.rowsPerPageOptions" value="{{r}}">{{r}}</option>
      </select><br/>
  Select Columns
  <ol>
  <li *ngFor="let c of settings.currList; let i=index">
      <select [(ngModel)]="settings.currList[i]" class="form-control selectColumnCombo" (change)="settings.columns.colListChanged()">
          <option *ngFor="let o of settings.origList" [ngValue]="o">{{o.caption}}</option>
      </select>
      <button class="btn btn-sm glyphicon glyphicon-trash" *ngIf="settings.currList.length>1" (click)="settings.deleteCol(c)"></button>
      <button class="btn btn-sm  glyphicon glyphicon-plus" (click)="settings.addCol(c)"></button>
      <button class="btn btn-sm  glyphicon glyphicon-chevron-down" *ngIf="i<settings.currList.length-1"(click)="settings.columns.moveCol(c,1)"></button>
      <button class="btn btn-sm  glyphicon glyphicon-chevron-up" *ngIf="i>0" (click)="settings.columns.moveCol(c,-1)"></button>
  </li>
</ol>
<button (click)="settings.resetColumns()" class="btn glyphicon glyphicon-repeat"></button>
</div>
<div>
    <table class="table table-bordered table-condensed table-hover table-striped " *ngIf="settings&&settings.columns">

      <thead>
        <tr>
          <th *ngFor="let map of settings.columns.getGridColumns()" class="headerWithFilter" [style.width]="settings.columns.__dataControlStyle(map)">

            <span (click)="settings.sort(map.column)">{{map.caption}}</span>


            <span class="glyphicon glyphicon-filter filterButton" [class.filteredFilterButton]="isFiltered(map.column)"
              (click)="settings.columns.showFilterDialog(map)"></span>
            <div class="filterDialog col-sm-4" *ngIf="settings.columns._shouldShowFilterDialog(map)">
              <div class="form-group">
                <data-control [settings]="settings.columns" [map]="map" [record]="settings.columns.filterHelper.filterRow" [notReadonly]="true"></data-control>
              </div>
              <button class="btn glyphicon glyphicon-ok btn-success" (click)="settings.columns.filterRows(map)"></button>
              <button class="btn glyphicon glyphicon-remove btn-primary" (click)="settings.columns.clearFilter(map)"></button>

            </div>
            <span class="glyphicon glyphicon-chevron-up pull-right" *ngIf="settings.sortedAscending(map.column)"></span>
            <span class="glyphicon glyphicon-chevron-down pull-right" *ngIf="settings.sortedDescending(map.column)"></span>
            <column-designer [settings]="settings.columns" [map]="map"></column-designer>





          </th>
          <th *ngIf="rowButtons&& rowButtons.length>0&&displayButtons" [class.col-xs-1]="rowButtons&&rowButtons.length<3" [class.col-xs-2]="rowButtons.length>=3"></th>
        </tr>
      </thead>
      <tbody>
        <tr *ngFor="let record of records" [className]="_getRowClass(record)" (click)="rowClicked(record)">

          <td *ngFor="let map of settings.columns.getGridColumns()" [className]="settings.columns._getColumnClass(map,record)">
            <data-control [settings]="settings.columns" [map]="map" [record]="record"></data-control>
          </td>
          <td *ngIf="rowButtons.length>0&&displayButtons" style="white-space:nowrap">
            <span *ngFor="let b of rowButtons">
              <button class="btn {{getButtonCssClass(b,record) }}" *ngIf="b.visible(record)" (click)="b.click(record)">{{b.name}}</button>
            </span>
          </td>
        </tr>

      </tbody>
    </table>
  </div>
  <data-area *ngIf="!settings.hideDataArea" [settings]="settings" [columns]="2"></data-area>
  `,
                styles: [`

.filterDialog {
    background-color: white;
    position: absolute;
    padding: 10px;
    border-color: gray;
    border-width: 2px;
    border-style: solid;
    z-index: 800;
    border-radius: 5px;

}

    .filterDialog .form-group {
        margin-right: 0;
        margin-left: 0;
    }

.designModeButton span {
    visibility: hidden;
}

.designModeButton:hover span {
    visibility: visible
}
.selectColumnCombo{
  display:inline-block;
  width:auto;
}
.selectColumnsArea{
  display:block;
}

.headerWithFilter {
}
.headerWithFilter .filterButton{
    visibility:hidden;
}
    .headerWithFilter .filteredFilterButton {
        visibility: visible;
    }
    .headerWithFilter:hover .filterButton {
        visibility: visible;
    }

.filterButton {
}
table input {
    min-width:75px;
}
table select {
    min-width:100px
}`]
            }] }
];
DataGridComponent.propDecorators = {
    displayButtons: [{ type: Input }],
    displayVCR: [{ type: Input }],
    records: [{ type: Input }],
    settings: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ColumnDesigner {
}
ColumnDesigner.decorators = [
    { type: Component, args: [{
                selector: 'column-designer',
                template: `
<div *ngIf="map.designMode" class="columnDesigner">
    <div class="form-group">
        <input type="text" class="form-control" [(ngModel)]="map.caption">
    </div>
    <div class="form-group">
      <label >Width</label>
      <div class="input-group">
      <input type="text" class="form-control" [(ngModel)]="map.width">
      <span class="input-group-btn">
        <button class="btn btn-default" type="button" (click)="settings.__changeWidth(map,-10)">-</button>
      </span>
      <span class="input-group-btn">
      <button class="btn btn-default" type="button" (click)="settings.__changeWidth(map,10)">+</button>
    </span>
    </div><!-- /input-group -->
      
    </div>

    <label>Input Type</label>
    <div class="form-group">
        <select class="form-control" [(ngModel)]="map.inputType" placeholder="inputType">
            <option value="" selected>text</option>
            <option value="number">number</option>
            <option value="date">date</option>
            <option value="checkbox">checkbox</option>
        </select>
    </div>

    <div class="form-group">
      <label>
           <input type="checkbox"  [(ngModel)]="map.readonly"> Readonly
      </label>
    </div>


    <div class="form-group">

        <button class="btn btn-success glyphicon glyphicon-ok pull-left" (click)="settings.designColumn(map)"></button>
        <div class="btn-group pull-right">
                <button class="btn btn-danger glyphicon glyphicon-trash " (click)="settings.deleteCol(map)"></button>
                <button class="btn btn-primary glyphicon glyphicon-plus " (click)="settings.addCol(map)"></button>
                <button class="btn btn-primary glyphicon glyphicon-chevron-left" (click)="settings.moveCol(map,-1)"></button>
                <button class="btn btn-primary glyphicon glyphicon-chevron-right" (click)="settings.moveCol(map,1)"></button>
        </div>
    </div>
</div>
<span class="designModeButton pull-right">
<span class="glyphicon glyphicon-pencil " (click)="settings.designColumn(map)" *ngIf="settings.allowDesignMode"></span>
</span>
`,
                styles: [`.columnDesigner {
  background-color: white;
  position: absolute;
  padding: 10px;
  border-color: gray;
  border-width: 2px;
  border-style: solid;
  z-index: 800;
  border-radius: 5px;
  width: 300px;
}

  .columnDesigner .form-group {
      margin-right: 0;
      margin-left: 0;
  }`]
            }] }
];
ColumnDesigner.propDecorators = {
    map: [{ type: Input }],
    settings: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DataFilterInfoComponent {
    constructor() {
        this.showFilterButton = false;
        this.showAddFilter = false;
        this.editFilterVisible = false;
    }
    /**
     * @param {?} col
     * @return {?}
     */
    getCurrentFilterValue(col) {
        /** @type {?} */
        let m = this.settings.origList.find((/**
         * @param {?} x
         * @return {?}
         */
        x => x.column == col));
        return this.settings.columns._getColDisplayValue(m, this.settings.filterHelper.filterRow);
    }
    /**
     * @return {?}
     */
    cancelAddFilter() {
    }
    /**
     * @param {?} col
     * @return {?}
     */
    showEditFilter(col) {
        this.filterColumnToAdd = this.settings.origList.find((/**
         * @param {?} x
         * @return {?}
         */
        x => x.column == col));
        this.editFilterVisible = true;
        this.showAddFilter = false;
    }
    /**
     * @return {?}
     */
    userFilterButton() {
        this.showFilterButton = !this.showFilterButton;
        this.settings.initOrigList();
        if (this.settings.filterHelper.filterColumns.length == 0)
            this.showAddAnotherFilterDialog();
    }
    /**
     * @return {?}
     */
    showAddAnotherFilterDialog() {
        this.showAddFilter = true;
        this.filterColumnToAdd = undefined;
    }
    /**
     * @return {?}
     */
    confirmEditFilter() {
        this.settings.columns.filterRows(this.filterColumnToAdd);
        this.editFilterVisible = false;
    }
    /**
     * @return {?}
     */
    clearEditFilter() {
        this.settings.columns.clearFilter(this.filterColumnToAdd);
        this.editFilterVisible = false;
    }
    /**
     * @return {?}
     */
    addFilter() {
        this.settings.columns.filterRows(this.filterColumnToAdd);
        this.showAddFilter = false;
    }
    /**
     * @return {?}
     */
    cancelAddNewFilter() {
        this.showAddFilter = false;
    }
}
DataFilterInfoComponent.decorators = [
    { type: Component, args: [{
                selector: 'Data-Filter',
                template: "<div *ngIf=\"this.showFilterButton\">\r\n    <ul>\r\n        <li *ngFor=\"let map of settings.columns.filterHelper.filterColumns\" (click)=\"this.showEditFilter(map)\">\r\n            {{map.caption}}: {{this.getCurrentFilterValue(map)}}\r\n        </li>\r\n        <li (click)=\"this.showAddAnotherFilterDialog()\" *ngIf=\"!showAddFilter&&!editFilterVisible\">...</li>\r\n    </ul>\r\n    <div *ngIf=\"this.editFilterVisible\">\r\n        {{this.filterColumnToAdd.caption}}:\r\n        <data-control2 *ngIf=\"this.filterColumnToAdd\" [settings]=\"settings.columns\" [map]=\"this.filterColumnToAdd\"\r\n            [record]=\"settings.columns.filterHelper.filterRow\" [notReadonly]=\"true\"></data-control2>\r\n        <button mat-icon-button (click)=\"this.confirmEditFilter()\">\r\n            <mat-icon>check</mat-icon>\r\n        </button>\r\n        <button mat-icon-button (click)=\"this.clearEditFilter()\">\r\n            <mat-icon>clear</mat-icon>\r\n        </button>\r\n    </div>\r\n    <div *ngIf=\"this.showAddFilter\">\r\n        <select [(ngModel)]=\"this.filterColumnToAdd\" class=\"form-control selectColumnCombo\"\r\n            (change)=\"settings.columns.colListChanged()\">\r\n            <option *ngFor=\"let o of settings.origList\" [ngValue]=\"o\">{{o.caption}} </option>\r\n        </select>\r\n        <data-control2 *ngIf=\"this.filterColumnToAdd\" [settings]=\"settings.columns\" [map]=\"this.filterColumnToAdd\"\r\n            [record]=\"settings.columns.filterHelper.filterRow\" [notReadonly]=\"true\"></data-control2>\r\n        <button mat-icon-button (click)=\"this.addFilter()\"><mat-icon>check</mat-icon></button>\r\n        <button mat-icon-button (click)=\"this.cancelAddNewFilter()\"><mat-icon>clear</mat-icon></button>\r\n    </div>\r\n</div>"
            }] }
];
DataFilterInfoComponent.propDecorators = {
    settings: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DataAreaCompnent {
    constructor() {
        this.settings = { columns: new ColumnCollection((/**
             * @return {?}
             */
            () => undefined), (/**
             * @return {?}
             */
            () => false), undefined, (/**
             * @return {?}
             */
            () => true)), lines: [] };
        this.labelWidth = 4;
        this.columns = 1;
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        if (this.settings && this.settings.columns) {
            this.settings.columns.onColListChange((/**
             * @return {?}
             */
            () => this.lastCols = undefined));
            /** @type {?} */
            let areaSettings = (/** @type {?} */ (this.settings));
            if (areaSettings.settings) {
                if (areaSettings.settings.labelWidth)
                    this.labelWidth = areaSettings.settings.labelWidth;
                if (areaSettings.settings.numberOfColumnAreas)
                    this.columns = areaSettings.settings.numberOfColumnAreas;
            }
        }
    }
    /**
     * @return {?}
     */
    getColumnsClass() {
        if (this.columns >= 1)
            return "col-sm-" + 12 / this.columns;
    }
    /**
     * @return {?}
     */
    theColumns() {
        /** @type {?} */
        let cols = this.settings.columns.getNonGridColumns();
        if (cols == this.lastAllCols)
            return this.lastCols;
        this.lastAllCols = cols;
        /** @type {?} */
        let r = [];
        this.lastCols = r;
        for (var i = 0; i < this.columns; i++) {
            r.push([]);
        }
        /** @type {?} */
        let itemsPerCol = Math.round(cols.length / this.columns);
        for (var i = 0; i < cols.length; i++) {
            r[Math.floor(i / itemsPerCol)].push(cols[i]);
        }
        return this.lastCols;
    }
}
DataAreaCompnent.decorators = [
    { type: Component, args: [{
                selector: 'data-area',
                template: `

<div class="form-horizontal row" *ngIf="settings.columns&&settings.columns.__showArea()" >

        <div class="{{getColumnsClass()}}" *ngFor="let col of theColumns()">
            <div class="form-group {{settings.columns._getColumnClass(map,settings.columns.currentRow())}}" *ngFor="let map of col" >
                <div class="col-sm-{{labelWidth}}">
                    <label class="control-label" *ngIf="!map.designMode">{{map.caption}}</label>
                    <column-designer [settings]="settings.columns" [map]="map"></column-designer>
                </div>
                <div class="col-sm-{{12-labelWidth}}">
                    <data-control [settings]="settings.columns" [map]="map" [record]="settings.columns.currentRow()"></data-control>
                </div>
            </div>
        </div>
</div>


`
            }] }
];
DataAreaCompnent.propDecorators = {
    settings: [{ type: Input }],
    labelWidth: [{ type: Input }],
    columns: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DataGrid2Component {
    constructor() {
        this.rightToLeft = false;
        this.displayButtons = true;
        this.displayVCR = true;
        this.rowButtons = [];
        this.keys = [];
        this.page = 1;
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if (window && window.getComputedStyle && this.dataGridDiv) {
            this.rightToLeft = window.getComputedStyle(this.dataGridDiv.nativeElement, null).getPropertyValue('direction') == 'rtl';
        }
    }
    /**
     * @param {?} map
     * @return {?}
     */
    getColFlex(map) {
        return '0 0 ' + this.getColWidth(map);
    }
    /**
     * @param {?} map
     * @return {?}
     */
    getColWidth(map) {
        /** @type {?} */
        let x = this.settings.columns.__dataControlStyle(map);
        if (!x)
            x = '200px';
        return x;
    }
    /**
     * @return {?}
     */
    test() {
        //this.dataGridDiv.nativeElement.scrollTop = 0;
    }
    /**
     * @return {?}
     */
    getTotalRows() {
        if (this.settings.totalRows)
            return Math.ceil(this.settings.totalRows / this.settings.rowsPerPage);
        if (this.rightToLeft)
            return 'רבים';
        return 'many';
    }
    /**
     * @param {?} x
     * @return {?}
     */
    dragStart(x) {
        this.tempDragColumn = x;
    }
    /**
     * @param {?} x
     * @param {?} event
     * @return {?}
     */
    dragOver(x, event) {
        event.preventDefault();
    }
    /**
     * @param {?} x
     * @return {?}
     */
    onDrop(x) {
        /** @type {?} */
        let oldPosition = this.settings.columns.items.indexOf(this.tempDragColumn);
        this.settings.columns.items.splice(oldPosition, 1);
        /** @type {?} */
        let newPosition = this.settings.columns.items.indexOf(x);
        if (newPosition == oldPosition)
            newPosition++;
        this.settings.columns.items.splice(newPosition, 0, this.tempDragColumn);
        this.settings.columns.colListChanged();
    }
    /**
     * @return {?}
     */
    _getHeight() {
        if (this.height) {
            if ((+this.height).toString() == this.height)
                return this.height + "px";
            return this.height;
        }
    }
    /**
     * @param {?} c
     * @return {?}
     */
    isFiltered(c) {
        return this.settings.columns.filterHelper.isFiltered(c);
    }
    /**
     * @param {?} b
     * @param {?} row
     * @return {?}
     */
    getButtonCssClass(b, row) {
        if (!b.cssClass)
            return "";
        if (isFunction$1(b.cssClass))
            return ((/** @type {?} */ (b.cssClass)))(row);
        return b.cssClass.toString();
    }
    /**
     * @private
     * @param {?} b
     * @return {?}
     */
    addButton(b) {
        if (!b.click)
            b.click = (/**
             * @param {?} r
             * @return {?}
             */
            (r) => { });
        if (!b.visible)
            b.visible = (/**
             * @param {?} r
             * @return {?}
             */
            r => true);
        if (!b.cssClass)
            b.cssClass = (/**
             * @param {?} r
             * @return {?}
             */
            r => "btn");
        else if (!isFunction$1(b.cssClass)) {
            /** @type {?} */
            let x = b.cssClass;
            b.cssClass = (/** @type {?} */ (((/**
             * @param {?} r
             * @return {?}
             */
            (r) => x))));
        }
        this.rowButtons.push(b);
        return b;
    }
    /**
     * @param {?} row
     * @return {?}
     */
    rowClicked(row) {
        this.settings.setCurrentRow(row);
    }
    /**
     * @return {?}
     */
    nextPage() {
        this.page++;
    }
    /**
     * @return {?}
     */
    previousPage() {
        if (this.page <= 1)
            return;
        this.page--;
    }
    /**
     * @param {?} what
     * @param {?} r
     * @return {?}
     */
    catchErrors(what, r) {
        what.catch((/**
         * @param {?} e
         * @return {?}
         */
        (e) => {
            this.showError(r);
        }));
    }
    /**
     * @private
     * @param {?} row
     * @return {?}
     */
    showError(row) {
        /** @type {?} */
        let message = row.error;
        if (!message)
            message = "";
        /** @type {?} */
        let foundStateErrors = false;
        row.__iterateColumns().forEach((/**
         * @param {?} c
         * @return {?}
         */
        c => {
            if (c.error) {
                if (!foundStateErrors) {
                    foundStateErrors = true;
                    message = "";
                }
                /** @type {?} */
                let m = c.caption + ": ";
                m += c.error;
                message += m + "\n";
            }
        }));
        alert(message);
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        if (!this.settings)
            return;
        this.rowButtons = [];
        if (this.settings.allowUpdate) {
            this.addButton({
                name: "",
                icon: 'check',
                cssClass: "glyphicon glyphicon-ok btn-success",
                visible: (/**
                 * @param {?} r
                 * @return {?}
                 */
                r => r.wasChanged()),
                click: (/**
                 * @param {?} r
                 * @return {?}
                 */
                r => {
                    this.catchErrors(this.settings._doSavingRow(r), r);
                }),
            });
            this.addButton({
                name: "",
                icon: 'cancel',
                cssClass: "btn btn-danger glyphicon glyphicon-ban-circle",
                visible: (/**
                 * @param {?} r
                 * @return {?}
                 */
                r => r.wasChanged()),
                click: (/**
                 * @param {?} r
                 * @return {?}
                 */
                r => {
                    r.reset();
                })
            });
        }
        if (this.settings.allowDelete)
            this.addButton({
                name: '',
                visible: (/**
                 * @param {?} r
                 * @return {?}
                 */
                (r) => {
                    return r && !r.isNew();
                }),
                icon: 'delete',
                click: (/**
                 * @param {?} r
                 * @return {?}
                 */
                r => {
                    if (this.settings.setCurrentRow && this.settings.settings.confirmDelete) {
                        this.settings.settings.confirmDelete(r, (/**
                         * @return {?}
                         */
                        () => this.catchErrors(r.delete(), r)));
                    }
                    else
                        this.catchErrors(r.delete(), r);
                }),
                cssClass: "btn-danger glyphicon glyphicon-trash"
            });
        if (this.settings._buttons)
            for (let b of this.settings._buttons) {
                this.addButton(b);
            }
        if (!this.records && this.settings) {
            this.settings.getRecords().then((/**
             * @param {?} r
             * @return {?}
             */
            (r) => {
                this.records = r;
            }));
        }
    }
    /**
     * @param {?} row
     * @return {?}
     */
    _getRowClass(row) {
        /** @type {?} */
        let r = 'dataGridRow ';
        if (this.settings.rowClass)
            r += this.settings.rowClass(row);
        if (row == this.settings.currentRow)
            r += " active";
        return r;
    }
    /**
     * @param {?} col
     * @param {?} row
     * @return {?}
     */
    _getRowColumnClass(col, row) {
        return this.settings.columns._getColumnClass(col, row) + ' dataGridDataCell';
    }
}
DataGrid2Component.decorators = [
    { type: Component, args: [{
                selector: 'data-grid2',
                template: "<div>\r\n  <div>\r\n    <div class=\"dataGrid\" #dataGridDiv *ngIf=\"settings&&settings.columns\" [style.height]=\"_getHeight()\">\r\n      <div class=\"inner-wrapper\">\r\n        <div class=\"dataGridHeaderArea\">\r\n          <div class=\"dataGridRow\">\r\n            <div class=\"dataGridHeaderCell headerWithFilter\" *ngFor=\"let map of settings.columns.getGridColumns()\"\r\n              [style.flex]=\"getColFlex(map)\" [style.width]=\"getColWidth(map)\" draggable=\"true\"\r\n              (dragstart)=\"dragStart(map)\" (dragover)=\"dragOver(map,$event)\" (drop)=\"onDrop(map)\">\r\n\r\n              <span (click)=\"settings.sort(map.column)\">{{map.caption}}</span>\r\n\r\n\r\n              <span class=\"filterButton\" [class.filteredFilterButton]=\"isFiltered(map.column)\"\r\n                (click)=\"settings.columns.showFilterDialog(map)\">\r\n                <mat-icon>details</mat-icon>\r\n              </span>\r\n              <div class=\"filterDialog col-sm-4\" *ngIf=\"settings.columns._shouldShowFilterDialog(map)\">\r\n                <div class=\"form-group\">\r\n                  <data-control2 [settings]=\"settings.columns\" [map]=\"map\"\r\n                    [record]=\"settings.columns.filterHelper.filterRow\" [notReadonly]=\"true\"></data-control2>\r\n                </div>\r\n                <button mat-icon-button (click)=\"settings.columns.filterRows(map)\">\r\n                  <mat-icon>check</mat-icon>\r\n                </button>\r\n                <button mat-icon-button (click)=\"settings.columns.clearFilter(map)\">\r\n                  <mat-icon>clear</mat-icon>\r\n                </button>\r\n\r\n              </div>\r\n              <mat-icon *ngIf=\"settings.sortedAscending(map.column)\">arrow_downward</mat-icon>\r\n              <mat-icon class=\"glyphicon glyphicon-chevron-down pull-right\"\r\n                *ngIf=\"settings.sortedDescending(map.column)\">arrow_upward</mat-icon>\r\n            </div>\r\n            <div class=\"dataGridButtonHeaderCell\" *ngIf=\"rowButtons&& rowButtons.length>0&&displayButtons\"\r\n              [class.col-xs-1]=\"rowButtons&&rowButtons.length<3\"></div>\r\n          </div>\r\n        </div>\r\n        <div class=\"dataGridBodyArea\">\r\n          <div class=\"dataGridRow\" *ngFor=\"let record of records\" [className]=\"_getRowClass(record)\"\r\n            (click)=\"rowClicked(record)\">\r\n\r\n            <div class=\"dataGridDataCell\" *ngFor=\"let map of settings.columns.getGridColumns()\"\r\n              [className]=\"_getRowColumnClass(map,record)\" [style.flex]=\"getColFlex(map)\"\r\n              [style.width]=\"getColWidth(map)\">\r\n              <data-control3 [settings]=\"settings.columns\" [map]=\"map\" [record]=\"record\" [rightToLeft]=\"rightToLeft\"></data-control3>\r\n            </div>\r\n            <div class=\"dataGridButtonCell\" *ngIf=\"rowButtons.length>0&&displayButtons\" style=\"white-space:nowrap\">\r\n              <ng-container *ngFor=\"let b of rowButtons\">\r\n                <button mat-icon-button *ngIf=\"b.visible(record)&&b.icon\" (click)=\"b.click(record)\" class=\"row-button\">\r\n                  <mat-icon *ngIf=\"b.icon\">{{b.icon}} </mat-icon> {{b.name}}\r\n                </button>\r\n                <button mat-raised-button *ngIf=\"b.visible(record)&&!b.icon\" (click)=\"b.click(record)\" class=\"row-button\">\r\n                   {{b.name}}\r\n                </button>\r\n              </ng-container>\r\n\r\n            </div>\r\n          </div>\r\n\r\n        </div>\r\n      </div>\r\n    </div>\r\n    <div class=\"dataGridFooterArea\" *ngIf=\"records\">\r\n      <div class=\"dataGridFooterButtonGroup\">\r\n        <button mat-icon-button (click)=\"settings.userChooseColumns()\">\r\n          <mat-icon>settings</mat-icon>\r\n        </button>\r\n        <button mat-icon-button (click)=\"dataFilter.userFilterButton()\">\r\n          <mat-icon>details</mat-icon>\r\n        </button>\r\n        {{rightToLeft?'\u05E9\u05D5\u05E8\u05D5\u05EA \u05D1\u05D3\u05E3': 'lines per page'}}\r\n        <select class=\"form-control\" style=\"width:80px;display:inline-block\" [(ngModel)]=\"settings.rowsPerPage\"\r\n          (change)=\"settings.getRecords()\">\r\n          <option *ngFor=\"let r of settings.rowsPerPageOptions\" value=\"{{r}}\">{{r}}</option>\r\n        </select></div>\r\n      <div class=\"dataGridFooterButtonGroup\">\r\n        <button mat-icon-button (click)=\"settings.firstPage()\">\r\n          <mat-icon>{{rightToLeft?'last_page':'first_page'}} </mat-icon>\r\n        </button>\r\n        <button mat-icon-button [disabled]=\"settings.page==1\" (click)=\"settings.previousPage()\">\r\n          <mat-icon>{{rightToLeft?'chevron_right':'chevron_left'}}</mat-icon>\r\n        </button>\r\n        {{rightToLeft?'\u05E2\u05DE\u05D5\u05D3':'Page'}} {{settings.page}} {{rightToLeft?' \u05DE\u05EA\u05D5\u05DA ':' of '}} {{getTotalRows()}}\r\n        <button mat-icon-button [disabled]=\"!(records.items&& records.items.length>0)\" (click)=\"settings.nextPage()\">\r\n          <mat-icon>{{!rightToLeft?'chevron_right':'chevron_left'}}</mat-icon>\r\n        </button>\r\n        <button mat-icon-button *ngIf=\"settings.allowUpdate &&settings.allowInsert\" (click)=\"settings.addNewRow()\">\r\n          <mat-icon>add</mat-icon>\r\n        </button>\r\n      </div>\r\n     \r\n\r\n\r\n      <Data-Filter [settings]=\"settings\" #dataFilter></Data-Filter>\r\n      <div *ngIf=\"settings.showSelectColumn\" class=\"selectColumnsArea\">\r\n        Select Columns\r\n        <ol>\r\n          <li *ngFor=\"let c of settings.currList; let i=index\">\r\n            <select [(ngModel)]=\"settings.currList[i]\" class=\"form-control selectColumnCombo\"\r\n              (change)=\"settings.columns.colListChanged()\">\r\n              <option *ngFor=\"let o of settings.origList\" [ngValue]=\"o\">{{o.caption}}</option>\r\n            </select>\r\n            <input [(ngModel)]=\"c.width\" style=\"width:50px;display:inline;\" >\r\n            <button mat-icon-button  *ngIf=\"settings.currList.length>1\"\r\n              (click)=\"settings.deleteCol(c)\"><mat-icon>cancel</mat-icon></button>\r\n            <button  mat-icon-button (click)=\"settings.addCol(c)\"><mat-icon>add</mat-icon></button>\r\n            <button mat-icon-button *ngIf=\"i<settings.currList.length-1\"\r\n              (click)=\"settings.columns.moveCol(c,1)\"><mat-icon>keyboard_arrow_down</mat-icon></button>\r\n            <button mat-icon-button *ngIf=\"i>0\"\r\n              (click)=\"settings.columns.moveCol(c,-1)\"><mat-icon>keyboard_arrow_up</mat-icon></button>\r\n          </li>\r\n        </ol>\r\n        <button (click)=\"settings.resetColumns()\" class=\"btn glyphicon glyphicon-repeat\"></button>\r\n      </div>\r\n    </div>\r\n  </div>\r\n  <data-area2 *ngIf=\"!settings.hideDataArea\" [settings]=\"settings\" ></data-area2>",
                styles: [".row-button{height:auto;line-height:normal}.inner-wrapper{display:inline-block}.dataGrid{overflow:auto;border:1px solid #e0e0e0}.dataGridRow{display:flex}.dataGridHeaderArea{position:-webkit-sticky;position:sticky;top:0;clear:both;background-color:#f5f5f5;z-index:3}.dataGridFooterButtonGroup{display:inline-block}.dataGridFooterArea{width:100%;padding:4px;bottom:0;left:0;background-color:#f5f5f5;border:1px solid #e0e0e0}.dataGridDataCell,.dataGridHeaderCell{padding:4px}.dataGridHeaderCell{height:36px}.dataGridDataCell{border-top:1px solid #e0e0e0}.filterDialog{background-color:#fff;position:absolute;padding:10px;z-index:800;border-radius:5px;width:200px;border:2px solid gray}.filterDialog .form-group{margin-right:0;margin-left:0}.designModeButton span{visibility:hidden}.designModeButton:hover span{visibility:visible}.selectColumnCombo{display:inline-block;width:auto}.selectColumnsArea{display:block}.headerWithFilter .filterButton{visibility:hidden}.headerWithFilter .filteredFilterButton,.headerWithFilter:hover .filterButton{visibility:visible}table input{min-width:75px}table select{min-width:100px}"]
            }] }
];
DataGrid2Component.propDecorators = {
    dataGridDiv: [{ type: ViewChild, args: ['dataGridDiv',] }],
    width: [{ type: Input }],
    height: [{ type: Input }],
    displayButtons: [{ type: Input }],
    displayVCR: [{ type: Input }],
    records: [{ type: Input }],
    settings: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class RestDataProvider {
    /**
     * @param {?} url
     * @param {?} http
     */
    constructor(url, http) {
        this.url = url;
        this.http = http;
    }
    /**
     * @param {?} name
     * @return {?}
     */
    provideFor(name) {
        return new ActualRestDataProvider(this.url + '/' + name, this.http);
    }
}
class ActualRestDataProvider {
    /**
     * @param {?} url
     * @param {?} http
     */
    constructor(url, http) {
        this.url = url;
        this.http = http;
    }
    /**
     * @param {?} where
     * @return {?}
     */
    count(where) {
        /** @type {?} */
        let url = new UrlBuilder(this.url);
        url.add("__action", "count");
        if (where) {
            where.__applyToConsumer(new FilterConsumnerBridgeToUrlBuilder(url));
        }
        return this.http.get(url.url).then((/**
         * @param {?} r
         * @return {?}
         */
        r => +(r.count)));
    }
    /**
     * @param {?} options
     * @return {?}
     */
    find(options) {
        /** @type {?} */
        let url = new UrlBuilder(this.url);
        if (options) {
            if (options.where) {
                options.where.__applyToConsumer(new FilterConsumnerBridgeToUrlBuilder(url));
            }
            if (options.orderBy && options.orderBy.Segments) {
                /** @type {?} */
                let sort = '';
                /** @type {?} */
                let order = '';
                options.orderBy.Segments.forEach((/**
                 * @param {?} c
                 * @return {?}
                 */
                c => {
                    if (sort.length > 0) {
                        sort += ", ";
                        order += ", ";
                    }
                    sort += c.column.jsonName;
                    order += c.descending ? "desc" : "asc";
                }));
                url.add('_sort', sort);
                url.add('_order', order);
            }
            if (options.limit)
                url.add('_limit', options.limit);
            if (options.page)
                url.add('_page', options.page);
            if (options.additionalUrlParameters)
                url.addObject(options.additionalUrlParameters);
        }
        return this.http.get(url.url);
    }
    /**
     * @param {?} id
     * @param {?} data
     * @return {?}
     */
    update(id, data) {
        return this.http.put(this.url + '/' + encodeURIComponent(id), data);
    }
    /**
     * @param {?} id
     * @return {?}
     */
    delete(id) {
        return this.http.delete(this.url + '/' + encodeURIComponent(id));
    }
    /**
     * @param {?} data
     * @return {?}
     */
    insert(data) {
        return this.http.post(this.url, data);
    }
}
/**
 * @param {?} add
 * @return {?}
 */
function JsonContent(add) {
    add('Content-type', "application/json");
}
class angularHttpProvider {
    /**
     * @param {?} http
     */
    constructor(http) {
        this.http = http;
    }
    /**
     * @param {?} url
     * @param {?} data
     * @return {?}
     */
    post(url, data) {
        return this.http.post(url, data).toPromise();
    }
    /**
     * @param {?} url
     * @return {?}
     */
    delete(url) {
        return this.http.delete(url).toPromise().then((/**
         * @param {?} x
         * @return {?}
         */
        x => { }));
    }
    /**
     * @param {?} url
     * @param {?} data
     * @return {?}
     */
    put(url, data) {
        return this.http.put(url, data).toPromise();
    }
    /**
     * @param {?} url
     * @return {?}
     */
    get(url) {
        return this.http.get(url).toPromise();
    }
}
class restDataProviderHttpProviderUsingFetch {
    /**
     * @param {?=} addRequestHeader
     */
    constructor(addRequestHeader) {
        this.addRequestHeader = addRequestHeader;
        if (!addRequestHeader)
            this.addRequestHeader = (/**
             * @return {?}
             */
            () => { });
    }
    /**
     * @param {?} url
     * @return {?}
     */
    get(url) {
        return myFetch(url, undefined, this.addRequestHeader).then((/**
         * @param {?} r
         * @return {?}
         */
        r => {
            return r;
        }));
    }
    /**
     * @param {?} url
     * @param {?} data
     * @return {?}
     */
    put(url, data) {
        return myFetch(url, {
            method: 'put',
            body: JSON.stringify(data)
        }, this.addRequestHeader, JsonContent);
    }
    /**
     * @param {?} url
     * @return {?}
     */
    delete(url) {
        /** @type {?} */
        let h = new Headers();
        this.addRequestHeader((/**
         * @param {?} name
         * @param {?} value
         * @return {?}
         */
        (name, value) => h.append(name, value)));
        return fetch(url, { method: 'delete', credentials: 'include' }).then(onSuccess$1, onError$1);
    }
    /**
     * @param {?} url
     * @param {?} data
     * @return {?}
     */
    post(url, data) {
        return myFetch(url, {
            method: 'post',
            body: JSON.stringify(data)
        }, this.addRequestHeader, JsonContent);
    }
}
/** @type {?} */
const wrapFetch = {
    wrap: (/**
     * @return {?}
     */
    () => (/**
     * @return {?}
     */
    () => { }))
};
/**
 * @param {?} url
 * @param {?} init
 * @param {...?} addRequestHeader
 * @return {?}
 */
function myFetch(url, init, ...addRequestHeader) {
    if (!init)
        init = {};
    if (!init.headers)
        init.headers = new Headers();
    /** @type {?} */
    var h = (/** @type {?} */ (init.headers));
    addRequestHeader.forEach((/**
     * @param {?} x
     * @return {?}
     */
    x => x((/**
     * @param {?} n
     * @param {?} v
     * @return {?}
     */
    (n, v) => h.append(n, v)))));
    init.credentials = 'include';
    /** @type {?} */
    let x = wrapFetch.wrap();
    return fetch(url, init).then((/**
     * @param {?} response
     * @return {?}
     */
    response => {
        x();
        return onSuccess$1(response);
    }), (/**
     * @param {?} error
     * @return {?}
     */
    error => {
        console.log(error);
        x();
        throw Promise.resolve(error);
    }));
}
/**
 * @param {?} response
 * @return {?}
 */
function onSuccess$1(response) {
    if (response.status == 204)
        return;
    if (response.status >= 200 && response.status < 300)
        return response.json();
    else
        throw response.json().then((/**
         * @param {?} x
         * @return {?}
         */
        x => {
            if (!x.message)
                x.message = response.statusText;
            return x;
        }));
}
/**
 * @param {?} error
 * @return {?}
 */
function onError$1(error) {
    throw Promise.resolve(error);
}
/**
 * @abstract
 * @template inParam, outParam
 */
class Action {
    /**
     * @param {?} serverUrl
     * @param {?=} actionUrl
     * @param {?=} addRequestHeader
     */
    constructor(serverUrl, actionUrl, addRequestHeader) {
        this.serverUrl = serverUrl;
        this.actionUrl = actionUrl;
        if (!addRequestHeader)
            addRequestHeader = (/**
             * @return {?}
             */
            () => { });
        if (!actionUrl) {
            this.actionUrl = this.constructor.name;
            if (this.actionUrl.endsWith('Action'))
                this.actionUrl = this.actionUrl.substring(0, this.actionUrl.length - 6);
        }
    }
    /**
     * @param {?} pIn
     * @return {?}
     */
    run(pIn) {
        return Action.provider.post(this.serverUrl + this.actionUrl, pIn);
    }
    /**
     * @param {?} reg
     * @return {?}
     */
    __register(reg) {
        reg(this.actionUrl, (/**
         * @param {?} d
         * @param {?} req
         * @param {?} res
         * @return {?}
         */
        (d, req, res) => __awaiter(this, void 0, void 0, function* () {
            try {
                /** @type {?} */
                var r = yield this.execute(d, req);
                res.success(r);
            }
            catch (err) {
                res.error(err);
            }
        })));
    }
}
Action.provider = new restDataProviderHttpProviderUsingFetch();

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class InMemoryDataProvider {
    constructor() {
        this.rows = {};
    }
    /**
     * @template T
     * @param {?} name
     * @param {?} factory
     * @return {?}
     */
    provideFor(name, factory) {
        if (!this.rows[name])
            this.rows[name] = [];
        return new ActualInMemoryDataProvider(factory, this.rows[name]);
    }
    /**
     * @return {?}
     */
    toString() { return "InMemoryDataProvider"; }
}
/**
 * @template T
 */
class ActualInMemoryDataProvider {
    /**
     * @param {?} factory
     * @param {?=} rows
     */
    constructor(factory, rows) {
        this.factory = factory;
        this.rows = rows;
        if (!rows)
            rows = [];
    }
    /**
     * @param {?=} where
     * @return {?}
     */
    count(where) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            let rows = this.rows;
            /** @type {?} */
            let j = 0;
            for (let i = 0; i < rows.length; i++) {
                if (!where) {
                    j++;
                }
                else {
                    /** @type {?} */
                    let x = new FilterConsumerBridgeToObject(rows[i]);
                    where.__applyToConsumer(x);
                    if (x.ok)
                        j++;
                }
            }
            return j;
        });
    }
    /**
     * @param {?=} options
     * @return {?}
     */
    find(options) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            let rows = this.rows;
            if (options) {
                if (options.where) {
                    rows = rows.filter((/**
                     * @param {?} i
                     * @return {?}
                     */
                    i => {
                        /** @type {?} */
                        let x = new FilterConsumerBridgeToObject(i);
                        options.where.__applyToConsumer(x);
                        return x.ok;
                    }));
                }
                if (options.orderBy) {
                    rows = rows.sort((/**
                     * @param {?} a
                     * @param {?} b
                     * @return {?}
                     */
                    (a, b) => {
                        /** @type {?} */
                        let r = 0;
                        for (let i = 0; i < options.orderBy.Segments.length; i++) {
                            /** @type {?} */
                            let seg = options.orderBy.Segments[i];
                            /** @type {?} */
                            let left = a[seg.column.jsonName];
                            /** @type {?} */
                            let right = b[seg.column.jsonName];
                            if (left > right)
                                r = 1;
                            else if (left < right)
                                r = -1;
                            if (r != 0) {
                                if (seg.descending)
                                    r *= -1;
                                return r;
                            }
                        }
                        return r;
                    }));
                }
                rows = pageArray(rows, options);
            }
            if (rows)
                return rows.map((/**
                 * @param {?} i
                 * @return {?}
                 */
                i => {
                    return this.map(i);
                }));
        });
    }
    /**
     * @param {?} i
     * @return {?}
     */
    map(i) {
        /** @type {?} */
        let r = JSON.parse(JSON.stringify(i));
        return r;
    }
    /**
     * @private
     * @param {?} id
     * @return {?}
     */
    idMatches(id) {
        if (!this.entity)
            this.entity = this.factory();
        /** @type {?} */
        let f = this.entity.__idColumn.isEqualTo(id);
        return (/**
         * @param {?} item
         * @return {?}
         */
        item => {
            /** @type {?} */
            let x = new FilterConsumerBridgeToObject(item);
            f.__applyToConsumer(x);
            return x.ok;
        });
    }
    /**
     * @param {?} id
     * @param {?} data
     * @return {?}
     */
    update(id, data) {
        /** @type {?} */
        let idMatches = this.idMatches(id);
        for (let i = 0; i < this.rows.length; i++) {
            if (idMatches(this.rows[i])) {
                this.rows[i] = Object.assign({}, this.rows[i], data);
                return Promise.resolve(this.map(this.rows[i]));
            }
        }
        throw new Error("couldn't find id to update: " + id);
    }
    /**
     * @param {?} id
     * @return {?}
     */
    delete(id) {
        /** @type {?} */
        let idMatches = this.idMatches(id);
        for (let i = 0; i < this.rows.length; i++) {
            if (idMatches(this.rows[i])) {
                this.rows.splice(i, 1);
                return Promise.resolve();
            }
        }
        throw new Error("couldn't find id to delete: " + id);
    }
    /**
     * @param {?} data
     * @return {?}
     */
    insert(data) {
        if (data.id)
            this.rows.forEach((/**
             * @param {?} i
             * @return {?}
             */
            i => {
                if (data.id == i.id)
                    throw Error("id already exists");
            }));
        this.rows.push(JSON.parse(JSON.stringify(data)));
        return Promise.resolve(JSON.parse(JSON.stringify(data)));
    }
}
/**
 * @param {?} rows
 * @param {?=} options
 * @return {?}
 */
function pageArray(rows, options) {
    if (!options)
        return rows;
    if (!options.limit)
        return rows;
    /** @type {?} */
    let page = 1;
    if (options.page)
        page = options.page;
    if (page < 1)
        page = 1;
    /** @type {?} */
    let x = 0;
    return rows.filter((/**
     * @param {?} i
     * @return {?}
     */
    i => {
        x++;
        /** @type {?} */
        let max = page * options.limit;
        /** @type {?} */
        let min = max - options.limit;
        return x > min && x <= max;
    }));
}
class FilterConsumerBridgeToObject {
    /**
     * @param {?} row
     */
    constructor(row) {
        this.row = row;
        this.ok = true;
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isEqualTo(col, val) {
        if (this.row[col.jsonName] != val)
            this.ok = false;
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isDifferentFrom(col, val) {
        if (this.row[col.jsonName] == val)
            this.ok = false;
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isGreaterOrEqualTo(col, val) {
        if (this.row[col.jsonName] < val)
            this.ok = false;
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isGreaterThan(col, val) {
        if (this.row[col.jsonName] <= val)
            this.ok = false;
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isLessOrEqualTo(col, val) {
        if (this.row[col.jsonName] > val)
            this.ok = false;
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isLessThan(col, val) {
        if (this.row[col.jsonName] >= val)
            this.ok = false;
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isContains(col, val) {
        /** @type {?} */
        let v = this.row[col.jsonName];
        if (!v) {
            this.ok = false;
            return;
        }
        /** @type {?} */
        let s = '' + v;
        if (s.indexOf(val) < 0)
            this.ok = false;
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    isStartsWith(col, val) {
        /** @type {?} */
        let v = this.row[col.jsonName];
        if (!v) {
            this.ok = false;
            return;
        }
        /** @type {?} */
        let s = '' + v;
        if (s.indexOf(val) != 0)
            this.ok = false;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class Context {
    /**
     * @param {?=} http
     */
    constructor(http) {
        this._onServer = false;
        this.cache = {};
        this._lookupCache = [];
        if (http instanceof HttpClient) {
            /** @type {?} */
            var prov = new angularHttpProvider(http);
            this._dataSource = new RestDataProvider(Context.apiBaseUrl, prov
            //,new restDataProviderHttpProviderUsingFetch()
            );
            Action.provider = prov;
        }
        else {
            this._dataSource = new InMemoryDataProvider();
        }
    }
    /**
     * @return {?}
     */
    clearAllCache() {
        this.cache = {};
        this._lookupCache = [];
    }
    /**
     * @return {?}
     */
    isSignedIn() {
        return !!this.user;
    }
    /**
     * @return {?}
     */
    get onServer() {
        return this._onServer;
    }
    /**
     * @return {?}
     */
    get user() { return this._user; }
    /**
     * @param {?} info
     * @return {?}
     */
    _setUser(info) {
        this._user = info;
    }
    /**
     * @param {?} roles
     * @return {?}
     */
    isAllowed(roles) {
        if (roles == undefined)
            return undefined;
        if (roles instanceof Array) {
            for (const role of roles) {
                if (this.isAllowed(role) === true) {
                    return true;
                }
            }
            return false;
        }
        if (isFunction(roles)) {
            return ((/** @type {?} */ (roles)))(this);
        }
        if (isBoolean(roles))
            return roles;
        if (roles instanceof Role) {
            roles = roles.key;
        }
        if (!this.user)
            return false;
        if (isString(roles))
            if (this.user.roles.indexOf(roles.toString()) >= 0)
                return true;
        return false;
    }
    /**
     * @template lookupIdType, T
     * @param {?} c
     * @return {?}
     */
    create(c) {
        /** @type {?} */
        let e = new c(this);
        e.setSource(this._dataSource);
        e._setContext(this);
        return e;
    }
    /**
     * @template lookupIdType, T
     * @param {?} c
     * @return {?}
     */
    for(c) {
        /** @type {?} */
        let classType = (/** @type {?} */ (c));
        if (this.cache[classType.__key])
            return (/** @type {?} */ (this.cache[classType.__key]));
        return this.cache[classType.__key] = new SpecificEntityHelper(this.create(c), this._lookupCache, this);
    }
}
Context.apiBaseUrl = 'api';
Context.decorators = [
    { type: Injectable }
];
/** @nocollapse */
Context.ctorParameters = () => [
    { type: HttpClient }
];
class ServerContext extends Context {
    constructor() {
        super(undefined);
        this._onServer = true;
    }
    /**
     * @param {?} req
     * @return {?}
     */
    setReq(req) {
        this.req = req;
        this._user = req.user ? req.user : undefined;
    }
    /**
     * @param {?} dataProvider
     * @return {?}
     */
    setDataProvider(dataProvider) {
        this._dataSource = dataProvider;
    }
    /**
     * @return {?}
     */
    getOrigin() {
        return this.req.getHeader('origin');
    }
}
/**
 * @template lookupIdType, T
 */
class SpecificEntityHelper {
    /**
     * @param {?} entity
     * @param {?} _lookupCache
     * @param {?} context
     */
    constructor(entity, _lookupCache, context) {
        this.entity = entity;
        this._lookupCache = _lookupCache;
        this.context = context;
    }
    /**
     * @param {?} filter
     * @return {?}
     */
    lookup(filter) {
        /** @type {?} */
        let key = this.entity.__getName();
        /** @type {?} */
        let lookup;
        this._lookupCache.forEach((/**
         * @param {?} l
         * @return {?}
         */
        l => {
            if (l.key == key)
                lookup = l.lookup;
        }));
        if (!lookup) {
            lookup = new Lookup(this.entity);
            this._lookupCache.push({ key, lookup });
        }
        return lookup.get(filter);
    }
    /**
     * @param {?} filter
     * @return {?}
     */
    lookupAsync(filter) {
        /** @type {?} */
        let key = this.entity.__getName();
        /** @type {?} */
        let lookup;
        this._lookupCache.forEach((/**
         * @param {?} l
         * @return {?}
         */
        l => {
            if (l.key == key)
                lookup = l.lookup;
        }));
        if (!lookup) {
            lookup = new Lookup(this.entity);
            this._lookupCache.push({ key, lookup });
        }
        return lookup.whenGet(filter);
    }
    /**
     * @param {?=} where
     * @return {?}
     */
    count(where) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            let dl = new DataList(this.entity);
            return yield dl.count(where);
        });
    }
    /**
     * @param {?} where
     * @param {?=} what
     * @return {?}
     */
    foreach(where, what) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            let options = {};
            if (where) {
                options.where = where(this.entity);
            }
            /** @type {?} */
            let items = yield this.entity.source.find(options);
            for (const item of items) {
                yield what(item);
            }
        });
    }
    /**
     * @param {?=} options
     * @return {?}
     */
    find(options) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            let dl = new DataList(this.entity);
            return yield dl.get(options);
        });
    }
    /**
     * @param {?=} where
     * @return {?}
     */
    findFirst(where) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            let r = yield this.entity.source.find({ where: where ? where(this.entity) : undefined });
            if (r.length == 0)
                return undefined;
            return r[0];
        });
    }
    /**
     * @param {?} items
     * @return {?}
     */
    toPojoArray(items) {
        /** @type {?} */
        let exc = new ColumnHashSet();
        exc.add(...this.entity._getExcludedColumns(this.entity, this.context));
        return Promise.all(items.map((/**
         * @param {?} f
         * @return {?}
         */
        f => f.__toPojo(exc))));
    }
    /**
     * @return {?}
     */
    create() {
        return this.entity.source.createNewItem();
    }
    /**
     * @param {?=} settings
     * @return {?}
     */
    gridSettings(settings) {
        return new GridSettings(this.entity, settings);
    }
}
/** @type {?} */
const allEntities = [];
/**
 * @template T
 * @param {?} theEntityClass
 * @return {?}
 */
function EntityClass(theEntityClass) {
    /** @type {?} */
    let original = theEntityClass;
    /** @type {?} */
    let f = class extends theEntityClass {
        /**
         * @param {...?} args
         */
        constructor(...args) {
            super(...args);
            this._setFactoryClassAndDoInitColumns(f);
            if (!this.__options.name) {
                this.__options.name = original.name;
            }
        }
    };
    allEntities.push(f);
    f.__key = theEntityClass.name + allEntities.indexOf(f);
    return f;
}
/**
 * @abstract
 */
class DirectSQL {
}
// @dynamic
class Role {
    /**
     * @param {?} key
     */
    constructor(key) {
        this.key = key;
    }
    /**
     * @param {?} allowed
     * @return {?}
     */
    static not(allowed) {
        return (/**
         * @param {?} c
         * @return {?}
         */
        c => !c.isAllowed(allowed));
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const authToken = 'authorization';
class JwtSessionManager {
    /**
     * @param {?} context
     */
    constructor(context) {
        this.context = context;
    }
    /**
     * @return {?}
     */
    loadSessionFromCookie() {
        /** @type {?} */
        let c = document.cookie;
        /** @type {?} */
        let i = c.indexOf(authToken + '=');
        if (i >= 0) {
            c = c.substring(i + authToken.length + 2).trim();
            i = c.indexOf(';');
            if (i >= 0) {
                c = c.substring(0, i - 1);
            }
            this._setToken(c);
        }
    }
    /**
     * @param {?} jwtToken
     * @param {?=} rememberOnDevice
     * @return {?}
     */
    setToken(jwtToken, rememberOnDevice = false) {
        return __awaiter(this, void 0, void 0, function* () {
            if (jwtToken) {
                this._setToken(jwtToken);
                /** @type {?} */
                let c = authToken + "=" + jwtToken;
                c += ';path=/';
                if (rememberOnDevice)
                    c += '; expires = Thu, 01 Jan 2076 00:00:00 GMT';
                document.cookie = c;
                return true;
            }
            else
                this.signout();
            return false;
        });
    }
    /**
     * @private
     * @param {?} token
     * @return {?}
     */
    _setToken(token) {
        this.currentToken = token;
        /** @type {?} */
        let user = undefined;
        if (this.currentToken) {
            {
                try {
                    user = new JwtHelperService().decodeToken(token);
                }
                catch (err) {
                    console.log(err);
                }
            }
        }
        this.context._setUser(user);
        if (this.tokenInfoChanged)
            this.tokenInfoChanged();
    }
    /**
     * @return {?}
     */
    signout() {
        this._setToken('');
        document.cookie = authToken + '=; expires = Thu, 01 Jan 1970 00:00:00 GMT';
    }
}
JwtSessionManager.decorators = [
    { type: Injectable }
];
/** @nocollapse */
JwtSessionManager.ctorParameters = () => [
    { type: Context }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class RouteHelperService {
    /**
     * @param {?} router
     * @param {?} injector
     */
    constructor(router, injector) {
        this.router = router;
        this.injector = injector;
    }
    /**
     * @param {?} toComponent
     * @return {?}
     */
    navigateToComponent(toComponent) {
        /** @type {?} */
        let done = false;
        this.router.config.forEach((/**
         * @param {?} path
         * @return {?}
         */
        path => {
            if (done)
                return;
            if (path.component == toComponent) {
                done = true;
                this.router.navigate(['/' + path.path]);
            }
        }));
        if (!done)
            console.warn("couldn't find path for ", toComponent, this.router.config);
    }
    /**
     * @param {?} route
     * @return {?}
     */
    canNavigateToRoute(route) {
        if (!route.canActivate)
            return true;
        for (let guard of route.canActivate) {
            /** @type {?} */
            let g = (/** @type {?} */ (this.injector.get(guard)));
            if (g && g.canActivate) {
                /** @type {?} */
                var r = new dummyRoute();
                r.routeConfig = route;
                /** @type {?} */
                let canActivate = g.canActivate(r, undefined);
                if (!canActivate)
                    return false;
            }
        }
        return true;
    }
}
RouteHelperService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
RouteHelperService.ctorParameters = () => [
    { type: Router },
    { type: Injector }
];
// @dynamic
class SignedInGuard {
    /**
     * @param {?} context
     * @param {?} router
     * @param {?} helper
     */
    constructor(context, router, helper) {
        this.context = context;
        this.router = router;
        this.helper = helper;
    }
    /**
     * @return {?}
     */
    isAllowed() {
        return true;
    }
    /**
     * @param {?} route
     * @return {?}
     */
    canActivate(route) {
        if (this.context.isSignedIn() && this.context.isAllowed(this.isAllowed())) {
            return true;
        }
        if (!(route instanceof dummyRoute)) {
            /** @type {?} */
            let x = SignedInGuard.componentToNavigateIfNotAllowed;
            if (x != undefined) {
                this.helper.navigateToComponent(x);
            }
            else
                this.router.navigate(['/']);
        }
        return false;
    }
}
SignedInGuard.decorators = [
    { type: Injectable }
];
/** @nocollapse */
SignedInGuard.ctorParameters = () => [
    { type: Context },
    { type: Router },
    { type: RouteHelperService }
];
class NotSignedInGuard {
    /**
     * @param {?} context
     * @param {?} router
     */
    constructor(context, router) {
        this.context = context;
        this.router = router;
    }
    /**
     * @param {?} route
     * @return {?}
     */
    canActivate(route) {
        if (this.context.isSignedIn())
            return false;
        return true;
    }
}
NotSignedInGuard.decorators = [
    { type: Injectable }
];
/** @nocollapse */
NotSignedInGuard.ctorParameters = () => [
    { type: Context },
    { type: Router }
];
class dummyRoute extends ActivatedRouteSnapshot {
    constructor() {
        super();
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class WaitComponent {
    constructor() { }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
}
WaitComponent.decorators = [
    { type: Component, args: [{
                selector: 'app-wait',
                template: "<mat-progress-spinner [mode]=\"'indeterminate'\" >\r\n</mat-progress-spinner>",
                styles: [""]
            }] }
];
/** @nocollapse */
WaitComponent.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
class BusyService {
    /**
     * @param {?} dialog
     */
    constructor(dialog) {
        this.dialog = dialog;
        this.id = 0;
        this.numOfWaits = 0;
        this.disableWait = false;
        BusyService.singleInstance = this;
    }
    /**
     * @template t
     * @param {?} what
     * @return {?}
     */
    donotWait(what) {
        return __awaiter(this, void 0, void 0, function* () {
            this.disableWait = true;
            try {
                return (yield what());
            }
            finally {
                this.disableWait = false;
            }
        });
    }
    /**
     * @template t
     * @param {?} what
     * @return {?}
     */
    donotWaitNonAsync(what) {
        this.disableWait = true;
        try {
            return (what());
        }
        finally {
            this.disableWait = false;
        }
    }
    /**
     * @param {?} id
     * @param {?} what
     * @return {?}
     */
    log(id, what) {
        //console.log(what + ' id:' + this.id + ' w:' + this.numOfWaits);
    }
    /**
     * @template t
     * @param {?} what
     * @return {?}
     */
    doWhileShowingBusy(what) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            let x = this.showBusy();
            try {
                return yield what();
            }
            finally {
                x();
            }
        });
    }
    /**
     * @return {?}
     */
    showBusy() {
        /** @type {?} */
        let id = this.id++;
        if (this.disableWait)
            return (/**
             * @return {?}
             */
            () => { });
        this.log(id, 'start busy ');
        if (this.numOfWaits == 0) {
            setTimeout((/**
             * @return {?}
             */
            () => {
                if (this.numOfWaits > 0 && !this.waitRef) {
                    this.log(id, 'actual start busy ');
                    this.waitRef = this.dialog.open(WaitComponent, { disableClose: true });
                }
            }), 300);
        }
        this.numOfWaits++;
        return (/**
         * @return {?}
         */
        () => {
            this.numOfWaits--;
            this.log(id, 'close busy ');
            if (this.numOfWaits == 0) {
                this.log(id, 'close top busy ');
                if (this.waitRef) {
                    this.log(id, 'actual close top busy ');
                    this.waitRef.close();
                    this.waitRef = undefined;
                }
            }
        });
    }
}
BusyService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
BusyService.ctorParameters = () => [
    { type: MatDialog }
];
class LoaderInterceptor {
    /**
     * @param {?} busy
     */
    constructor(busy) {
        this.busy = busy;
    }
    /**
     * @param {?} req
     * @param {?} next
     * @return {?}
     */
    intercept(req, next) {
        /** @type {?} */
        let x = this.busy.showBusy();
        return next.handle(req).pipe(finalize((/**
         * @return {?}
         */
        () => x())));
    }
}
LoaderInterceptor.decorators = [
    { type: Injectable }
];
/** @nocollapse */
LoaderInterceptor.ctorParameters = () => [
    { type: BusyService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DataArea2Compnent {
    constructor() {
        this.settings = { columns: new ColumnCollection((/**
             * @return {?}
             */
            () => undefined), (/**
             * @return {?}
             */
            () => false), undefined, (/**
             * @return {?}
             */
            () => true)), lines: [] };
    }
}
DataArea2Compnent.decorators = [
    { type: Component, args: [{
                selector: 'data-area2',
                template: "<ng-container *ngFor=\"let line of settings.lines\">\r\n  \r\n    <div class=\"one-line\">\r\n        <data-control2 [settings]=\"settings.columns\" [map]=\"map\" [record]=\"settings.columns.currentRow()\"\r\n            *ngFor=\"let map of line\">\r\n        </data-control2>\r\n    </div>\r\n</ng-container>\r\n<div  *ngIf=\"!settings.lines\">\r\n    <data-control2 [settings]=\"settings.columns\" [map]=\"map\" [record]=\"settings.columns.currentRow()\"\r\n        *ngFor=\"let map of settings.columns.items\">\r\n    </data-control2>\r\n</div>\r\n",
                encapsulation: ViewEncapsulation.None,
                styles: [".one-line{display:flex;justify-content:space-between}.one-line>*{padding-left:.2em;padding-right:.2em;flex-basis:1px;flex-grow:1}.one-line>:first-child{padding-right:0}.one-line>:last-child{padding-left:0}"]
            }] }
];
DataArea2Compnent.propDecorators = {
    settings: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class RadWebModule {
}
RadWebModule.decorators = [
    { type: NgModule, args: [{
                declarations: [DataControlComponent, DataControl2Component, DataArea2Compnent, DataGridComponent, ColumnDesigner, DataFilterInfoComponent, DataAreaCompnent, DataGrid2Component, WaitComponent, DataControl3Component],
                imports: [FormsModule, CommonModule, HttpClientModule, MatProgressSpinnerModule, MatDialogModule, BrowserAnimationsModule,
                    MatFormFieldModule,
                    MatButtonModule,
                    MatInputModule, MatIconModule, ReactiveFormsModule, MatCheckboxModule],
                providers: [Context, JwtSessionManager, NotSignedInGuard, SignedInGuard, RouteHelperService,
                    BusyService,
                    { provide: HTTP_INTERCEPTORS, useClass: LoaderInterceptor, multi: true }],
                exports: [DataControlComponent, DataControl2Component, DataGridComponent, ColumnDesigner, DataFilterInfoComponent, DataAreaCompnent, DataGrid2Component, DataArea2Compnent],
                entryComponents: [WaitComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LogSQLConnectionProvider {
    /**
     * @param {?} origin
     * @param {?} allQueries
     */
    constructor(origin, allQueries) {
        this.origin = origin;
        this.allQueries = allQueries;
    }
    /**
     * @return {?}
     */
    createCommand() {
        return new LogSQLCommand(this.origin.createCommand(), this.allQueries);
    }
}
class LogSQLCommand {
    /**
     * @param {?} origin
     * @param {?} allQueries
     */
    constructor(origin, allQueries) {
        this.origin = origin;
        this.allQueries = allQueries;
        this.args = {};
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    addParameterToCommandAndReturnParameterName(col, val) {
        /** @type {?} */
        let r = this.origin.addParameterToCommandAndReturnParameterName(col, val);
        this.args[r] = val;
        return r;
    }
    /**
     * @param {?} sql
     * @return {?}
     */
    query(sql) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.allQueries) {
                console.log('Query:', sql);
                console.log("Arguments:", this.args);
            }
            try {
                return yield this.origin.query(sql);
            }
            catch (err) {
                console.log('Query:', sql);
                console.log("Arguments", this.args);
                console.log('Error:', err);
                throw err;
            }
        });
    }
}
// @dynamic
/**
 * @template T
 */
class ActualSQLServerDataProvider {
    /**
     * @param {?} entityFactory
     * @param {?} name
     * @param {?} sql
     * @param {?} factory
     */
    constructor(entityFactory, name, sql, factory) {
        this.entityFactory = entityFactory;
        this.name = name;
        this.sql = sql;
        this.factory = factory;
        this.sql = ActualSQLServerDataProvider.decorateSqlConnectionProvider(sql);
    }
    /**
     * @param {?} sql
     * @return {?}
     */
    static decorateSqlConnectionProvider(sql) {
        return new LogSQLConnectionProvider(sql, ActualSQLServerDataProvider.LogToConsole);
    }
    /**
     * @return {?}
     */
    createDirectSQLCommand() {
        return this.sql.createCommand();
    }
    /**
     * @param {?} where
     * @return {?}
     */
    count(where) {
        if (!this.entity)
            this.entity = this.entityFactory();
        /** @type {?} */
        let select = 'select count(*) from ' + this.entity.__getDbName();
        /** @type {?} */
        let r = this.sql.createCommand();
        if (where) {
            /** @type {?} */
            let wc = new FilterConsumerBridgeToSqlRequest(r);
            where.__applyToConsumer(wc);
            select += wc.where;
        }
        return r.query(select).then((/**
         * @param {?} r
         * @return {?}
         */
        r => {
            return r.rows[0].count;
        }));
    }
    /**
     * @param {?=} options
     * @return {?}
     */
    find(options) {
        if (!this.entity)
            this.entity = this.entityFactory();
        /** @type {?} */
        let select = 'select ';
        /** @type {?} */
        let colKeys = [];
        this.entity.__iterateColumns().forEach((/**
         * @param {?} x
         * @return {?}
         */
        x => {
            if (x.__isVirtual()) ;
            else {
                if (colKeys.length > 0)
                    select += ', ';
                select += x.__getDbName();
                colKeys.push(x);
            }
        }));
        select += ' from ' + this.entity.__getDbName();
        /** @type {?} */
        let r = this.sql.createCommand();
        if (options) {
            if (options.where) {
                /** @type {?} */
                let where = new FilterConsumerBridgeToSqlRequest(r);
                options.where.__applyToConsumer(where);
                select += where.where;
            }
            if (options.orderBy) {
                /** @type {?} */
                let first = true;
                options.orderBy.Segments.forEach((/**
                 * @param {?} c
                 * @return {?}
                 */
                c => {
                    if (first) {
                        select += ' Order By ';
                        first = false;
                    }
                    else
                        select += ', ';
                    select += c.column.__getDbName();
                    if (c.descending)
                        select += ' desc';
                }));
            }
            if (options.limit) {
                /** @type {?} */
                let page = 1;
                if (options.page)
                    page = options.page;
                if (page < 1)
                    page = 1;
                select += ' limit ' + options.limit + ' offset ' + (page - 1) * options.limit;
            }
        }
        return r.query(select).then((/**
         * @param {?} r
         * @return {?}
         */
        r => {
            return r.rows.map((/**
             * @param {?} y
             * @return {?}
             */
            y => {
                /** @type {?} */
                let result = {};
                for (let x in y) {
                    /** @type {?} */
                    let col = colKeys[r.getColumnIndex(x)];
                    result[col.jsonName] = col.__getStorage().fromDb(y[x]);
                }
                return result;
            }));
        }));
    }
    /**
     * @param {?} id
     * @param {?} data
     * @return {?}
     */
    update(id, data) {
        if (!this.entity)
            this.entity = this.entityFactory();
        /** @type {?} */
        let r = this.sql.createCommand();
        /** @type {?} */
        let f = new FilterConsumerBridgeToSqlRequest(r);
        this.entity.__idColumn.isEqualTo(id).__applyToConsumer(f);
        /** @type {?} */
        let statement = 'update ' + this.entity.__getDbName() + ' set ';
        /** @type {?} */
        let added = false;
        /** @type {?} */
        let resultFilter = this.entity.__idColumn.isEqualTo(id);
        if (data.id != undefined)
            resultFilter = this.entity.__idColumn.isEqualTo(data.id);
        this.entity.__iterateColumns().forEach((/**
         * @param {?} x
         * @return {?}
         */
        x => {
            if (x instanceof CompoundIdColumn) {
                resultFilter = x.resultIdFilter(id, data);
            }
            if (x.__dbReadOnly()) ;
            else {
                /** @type {?} */
                let v = x.__getStorage().toDb(data[x.jsonName]);
                if (v != undefined) {
                    if (!added)
                        added = true;
                    else
                        statement += ', ';
                    statement += x.__getDbName() + ' = ' + r.addParameterToCommandAndReturnParameterName(x, v);
                }
            }
        }));
        statement += f.where;
        return r.query(statement).then((/**
         * @return {?}
         */
        () => {
            return this.find({ where: resultFilter }).then((/**
             * @param {?} y
             * @return {?}
             */
            y => y[0]));
        }));
    }
    /**
     * @param {?} id
     * @return {?}
     */
    delete(id) {
        if (!this.entity)
            this.entity = this.entityFactory();
        /** @type {?} */
        let r = this.sql.createCommand();
        /** @type {?} */
        let f = new FilterConsumerBridgeToSqlRequest(r);
        this.entity.__idColumn.isEqualTo(id).__applyToConsumer(f);
        /** @type {?} */
        let statement = 'delete from ' + this.entity.__getDbName();
        statement += f.where;
        return r.query(statement).then((/**
         * @return {?}
         */
        () => {
            return this.find({ where: this.entity.__idColumn.isEqualTo(id) }).then((/**
             * @param {?} y
             * @return {?}
             */
            y => y[0]));
        }));
    }
    /**
     * @param {?} data
     * @return {?}
     */
    insert(data) {
        if (!this.entity)
            this.entity = this.entityFactory();
        /** @type {?} */
        let r = this.sql.createCommand();
        /** @type {?} */
        let cols = '';
        /** @type {?} */
        let vals = '';
        /** @type {?} */
        let added = false;
        /** @type {?} */
        let resultFilter = this.entity.__idColumn.isEqualTo(data[this.entity.__idColumn.jsonName]);
        this.entity.__iterateColumns().forEach((/**
         * @param {?} x
         * @return {?}
         */
        (x) => {
            if (x instanceof CompoundIdColumn) {
                resultFilter = x.resultIdFilter(undefined, data);
            }
            if (x.__dbReadOnly()) ;
            else {
                /** @type {?} */
                let v = x.__getStorage().toDb(data[x.jsonName]);
                if (v != undefined) {
                    if (!added)
                        added = true;
                    else {
                        cols += ', ';
                        vals += ', ';
                    }
                    cols += x.__getDbName();
                    vals += r.addParameterToCommandAndReturnParameterName(x, v);
                }
            }
        }));
        /** @type {?} */
        let statement = `insert into ${this.entity.__getDbName()} (${cols}) values (${vals})`;
        return r.query(statement).then((/**
         * @return {?}
         */
        () => {
            return this.find({ where: resultFilter }).then((/**
             * @param {?} y
             * @return {?}
             */
            y => {
                return y[0];
            }));
        }));
    }
}
ActualSQLServerDataProvider.LogToConsole = false;

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class WebSqlDataProvider {
    /**
     * @param {?} databaseName
     */
    constructor(databaseName) {
        this.databaseName = databaseName;
        //@ts-ignore
        this.db = window.openDatabase(databaseName, '1.0', databaseName, 2 * 1024 * 1024);
    }
    /**
     * @template T
     * @param {?} name
     * @param {?} factory
     * @return {?}
     */
    provideFor(name, factory) {
        return new ActualSQLServerDataProvider((/**
         * @return {?}
         */
        () => {
            /** @type {?} */
            var e = factory();
            /** @type {?} */
            let result = '';
            e.__iterateColumns().forEach((/**
             * @param {?} x
             * @return {?}
             */
            x => {
                if (!x.__dbReadOnly()) {
                    if (result.length != 0)
                        result += ',';
                    result += '\r\n  ';
                    result += this.addColumnSqlSyntax(x);
                    if (x == e.__idColumn)
                        result += ' primary key';
                }
            }));
            this.db.transaction((/**
             * @param {?} t
             * @return {?}
             */
            t => t.executeSql('create table if not exists ' + e.__getDbName() + ' (' + result + '\r\n)')));
            return e;
        }), name, new WebSqlBridgeToSQLConnection(this.db), factory);
    }
    /**
     * @private
     * @param {?} x
     * @return {?}
     */
    addColumnSqlSyntax(x) {
        /** @type {?} */
        let result = x.__getDbName();
        if (x instanceof DateTimeColumn)
            result += " integer";
        else if (x instanceof DateColumn)
            result += " integer";
        else if (x instanceof BoolColumn)
            result += " integer default 0 not null";
        else if (x instanceof NumberColumn) {
            if (x.__numOfDecimalDigits == 0)
                result += " integer default 0 not null";
            else
                result += ' real default 0 not null';
        }
        else if (x instanceof ClosedListColumn) {
            result += ' integer default 0 not null';
        }
        else
            result += " text default '' not null ";
        return result;
    }
    /**
     * @return {?}
     */
    toString() { return "WebSqlDataProvider"; }
}
class WebSqlBridgeToSQLConnection {
    //@ts-ignore
    /**
     * @param {?} source
     */
    constructor(source) {
        this.source = source;
    }
    /**
     * @return {?}
     */
    createCommand() {
        return new WebSqlBridgeToSQLCommand(this.source);
    }
}
class WebSqlBridgeToSQLCommand {
    //@ts-ignore
    /**
     * @param {?} source
     */
    constructor(source) {
        this.source = source;
        this.values = [];
    }
    /**
     * @param {?} col
     * @param {?} val
     * @return {?}
     */
    addParameterToCommandAndReturnParameterName(col, val) {
        this.values.push(val);
        return '~' + this.values.length + '~';
    }
    /**
     * @param {?} sql
     * @return {?}
     */
    query(sql) {
        return new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        (resolve, reject) => this.source.transaction((/**
         * @param {?} t
         * @return {?}
         */
        t => {
            /** @type {?} */
            let s = sql;
            /** @type {?} */
            let v = [];
            /** @type {?} */
            var m = s.match(/~\d+~/g);
            if (m != null)
                m.forEach((/**
                 * @param {?} mr
                 * @return {?}
                 */
                mr => {
                    s = s.replace(mr, '?');
                    v.push(this.values[Number.parseInt(mr.substring(1, mr.length - 1)) - 1]);
                }));
            t.executeSql(s, v, (/**
             * @param {?} t1
             * @param {?} r
             * @return {?}
             */
            (t1, r) => resolve(new WebSqlBridgeToSQLQueryResult(r))), (/**
             * @param {?} t2
             * @param {?} err
             * @return {?}
             */
            (t2, err) => {
                reject(err.message);
                return undefined;
            }));
        }))));
    }
}
class WebSqlBridgeToSQLQueryResult {
    //@ts-ignore
    /**
     * @param {?} r
     */
    constructor(r) {
        this.r = r;
        this.rows = [];
        for (let i = 0; i < r.rows.length; i++) {
            this.rows.push(r.rows.item(i));
        }
    }
    /**
     * @param {?} index
     * @return {?}
     */
    getcolumnNameAtIndex(index) {
        return undefined;
    }
    /**
     * @param {?} name
     * @return {?}
     */
    getColumnIndex(name) {
        if (this.rows.length == 0)
            return -1;
        /** @type {?} */
        let i = 0;
        for (let m in this.rows[0]) {
            if (m == name)
                return i;
            i++;
        }
        return -1;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class JsonStorageDataProvider {
    /**
     * @param {?} helper
     */
    constructor(helper) {
        this.helper = helper;
    }
    /**
     * @param {?=} options
     * @return {?}
     */
    find(options) {
        return this.helper.doWork((/**
         * @param {?} dp
         * @param {?} save
         * @return {?}
         */
        (dp, save) => dp.find(options)));
    }
    /**
     * @param {?=} where
     * @return {?}
     */
    count(where) {
        return this.helper.doWork((/**
         * @param {?} dp
         * @param {?} save
         * @return {?}
         */
        (dp, save) => dp.count(where)));
    }
    /**
     * @param {?} id
     * @param {?} data
     * @return {?}
     */
    update(id, data) {
        return this.helper.doWork((/**
         * @param {?} dp
         * @param {?} save
         * @return {?}
         */
        (dp, save) => dp.update(id, data).then((/**
         * @param {?} x
         * @return {?}
         */
        x => {
            save();
            return x;
        }))));
    }
    /**
     * @param {?} id
     * @return {?}
     */
    delete(id) {
        return this.helper.doWork((/**
         * @param {?} dp
         * @param {?} save
         * @return {?}
         */
        (dp, save) => dp.delete(id).then((/**
         * @param {?} x
         * @return {?}
         */
        x => {
            save();
            return x;
        }))));
    }
    /**
     * @param {?} data
     * @return {?}
     */
    insert(data) {
        return this.helper.doWork((/**
         * @param {?} dp
         * @param {?} save
         * @return {?}
         */
        (dp, save) => dp.insert(data).then((/**
         * @param {?} x
         * @return {?}
         */
        x => {
            save();
            return x;
        }))));
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class DataApi {
    /**
     * @param {?} rowType
     * @param {?=} options
     */
    constructor(rowType, options) {
        this.rowType = rowType;
        this.options = options;
        this.excludedColumns = new ColumnHashSet();
        this.readonlyColumns = new ColumnHashSet();
        if (!options)
            this.options = {};
        if (this.options.readonlyColumns)
            this.readonlyColumns.add(...this.options.readonlyColumns(rowType));
        if (this.options.excludeColumns) {
            this.excludedColumns.add(...this.options.excludeColumns(rowType));
            this.readonlyColumns.add(...this.options.excludeColumns(rowType));
        }
    }
    /**
     * @return {?}
     */
    getRoute() {
        if (!this.options.name)
            return this.rowType.__getName();
        return this.options.name;
    }
    /**
     * @param {?} response
     * @param {?} id
     * @return {?}
     */
    get(response, id) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.options.allowRead == false) {
                response.methodNotAllowed();
                return;
            }
            yield this.doOnId(response, id, (/**
             * @param {?} row
             * @return {?}
             */
            (row) => __awaiter(this, void 0, void 0, function* () { return response.success(yield row.__toPojo(this.excludedColumns)); })));
        });
    }
    /**
     * @param {?} response
     * @param {?} request
     * @return {?}
     */
    count(response, request) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                /** @type {?} */
                let where = this.buildWhere(request);
                response.success({ count: +(yield this.rowType.source.count(where)) });
            }
            catch (err) {
                response.error(err);
            }
        });
    }
    /**
     * @param {?} response
     * @param {?} request
     * @return {?}
     */
    getArray(response, request) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.options.allowRead == false) {
                response.methodNotAllowed();
                return;
            }
            try {
                /** @type {?} */
                let findOptions = {};
                if (this.options && this.options.get) {
                    Object.assign(findOptions, this.options.get);
                }
                findOptions.where = this.buildWhere(request);
                if (request) {
                    /** @type {?} */
                    let sort = (/** @type {?} */ (request.get("_sort")));
                    if (sort != undefined) {
                        /** @type {?} */
                        let dir = request.get('_order');
                        /** @type {?} */
                        let dirItems = [];
                        if (dir)
                            dirItems = dir.split(',');
                        findOptions.orderBy = new Sort();
                        sort.split(',').forEach((/**
                         * @param {?} name
                         * @param {?} i
                         * @return {?}
                         */
                        (name, i) => {
                            /** @type {?} */
                            let col = this.rowType.__getColumnByJsonName(name.trim());
                            if (col) {
                                findOptions.orderBy.Segments.push({
                                    column: col,
                                    descending: i < dirItems.length && dirItems[i].toLowerCase().trim().startsWith("d")
                                });
                            }
                        }));
                    }
                    /** @type {?} */
                    let limit = +request.get("_limit");
                    if (!limit)
                        limit = 25;
                    findOptions.limit = limit;
                    findOptions.page = +request.get("_page");
                }
                yield this.rowType.source.find(findOptions)
                    .then((/**
                 * @param {?} r
                 * @return {?}
                 */
                (r) => __awaiter(this, void 0, void 0, function* () {
                    response.success(yield Promise.all(r.map((/**
                     * @param {?} y
                     * @return {?}
                     */
                    (y) => __awaiter(this, void 0, void 0, function* () { return yield y.__toPojo(this.excludedColumns); })))));
                })));
            }
            catch (err) {
                response.error(err);
            }
        });
    }
    /**
     * @private
     * @param {?} request
     * @return {?}
     */
    buildWhere(request) {
        /** @type {?} */
        var where;
        if (this.options && this.options.get && this.options.get.where)
            where = this.options.get.where(this.rowType);
        if (request) {
            this.rowType.__iterateColumns().forEach((/**
             * @param {?} col
             * @return {?}
             */
            col => {
                /**
                 * @param {?} key
                 * @param {?} theFilter
                 * @return {?}
                 */
                function addFilter(key, theFilter) {
                    /** @type {?} */
                    let val = request.get(col.jsonName + key);
                    if (val != undefined) {
                        /** @type {?} */
                        let addFilter = (/**
                         * @param {?} val
                         * @return {?}
                         */
                        (val) => {
                            /** @type {?} */
                            let f = theFilter(col.fromRawValue(val));
                            if (f) {
                                if (where)
                                    where = new AndFilter(where, f);
                                else
                                    where = f;
                            }
                        });
                        if (val instanceof Array) {
                            val.forEach((/**
                             * @param {?} v
                             * @return {?}
                             */
                            v => {
                                addFilter(v);
                            }));
                        }
                        else
                            addFilter(val);
                    }
                }
                addFilter('', (/**
                 * @param {?} val
                 * @return {?}
                 */
                val => col.isEqualTo(val)));
                addFilter('_gt', (/**
                 * @param {?} val
                 * @return {?}
                 */
                val => col.isGreaterThan(val)));
                addFilter('_gte', (/**
                 * @param {?} val
                 * @return {?}
                 */
                val => col.isGreaterOrEqualTo(val)));
                addFilter('_lt', (/**
                 * @param {?} val
                 * @return {?}
                 */
                val => col.isLessThan(val)));
                addFilter('_lte', (/**
                 * @param {?} val
                 * @return {?}
                 */
                val => col.isLessOrEqualTo(val)));
                addFilter('_ne', (/**
                 * @param {?} val
                 * @return {?}
                 */
                val => col.isDifferentFrom(val)));
                addFilter('_contains', (/**
                 * @param {?} val
                 * @return {?}
                 */
                val => {
                    /** @type {?} */
                    let c = (/** @type {?} */ (col));
                    if (c != null && c.isContains) {
                        return c.isContains(val);
                    }
                }));
                addFilter('_st', (/**
                 * @param {?} val
                 * @return {?}
                 */
                val => {
                    /** @type {?} */
                    let c = (/** @type {?} */ (col));
                    if (c != null && c.isContains) {
                        return c.isStartsWith(val);
                    }
                }));
            }));
        }
        return where;
    }
    /**
     * @private
     * @param {?} response
     * @param {?} id
     * @param {?} what
     * @return {?}
     */
    doOnId(response, id, what) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                /** @type {?} */
                let where = this.rowType.__idColumn.isEqualTo(id);
                if (this.options && this.options.get && this.options.get.where)
                    where = new AndFilter(where, this.options.get.where(this.rowType));
                yield this.rowType.source.find({ where })
                    .then((/**
                 * @param {?} r
                 * @return {?}
                 */
                (r) => __awaiter(this, void 0, void 0, function* () {
                    if (r.length == 0)
                        response.notFound();
                    else if (r.length > 1)
                        response.error({ message: "id is not unique" });
                    else
                        yield what(r[0]);
                })));
            }
            catch (err) {
                response.error(err);
            }
        });
    }
    /**
     * @param {?} response
     * @param {?} id
     * @param {?} body
     * @return {?}
     */
    put(response, id, body) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.options.allowUpdate) {
                response.methodNotAllowed();
                return;
            }
            yield this.doOnId(response, id, (/**
             * @param {?} row
             * @return {?}
             */
            (row) => __awaiter(this, void 0, void 0, function* () {
                row.__fromPojo(body, this.readonlyColumns);
                yield row.save(this.options.validate, this.options.onSavingRow);
                response.success(yield row.__toPojo(this.excludedColumns));
            })));
        });
    }
    /**
     * @param {?} response
     * @param {?} id
     * @return {?}
     */
    delete(response, id) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.options.allowDelete) {
                response.methodNotAllowed();
                return;
            }
            yield this.doOnId(response, id, (/**
             * @param {?} row
             * @return {?}
             */
            (row) => __awaiter(this, void 0, void 0, function* () {
                yield row.delete();
                response.deleted();
            })));
        });
    }
    /**
     * @param {?} response
     * @param {?} body
     * @return {?}
     */
    post(response, body) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.options.allowInsert) {
                response.methodNotAllowed();
                return;
            }
            try {
                /** @type {?} */
                let r = this.rowType.source.createNewItem();
                r.__fromPojo(body, this.readonlyColumns);
                yield r.save(this.options.validate, this.options.onSavingRow);
                response.created(yield r.__toPojo(this.excludedColumns));
            }
            catch (err) {
                response.error(err);
            }
        });
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LocalStorageDataProvider {
    constructor() {
    }
    /**
     * @template T
     * @param {?} name
     * @param {?} factory
     * @return {?}
     */
    provideFor(name, factory) {
        return new JsonStorageDataProvider(new LocalJsonStorage(name, factory));
    }
}
class LocalJsonStorage {
    /**
     * @param {?} key
     * @param {?} factory
     */
    constructor(key, factory) {
        this.key = key;
        this.factory = factory;
    }
    /**
     * @template T
     * @param {?} what
     * @return {?}
     */
    doWork(what) {
        /** @type {?} */
        let data = localStorage.getItem(this.key);
        try {
            data = JSON.parse(data);
        }
        catch (err) {
            data = [];
        }
        if (!(data instanceof Array))
            data = [];
        /** @type {?} */
        let dp = new ActualInMemoryDataProvider(this.factory, data);
        return what(dp, (/**
         * @return {?}
         */
        () => localStorage.setItem(this.key, JSON.stringify(data, undefined, 2))));
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ActualDirectSQL extends DirectSQL {
    /**
     * @param {?} dp
     */
    constructor(dp) {
        super();
        this.dp = dp;
        this.dp = ActualSQLServerDataProvider.decorateSqlConnectionProvider(dp);
    }
    /**
     * @param {?} sql
     * @return {?}
     */
    execute(sql) {
        /** @type {?} */
        let c = this.dp.createCommand();
        return c.query(sql);
    }
}
class myServerAction extends Action {
    /**
     * @param {?} name
     * @param {?} types
     * @param {?} options
     * @param {?} originalMethod
     */
    constructor(name, types, options, originalMethod) {
        super(Context.apiBaseUrl + '/', name);
        this.types = types;
        this.options = options;
        this.originalMethod = originalMethod;
    }
    /**
     * @protected
     * @param {?} info
     * @param {?} req
     * @return {?}
     */
    execute(info, req) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            let result = { data: {} };
            yield ((/** @type {?} */ (this.dataSource))).doInTransaction((/**
             * @param {?} ds
             * @return {?}
             */
            (ds) => __awaiter(this, void 0, void 0, function* () {
                /** @type {?} */
                let context = new ServerContext();
                context.setReq(req);
                context.setDataProvider(ds);
                if (!context.isAllowed(this.options.allowed))
                    throw 'not allowed';
                if (this.types)
                    for (let i = 0; i < this.types.length; i++) {
                        if (info.args.length < i) {
                            info.args.push(undefined);
                        }
                        if (this.types[i] == Context || this.types[i] == ServerContext) {
                            info.args[i] = context;
                        }
                        else if (this.types[i] == DirectSQL && ds) {
                            info.args[i] = ((/** @type {?} */ ((/** @type {?} */ (ds))))).getDirectSql();
                        }
                    }
                try {
                    result.data = yield this.originalMethod(info.args);
                }
                catch (err) {
                    console.error(err);
                    throw err;
                }
            })));
            return result;
        });
    }
}
/** @type {?} */
const actionInfo = {
    allActions: (/** @type {?} */ ([])),
    runningOnServer: false
};
/**
 * @param {?} options
 * @return {?}
 */
function ServerFunction(options) {
    return (/**
     * @param {?} target
     * @param {?} key
     * @param {?} descriptor
     * @return {?}
     */
    (target, key, descriptor) => {
        /** @type {?} */
        var originalMethod = descriptor.value;
        /** @type {?} */
        var types = Reflect.getMetadata("design:paramtypes", target, key);
        /** @type {?} */
        let serverAction = new myServerAction(key, types, options, (/**
         * @param {?} args
         * @return {?}
         */
        args => originalMethod.apply(undefined, args)));
        descriptor.value = (/**
         * @param {...?} args
         * @return {?}
         */
        function (...args) {
            return __awaiter(this, void 0, void 0, function* () {
                if (!actionInfo.runningOnServer) {
                    if (options.blockUser === false) {
                        return yield BusyService.singleInstance.donotWait((/**
                         * @return {?}
                         */
                        () => __awaiter(this, void 0, void 0, function* () { return (yield serverAction.run({ args })).data; })));
                    }
                    else
                        return (yield serverAction.run({ args })).data;
                }
                else
                    return (yield originalMethod.apply(undefined, args));
            });
        });
        actionInfo.allActions.push(descriptor.value);
        descriptor.value[serverActionField] = serverAction;
        return descriptor;
    });
}
/** @type {?} */
const serverActionField = Symbol('serverActionField');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class IdEntity extends Entity {
    /**
     * @param {?=} options
     */
    constructor(options) {
        super(options);
        this.id = new IdColumn();
        this.disableNewId = false;
        this.id.allowApiUpdate = false;
        /** @type {?} */
        let x = this.__onSavingRow;
        this.__onSavingRow = (/**
         * @return {?}
         */
        () => {
            if (this.isNew() && !this.id.value && !this.disableNewId)
                this.id.setToNewId();
            return x();
        });
    }
    /**
     * @return {?}
     */
    setEmptyIdForNewRow() {
        this.id.value = '';
        this.disableNewId = true;
    }
}
class IdColumn extends StringColumn {
    /**
     * @return {?}
     */
    setToNewId() {
        this.value = v4();
    }
}
/**
 * @param {?} row
 * @param {?} column
 * @param {?=} message
 * @return {?}
 */
function checkForDuplicateValue(row, column, message) {
    return __awaiter(this, void 0, void 0, function* () {
        if (row.isNew() || column.value != column.originalValue) {
            /** @type {?} */
            let rows = yield row.source.find({ where: column.isEqualTo(column.value) });
            console.log(rows.length);
            if (rows.length > 0)
                column.error = message || 'Already exists';
        }
    });
}
/**
 * @template type
 * @param {?} original
 * @param {?} addValues
 * @return {?}
 */
function DecorateDataColumnSettings(original, addValues) {
    /** @type {?} */
    let result = {};
    if (typeof (original) == "string")
        result.caption = original;
    else
        result = original;
    addValues(result);
    return result;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { BusyService, RadWebModule, myFetch, RestDataProvider, angularHttpProvider, restDataProviderHttpProviderUsingFetch, wrapFetch, Action, isNewRow, extractSortFromSettings, translateSort, testing, DataAreaSettings, GridSettings, FilterHelper, DataList, Sort, Lookup, UrlBuilder, FilterConsumnerBridgeToUrlBuilder, lookupRowInfo, DefaultStorage, DateTimeDateStorage, CharDateStorage, DateTimeStorage, Column, Filter, AndFilter, Entity, ColumnHashSet, CompoundIdColumn, EntitySource, __EntityValueProvider, StringColumn, DateColumn, DateTimeColumn, NumberColumn, BoolColumn, BoolStorage, ClosedListColumn, ColumnCollection, FilterConsumerBridgeToSqlRequest, makeTitle, isFunction$1 as isFunction, functionOrString, pageArray, InMemoryDataProvider, ActualInMemoryDataProvider, WebSqlDataProvider, ActualSQLServerDataProvider, JsonStorageDataProvider, DataApi, LocalStorageDataProvider, EntityClass, Context, ServerContext, SpecificEntityHelper, allEntities, DirectSQL, Role, JwtSessionManager, RouteHelperService, SignedInGuard, NotSignedInGuard, ServerFunction, ActualDirectSQL, myServerAction, actionInfo, serverActionField, checkForDuplicateValue, DecorateDataColumnSettings, IdEntity, IdColumn, ColumnDesigner as ɵf, DataAreaCompnent as ɵh, DataArea2Compnent as ɵd, DataControlComponent as ɵb, DataControl2Component as ɵc, DataControl3Component as ɵk, DataFilterInfoComponent as ɵg, DataGridComponent as ɵe, DataGrid2Component as ɵi, LoaderInterceptor as ɵa, WaitComponent as ɵj };

//# sourceMappingURL=radweb.js.map